Package: ciecl
Title: International Classification of Diseases 'ICD-10'/'ICD-11' for
        Chile
Version: 0.9.6
Authors@R: 
    person("Rodolfo", "Tasso Suazo",
           email = "rtasso@uchile.cl",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-1264-4933"))
Description: Tools for working with the International Classification of
    Diseases ('ICD-10' Chile official 'MINSAL'/'DEIS' v2018). Includes
    optimized 'SQL' search with 'SQLite', fuzzy matching of medical terms
    ('Jaro-Winkler'), Charlson and Elixhauser comorbidity calculation,
    'WHO' 'ICD-11' 'API' integration, and hierarchical code validation.
    Data from Centro FIC Chile 'DEIS' <https://deis.minsal.cl/centrofic/>.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Depends: R (>= 4.1.0)
Imports: DBI, RSQLite, stringdist, stringr, dplyr, tibble, tools, utils
Suggests: comorbidity, gt, httr2, readxl, usethis, withr, writexl,
        testthat (>= 3.1.5), knitr, litedown (>= 0.9), rmarkdown
VignetteBuilder: knitr
URL: https://rodotasso.github.io/ciecl/,
        https://github.com/RodoTasso/ciecl
BugReports: https://github.com/RodoTasso/ciecl/issues
Config/testthat/edition: 3
Config/Needs/website: pkgdown
Language: en, es
NeedsCompilation: no
Packaged: 2026-04-19 00:45:44 UTC; tassu
Author: Rodolfo Tasso Suazo [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1264-4933>)
Maintainer: Rodolfo Tasso Suazo <rtasso@uchile.cl>
Repository: CRAN
Date/Publication: 2026-04-19 14:30:02 UTC
Built: R 4.6.0; ; 2026-04-29 00:40:19 UTC; unix

# Generated by roxygen2: do not edit by hand

export(cie10_clear_cache)
export(cie10_disconnect)
export(cie10_sql)
export(cie11_search)
export(cie_comorbid)
export(cie_expand)
export(cie_guia_busqueda)
export(cie_lookup)
export(cie_map_comorbid)
export(cie_normalizar)
export(cie_search)
export(cie_siglas)
export(cie_table)
export(cie_validate_vector)
importFrom(DBI,dbConnect)
importFrom(DBI,dbDisconnect)
importFrom(DBI,dbExecute)
importFrom(DBI,dbExistsTable)
importFrom(DBI,dbGetQuery)
importFrom(DBI,dbIsValid)
importFrom(DBI,dbWriteTable)
importFrom(RSQLite,SQLite)
importFrom(dplyr,"%>%")
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,slice_head)
importFrom(stringdist,stringsim)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_trim)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tibble,tribble)
importFrom(utils,data)
importFrom(utils,packageVersion)

> **Language / Idioma:** **English** \|
> [Español](https://rodotasso.github.io/ciecl/articles/ciecl-es.html)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ciecl <img src="man/figures/logo-CDSP_color.png" align="right" height="60" alt="CDSP"> <img src="man/figures/logo.png" align="right" height="139" alt="ciecl logo">

**Data Science for Public Health Group** \| University of Chile

<!-- badges: start -->

[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![CRAN
status](https://www.r-pkg.org/badges/version/ciecl)](https://CRAN.R-project.org/package=ciecl)
[![GitHub
release](https://img.shields.io/github/v/release/RodoTasso/ciecl)](https://github.com/RodoTasso/ciecl/releases)
[![R-CMD-check](https://github.com/RodoTasso/ciecl/actions/workflows/r.yml/badge.svg)](https://github.com/RodoTasso/ciecl/actions/workflows/r.yml)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/ciecl)](https://cran.r-project.org/package=ciecl)
[![Coverage](https://img.shields.io/badge/coverage-96%25-brightgreen)](https://github.com/RodoTasso/ciecl/actions/workflows/test-coverage.yaml)
[![pkgdown](https://img.shields.io/badge/docs-pkgdown-blue)](https://rodotasso.github.io/ciecl/)
![Visitors](https://komarev.com/ghpvc/?username=RodoTasso&repo=ciecl&label=Visitors&color=blue)
<!-- badges: end -->

**Official Chilean ICD-10 Classification (CIE-10) for R**.

Specialized package for searching, validating, and analyzing ICD-10
codes in the Chilean context. Includes **39,877 codes** (categories and
subcategories) from the official MINSAL/DEIS v2018 catalog, with
optimized search, comorbidity calculation, and WHO ICD-11 API access.

## Purpose

`ciecl` facilitates working with ICD-10 codes in health research and
data analysis in Chile, eliminating the need to manually manipulate
Excel files and providing specialized tools for:

- Fast validation of diagnostic codes
- Error-tolerant search (fuzzy search)
- Automatic calculation of comorbidity indices (Charlson, Elixhauser)
- Optimized SQL queries over more than 39,000 codes
- Hierarchical expansion of categories to subcategories (e.g., E11 -\>
  E11.0, E11.1, …, E11.9)

## Why use ciecl

### Advantages:

1.  **Performance**: SQLite indexed database with 10-100x faster
    searches than Excel
2.  **Native integration**: Works directly in R without heavy external
    dependencies
3.  **Fuzzy search**: Finds “bacterial pneumonia” even if misspelled
    (tolerates typos)
4.  **Vectorized validation**: Processes thousands of codes in
    milliseconds
5.  **Automatic normalization**: Accepts E110, E11.0, e11.0
    interchangeably
6.  **No encoding errors**: XLSX files have issues with accents on
    different systems
7.  **Reproducibility**: Version-controlled ICD-10 catalog (doesn’t
    change between computers)
8.  **Predefined comorbidities**: Ready-to-use Charlson/Elixhauser
    mappings
9.  **ICD-11 API**: Direct access to updated international
    classification
10. **Smart cache**: Stores frequent queries for faster speed

### Comparative example:

``` r
# With XLSX (slow, manual, error-prone)
library(readxl)
cie10 <- read_excel("CIE-10.xlsx")
diabetes_codes <- cie10[grepl("diabetes", tolower(cie10$description)), ]

# With ciecl (fast, robust, cached)
library(ciecl)
diabetes_codes <- cie_search("diabetes")
```

## Comparison with similar packages

| Feature | ciecl | icd (CRAN) | comorbidity (CRAN) | touch (CRAN) |
|----|----|----|----|----|
| Official Chilean ICD-10 MINSAL/DEIS | **Yes** | No | No | No |
| Fuzzy search (Jaro-Winkler) | **Yes** | No | No | No |
| Chilean medical abbreviations (IAM, ACV, EPOC) | **Yes** | No | No | No |
| Charlson/Elixhauser comorbidities | Yes | Yes | **Yes** | No |
| WHO ICD-11 API | **Yes** | No | No | No |
| SQLite cache with FTS5 | **Yes** | No | No | No |
| Local Chilean adaptation | **Yes** | USA/generic only | No | USA only (ICD-10-CM) |

The ICD-10 codes included in `ciecl` are established by Decree 356/2017
of Chile’s Ministry of Health as the official disease classification
system. The dataset is not modifiable by the package — it can only be
updated by institutional decree from MINSAL through DEIS.

## Main features

- **39,877 ICD-10 codes**: Complete official MINSAL/DEIS v2018 catalog
- **Fuzzy search**: Jaro-Winkler algorithm to tolerate spelling errors
- **Direct SQL**: Full database access for complex queries
- **Vectorization**: Processes thousands of codes simultaneously
- **SQLite cache**: Stores frequent results for instant queries
- **Comorbidities**: Validated Charlson and Elixhauser mappings
- **Hierarchical expansion**: Gets all subcodes from a category
- **ICD-11 API**: Search in updated international classification (WHO)
- **Minimal dependencies**: Only 8 required packages for core
  functionality

## Installation

### All platforms (Windows, macOS, Linux)

``` r
# From CRAN
install.packages("ciecl")

# From GitHub
# install.packages("pak")
pak::pak("RodoTasso/ciecl")

# Alternative with devtools
devtools::install_github("RodoTasso/ciecl")

# Full installation (includes optional packages)
pak::pak("RodoTasso/ciecl", dependencies = TRUE)
```

### System requirements by platform

#### Windows

No additional system dependencies required. Installation works directly.

#### macOS

Make sure you have Xcode Command Line Tools installed:

``` bash
xcode-select --install
```

#### Linux (Ubuntu/Debian)

``` bash
# Dependencies for compiling R packages
sudo apt-get update
sudo apt-get install -y \
  r-base-dev \
  libcurl4-openssl-dev \
  libssl-dev \
  libxml2-dev
```

**Note**: The package has **minimal dependencies** for core
functionality. Additional packages are only required for specific
functions.

## Quick start

``` r
library(ciecl)

# Exact search (supports multiple formats)
cie_lookup("E11.0")   # With dot
cie_lookup("E110")    # Without dot
cie_lookup("E11")     # Category only

# Vectorized - multiple codes
cie_lookup(c("E11.0", "I10", "Z00"))

# With complete formatted description
cie_lookup("E110", descripcion_completa = TRUE)

# Extract code from text with prefixes/suffixes (scalar code only)
cie_lookup("CIE:E11.0", extract = TRUE)       # Returns E11.0
cie_lookup("E11.0-confirmed", extract = TRUE) # Returns E11.0
cie_lookup("dx-G20", extract = TRUE)          # Returns G20
# Note: extract=TRUE only works with scalar codes, use FALSE for vectors

# Fuzzy search with typos
cie_search("diabetis mellitus")  # Finds "diabetes mellitus"

# Search by medical abbreviations (88 supported abbreviations)
cie_search("IAM")   # Acute Myocardial Infarction
cie_search("TBC")   # Tuberculosis
cie_search("DM2")   # Type 2 Diabetes Mellitus
cie_search("EPOC")  # Chronic Obstructive Pulmonary Disease
cie_search("HTA")   # Arterial Hypertension

# See all available abbreviations
cie_siglas()

# Direct SQL
cie10_sql("SELECT * FROM cie10 WHERE codigo LIKE 'E11%' LIMIT 3")

# Comorbidities (requires: install.packages("comorbidity"))
df %>% cie_comorbid(id = "patient", code = "diagnosis", map = "charlson")
```

## ICD-11 Configuration (optional)

To use `cie11_search()` and access the updated international
classification, you need free WHO credentials:

### Step 1: Get credentials

1.  Visit <https://icd.who.int/icdapi>
2.  Register with your email (free process)
3.  You will receive a `Client ID` and `Client Secret`

### Step 2: Configure in R

**Option A: Environment variable (recommended)**

Create a `.Renviron` file in your working directory or home:

``` r
# In .Renviron
ICD_API_KEY=your_client_id:your_client_secret
```

**Option B: In each session**

``` r
Sys.setenv(ICD_API_KEY = "your_client_id:your_client_secret")
```

### Step 3: Use ICD-11

``` r
library(ciecl)

# Search in ICD-11 (requires httr2)
cie11_search("diabetes mellitus", max_results = 5)
#> # A tibble: 5 x 3
#>   code    title                                    chapter
#>   <chr>   <chr>                                    <chr>
#> 1 5A14    Diabetes mellitus, unspecified type      05
#> 2 5A11    Type 2 diabetes mellitus                 05
#> 3 5A10    Type 1 diabetes mellitus                 05
```

**Note**: ICD-11 is optional. All core functions (ICD-10) work without
an API key.

## Data

Based on official **ICD-10 MINSAL/DEIS v2018** catalog:

- **Source**: [Department of Health Statistics and Information
  (DEIS)](https://deis.minsal.cl)
- **FIC Chile Center**: <https://deis.minsal.cl/centrofic/>
- **Documents repository**: <https://deis.minsal.cl>

ICD-10 data is for public use according to [Decree 356 Exempt
(2017)](https://www.bcn.cl/leychile/navegar?i=1112064) from the Ministry
of Health establishing official ICD-10 use in Chile.

## Contributing

Contributions are welcome:

- Report bugs at [GitHub
  Issues](https://github.com/RodoTasso/ciecl/issues)
- Suggest improvements or new features
- Submit pull requests

## License

MIT (package code) + ICD-10 MINSAL public use data

## Author

**Rodolfo Tasso Suazo** \| <rtasso@uchile.cl>

### Institutional Affiliation

<p align="center">

<img src="man/figures/logo-CDSP_color.png" height="100"
  alt="Data Science for Public Health Group">
</p>

**Data Science for Public Health Group**<br> School of Public Health,
Faculty of Medicine<br> University of Chile

This package was developed as part of the work of the Data Science for
Public Health Group, dedicated to applying computational and statistical
methods to improve public health research in Chile.

## Links

- **Repository**: <https://github.com/RodoTasso/ciecl>
- **Report issues**: <https://github.com/RodoTasso/ciecl/issues>
- **DEIS MINSAL**: <https://deis.minsal.cl>
- **FIC Chile Center**: <https://deis.minsal.cl/centrofic/>
- **WHO ICD-11 API**: <https://icd.who.int/icdapi>

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## -----------------------------------------------------------------------------
# library(ciecl)
# 
# # Ver estructura del dataset
# head(cie10_cl)
# #> # A tibble: 6 x 8
# #>   codigo descripcion                    categoria inclusion exclusion capitulo
# #>   <chr>  <chr>                          <chr>     <chr>     <chr>     <chr>
# #> 1 A00    Colera                         A00       ...       ...       01
# #> 2 A00.0  Colera debido a Vibrio...      A00       ...       ...       01

## -----------------------------------------------------------------------------
# # Todas estas busquedas encuentran "neumonia"
# cie_search("neumonia")
# cie_search("neumonía")
# cie_search("NEUMONIA")
# 
# # Funciona con n
# cie_search("rinon")     # Encuentra "rinon"
# cie_search("espanol")   # Encuentra terminos con "n"

## -----------------------------------------------------------------------------
# # Ver todas las siglas disponibles
# cie_siglas()
# 
# # Filtrar por categoria
# cie_siglas("cardiovascular")
# cie_siglas("respiratoria")
# cie_siglas("oncologica")
# 
# # Buscar usando siglas
# cie_search("IAM")   # Infarto Agudo del Miocardio
# cie_search("EPOC")  # Enfermedad Pulmonar Obstructiva Cronica
# cie_search("DM2")   # Diabetes Mellitus tipo 2
# cie_search("HTA")   # Hipertension Arterial
# cie_search("TBC")   # Tuberculosis

## -----------------------------------------------------------------------------
# # Busqueda en espanol (default)
# cie11_search("diabetes mellitus", lang = "es")
# 
# # Busqueda en ingles
# cie11_search("diabetes mellitus", lang = "en")
# 
# # Comparar resultados
# es <- cie11_search("infarto", lang = "es")
# en <- cie11_search("infarction", lang = "en")

## -----------------------------------------------------------------------------
# # Configurar idioma por defecto en sesion
# options(ciecl.lang = "es")
# 
# # Ahora todas las busquedas CIE-11 usaran espanol
# cie11_search("neumonia")

## -----------------------------------------------------------------------------
# # Verificar encoding del dataset
# Encoding(cie10_cl$descripcion[1])
# #> [1] "UTF-8"

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(ciecl)
library(dplyr)

## ----datos--------------------------------------------------------------------
set.seed(42)

egresos <- data.frame(
  ID_EGRESO = 1:200,
  ANO       = sample(2018:2022, 200, replace = TRUE),
  DIAG1     = sample(
    c(
      # Codigos frecuentes en egresos DEIS (sin punto, formato original)
      "J189", "O800", "Z380", "K359", "N390",
      "I10X", "J449", "E119", "O829", "J069",
      "K922", "N185", "I509", "C509", "A099",
      # Algunos con sufijo X (convencion DEIS)
      "N40X", "K800", "I259", "J180", "E149"
    ),
    size    = 200,
    replace = TRUE,
    prob    = c(
      0.12, 0.10, 0.09, 0.07, 0.06,
      0.08, 0.06, 0.07, 0.05, 0.04,
      0.04, 0.03, 0.05, 0.03, 0.03,
      0.03, 0.03, 0.04, 0.03, 0.05
    )
  ),
  stringsAsFactors = FALSE
)

cat("Registros totales:", nrow(egresos), "\n")
cat("Codigos unicos:", length(unique(egresos$DIAG1)), "\n")
cat("Periodo:", min(egresos$ANO), "-", max(egresos$ANO), "\n\n")
head(egresos)

## ----lookup-------------------------------------------------------------------
# Filtrar registros validos (sin NA en diagnostico)
datos_filtrados <- egresos %>%
  filter(!is.na(DIAG1))

# Codigos unicos a procesar
codigos_unicos <- unique(datos_filtrados$DIAG1)
cat("Codigos unicos a procesar:", length(codigos_unicos), "\n")

# Busqueda vectorizada con cie_lookup
inicio <- Sys.time()

resultado_ciecl <- cie_lookup(
  codigo             = codigos_unicos,
  normalizar         = TRUE,
  descripcion_completa = TRUE
)

tiempo <- round(as.numeric(difftime(Sys.time(), inicio, units = "secs")), 2)

cat("Codigos encontrados:", nrow(resultado_ciecl), "de", length(codigos_unicos), "\n")
cat("Tiempo de busqueda:", tiempo, "segundos\n")
cat("Tasa de exito:", round(nrow(resultado_ciecl) / length(codigos_unicos) * 100, 1), "%\n\n")

# Muestra de resultados
head(resultado_ciecl[, c("codigo", "descripcion")], 5)

## ----unir---------------------------------------------------------------------
# Tabla de lookup: codigo sin punto -> descripcion completa
# cie_lookup() devuelve con punto (J18.9), la BBDD tiene sin punto (J189)
tabla_lookup <- resultado_ciecl %>%
  mutate(
    codigo_sin_punto = gsub("\\.", "", codigo),
    DIAG_COMPLETO    = paste0(codigo, " - ", descripcion)
  ) %>%
  select(codigo_sin_punto, DIAG_COMPLETO)

# Funcion para normalizar el codigo de la BBDD antes del join:
# 1. Quitar puntos si los hubiera
# 2. Quitar sufijo X final (I10X -> I10, N40X -> N40)
normalizar_codigo_bbdd <- function(codigo) {
  codigo <- gsub("\\.", "", codigo)
  codigo <- gsub("X$", "", codigo)
  return(codigo)
}

# Unir con los datos originales
datos_con_desc <- datos_filtrados %>%
  mutate(codigo_busqueda = normalizar_codigo_bbdd(DIAG1)) %>%
  left_join(tabla_lookup, by = c("codigo_busqueda" = "codigo_sin_punto")) %>%
  mutate(
    DIAG_COMPLETO = if_else(is.na(DIAG_COMPLETO), DIAG1, DIAG_COMPLETO)
  ) %>%
  select(-codigo_busqueda)

# Verificar resultado
n_con_desc <- sum(grepl(" - ", datos_con_desc$DIAG_COMPLETO), na.rm = TRUE)
pct_exito  <- round(n_con_desc / nrow(datos_con_desc) * 100, 1)

cat("Registros con descripcion CIE-10:", n_con_desc, "de", nrow(datos_con_desc), "\n")
cat("Tasa de exito:", pct_exito, "%\n\n")

# Ejemplos de diagnosticos procesados
datos_con_desc %>%
  select(DIAG1, DIAG_COMPLETO) %>%
  distinct() %>%
  head(10)

## ----perfil-------------------------------------------------------------------
perfil_diag <- datos_con_desc %>%
  filter(grepl(" - ", DIAG_COMPLETO)) %>%
  count(DIAG_COMPLETO, sort = TRUE) %>%
  mutate(
    pct       = round(n / sum(n) * 100, 1),
    pct_acum  = cumsum(pct)
  )

cat("Top 10 diagnosticos:\n\n")
head(perfil_diag, 10)

## ----cobertura----------------------------------------------------------------
top10_pct <- sum(head(perfil_diag, 10)$pct)
cat("Los 10 diagnosticos mas frecuentes representan el", top10_pct, "% del total de egresos\n")

## ----busqueda_fuzzy-----------------------------------------------------------
# Terminos clinicos comunes en egresos (con y sin typos)
terminos <- c("neumonia", "diabetis", "hipertension")

for (termino in terminos) {
  cat("Buscando:", termino, "\n")
  res <- cie_search(termino, threshold = 0.70, max_results = 3)
  if (nrow(res) > 0) {
    print(res[, c("codigo", "descripcion")])
  }
  cat("\n")
}

## ----validacion---------------------------------------------------------------
validos <- cie_validate_vector(codigos_unicos)

cat("Codigos validos:  ", sum(validos), "\n")
cat("Codigos invalidos:", sum(!validos), "\n")

if (any(!validos)) {
  cat("\nCodigos no reconocidos:\n")
  print(codigos_unicos[!validos])
}

#' Buscar codigos CIE-11 via API OMS
#'
#' @param texto String termino busqueda espanol/ingles
#' @param api_key String opcional, Client ID + Secret OMS separados ":"
#'   Obtener en: https://icd.who.int/icdapi
#' @param lang Character, idioma respuesta ("es" o "en")
#' @param max_results Integer, maximo resultados (default 10)
#' @param release Character, version de release CIE-11 a consultar
#'   (default "2024-01"). Ver releases disponibles en la API OMS.
#' @return tibble con codigos CIE-11 + titulos o vacio si error
#' @family api
#' @seealso \code{\link{cie_search}}, \code{\link{cie_lookup}}
#' @export
#' @importFrom tibble as_tibble
#' @importFrom dplyr slice_head select matches
#' @examples
#' # Ver parametros disponibles
#' args(cie11_search)
#'
#' \donttest{
#' # Requiere credenciales OMS gratuitas (https://icd.who.int/icdapi)
#' Sys.setenv(ICD_API_KEY = "client_id:client_secret")
#' cie11_search("depresion mayor")
#' }
cie11_search <- function(texto, api_key = NULL, lang = "es",
                         max_results = 10, release = "2024-01") {
  # Validacion de inputs
  if (!is.character(texto) || length(texto) != 1 || is.na(texto)) {
    stop("'texto' debe ser un string de largo 1")
  }
  if (nchar(trimws(texto)) == 0) {
    stop("'texto' no puede estar vacio")
  }
  if (!is.character(lang) || !lang %in% c("es", "en")) {
    stop("'lang' debe ser \"es\" o \"en\"")
  }
  if (!is.numeric(max_results) || length(max_results) != 1 ||
      max_results < 1 || max_results != as.integer(max_results)) {
    stop("'max_results' debe ser un entero positivo")
  }
  if (!is.character(release) || length(release) != 1 ||
      !grepl("^\\d{4}-\\d{2}$", release)) {
    stop("'release' debe ser formato 'YYYY-MM' (ej. '2024-01')")
  }

  # Verificar que httr2 este instalado
  if (!requireNamespace("httr2", quietly = TRUE)) {
    stop("El paquete 'httr2' es necesario para esta funci\u00f3n.\n",
         "Inst\u00e1lalo con: install.packages('httr2')")
  }
  
  # Obtener API key (env var o argumento)
  if (is.null(api_key)) {
    api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
    if (is.na(api_key)) {
      stop("API key OMS requerida. Ver: https://icd.who.int/icdapi")
    }
  }
  
  # Separar client_id y client_secret
  credentials <- strsplit(api_key, ":")[[1]]
  if (length(credentials) != 2) {
    stop("API key debe tener formato 'client_id:client_secret'")
  }
  client_id <- credentials[1]
  client_secret <- credentials[2]
  
  tryCatch({
    # Paso 1: Obtener token OAuth
    token_url <- "https://icdaccessmanagement.who.int/connect/token"
    token_req <- httr2::request(token_url) %>%
      httr2::req_method("POST") %>%
      httr2::req_body_form(
        client_id = client_id,
        client_secret = client_secret,
        scope = "icdapi_access",
        grant_type = "client_credentials"
      )
    
    token_resp <- httr2::req_perform(token_req)
    token_data <- httr2::resp_body_json(token_resp)
    access_token <- token_data$access_token
    
    # Paso 2: Buscar en CIE-11 con el token
    search_url <- paste0(
      "https://id.who.int/icd/release/11/", release, "/mms/search"
    )
    
    search_req <- httr2::request(search_url) %>%
      httr2::req_url_query(
        q = texto,
        flatResults = "true",
        useFlexisearch = "true"
      ) %>%
      httr2::req_headers(
        Authorization = paste("Bearer", access_token),
        `API-Version` = "v2",
        `Accept-Language` = lang
      )
    
    search_resp <- httr2::req_perform(search_req)
    json <- httr2::resp_body_json(search_resp, simplifyVector = TRUE)
    
    # Parsear resultados
    has_results <- "destinationEntities" %in% names(json) &&
      length(json$destinationEntities) > 0
    if (has_results) {
      # Limpiar HTML tags del titulo
      html_pattern <- "<em class='found'>|</em>"
      titulos_limpios <- gsub(html_pattern, "", json$destinationEntities$title)
      
      resultados <- tibble::tibble(
        codigo = json$destinationEntities$theCode,
        titulo = titulos_limpios,
        capitulo = json$destinationEntities$chapter
      ) %>%
        dplyr::slice_head(n = max_results)
      
      return(resultados)
    } else {
      message("Sin resultados CIE-11 para: ", texto)
      return(tibble::tibble(
        codigo = character(), 
        titulo = character(),
        capitulo = character()
      ))
    }
    
  }, error = function(e) {
    warning("Error API CIE-11: ", e$message, "\nRetornando resultado vacio. ",
            "Usa cie_search() para fallback local CIE-10")
    return(tibble::tibble(
      codigo = character(), 
      titulo = character(),
      capitulo = character()
    ))
  })
}

# Tests para cie11_search (API CIE-11 OMS)

test_that("cie11_search requiere API key", {
  # Limpiar variable de entorno temporalmente
  old_key <- Sys.getenv("ICD_API_KEY")
  Sys.unsetenv("ICD_API_KEY")
  on.exit(Sys.setenv(ICD_API_KEY = old_key))

  expect_error(cie11_search("diabetes"), "API key OMS requerida")
})

test_that("cie11_search valida formato de API key", {
  skip_if_not_installed("httr2")

  # API key sin separador ":"
 expect_error(
    cie11_search("diabetes", api_key = "invalid_key_format"),
    "client_id:client_secret"
  )
})

test_that("cie11_search requiere httr2", {
  skip_if(requireNamespace("httr2", quietly = TRUE))

  expect_error(
    cie11_search("diabetes", api_key = "test:test"),
    "httr2"
  )
})

# ============================================================
# PRUEBAS CON API REAL (skip_on_cran, requiere ICD_API_KEY)
# ============================================================

test_that("cie11_search con API real retorna resultados", {
  skip_on_cran()
  skip_if_not_installed("httr2")
  skip_if_offline()

  # Verificar que existe API key en environment
  api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  skip_if(is.na(api_key) || api_key == "",
          "ICD_API_KEY no configurada para tests reales")

  # Buscar termino comun que debe existir
  resultado <- cie11_search("diabetes", max_results = 2)

  expect_s3_class(resultado, "tbl_df")
  expect_true("codigo" %in% names(resultado))
  expect_true("titulo" %in% names(resultado))
  expect_true("capitulo" %in% names(resultado))
  expect_lte(nrow(resultado), 2)
  if (nrow(resultado) > 0) {
    expect_false(any(grepl("<em", resultado$titulo, fixed = TRUE)))
    expect_false(any(grepl("</em>", resultado$titulo, fixed = TRUE)))
  }
})


test_that("cie11_search con API real respeta max_results", {
  skip_on_cran()
  skip_if_not_installed("httr2")
  skip_if_offline()

  api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  skip_if(is.na(api_key) || api_key == "",
          "ICD_API_KEY no configurada para tests reales")

  resultado_5 <- cie11_search("cancer", max_results = 5)
  resultado_10 <- cie11_search("cancer", max_results = 10)

  expect_lte(nrow(resultado_5), 5)
  expect_lte(nrow(resultado_10), 10)
})


test_that("cie11_search con API real soporta idioma espanol", {
  skip_on_cran()
  skip_if_not_installed("httr2")
  skip_if_offline()

  api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  skip_if(is.na(api_key) || api_key == "",
          "ICD_API_KEY no configurada para tests reales")

  resultado <- cie11_search("hipertension", lang = "es")

  expect_s3_class(resultado, "tbl_df")
  # Si hay resultados, deberian tener titulos en espanol
  if (nrow(resultado) > 0) {
    # Titulos deberian ser texto en espanol
    expect_type(resultado$titulo, "character")
  }
})


test_that("cie11_search con API real soporta idioma ingles", {
  skip_on_cran()
  skip_if_not_installed("httr2")
  skip_if_offline()

  api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  skip_if(is.na(api_key) || api_key == "",
          "ICD_API_KEY no configurada para tests reales")

  resultado <- cie11_search("hypertension", lang = "en")

  expect_s3_class(resultado, "tbl_df")
})


test_that("cie11_search con API real maneja busqueda sin resultados", {
  skip_on_cran()
  skip_if_not_installed("httr2")
  skip_if_offline()

  api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  skip_if(is.na(api_key) || api_key == "",
          "ICD_API_KEY no configurada para tests reales")

  # Buscar termino que probablemente no existe
  expect_message(
    resultado <- cie11_search("xyznonexistent12345"),
    "Sin resultados"
  )

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})


test_that("cie11_search con API real limpia tags HTML", {
  skip_on_cran()
  skip_if_not_installed("httr2")
  skip_if_offline()

  api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  skip_if(is.na(api_key) || api_key == "",
          "ICD_API_KEY no configurada para tests reales")

  resultado <- cie11_search("diabetes")

  if (nrow(resultado) > 0) {
    # No deberia haber tags <em> en los titulos
    expect_false(any(grepl("<em", resultado$titulo, fixed = TRUE)))
    expect_false(any(grepl("</em>", resultado$titulo, fixed = TRUE)))
  }
})


# ============================================================
# PRUEBAS DE ESTRUCTURA DE RESPUESTA DETALLADA
# ============================================================

test_that("cie11_search retorna tibble con tipos correctos incluso en error", {
  skip_if_not_installed("httr2")

  suppressWarnings({
    resultado <- cie11_search("test", api_key = "fake:key")
  })

  # Columnas
  expect_true("codigo" %in% names(resultado))
  expect_true("titulo" %in% names(resultado))
  expect_true("capitulo" %in% names(resultado))

  # Tipos (aunque vacio)
  expect_type(resultado$codigo, "character")
  expect_type(resultado$titulo, "character")
  expect_type(resultado$capitulo, "character")
})

test_that("cie11_search maneja texto de busqueda con caracteres especiales", {
  skip_if_not_installed("httr2")

  # No debe crashear con caracteres especiales
  expect_warning(
    resultado <- cie11_search("diabetes & mellitus", api_key = "test:test"),
    "Error API"
  )
  expect_s3_class(resultado, "tbl_df")

  expect_warning(
    resultado2 <- cie11_search("c\u00e1ncer", api_key = "test:test"),
    "Error API"
  )
  expect_s3_class(resultado2, "tbl_df")
})

test_that("cie11_search rechaza texto vacio", {
  skip_if_not_installed("httr2")

  # Texto vacio debe dar error con validacion de input
  expect_error(
    cie11_search("", api_key = "test:test"),
    "vacio"
  )
})

test_that("cie11_search valida tipos de input", {
  skip_if_not_installed("httr2")

  expect_error(cie11_search(123), "string")
  expect_error(cie11_search(c("a", "b")), "string")
  expect_error(cie11_search(NA_character_), "string")
  expect_error(
    cie11_search("test", lang = "fr", api_key = "a:b"), "lang"
  )
  expect_error(
    cie11_search("test", max_results = -1, api_key = "a:b"),
    "entero positivo"
  )
  expect_error(
    cie11_search("test", release = "invalid", api_key = "a:b"),
    "YYYY-MM"
  )
  expect_error(
    cie11_search("test", release = 2024, api_key = "a:b"),
    "YYYY-MM"
  )
})

# ============================================================
# PRUEBAS ADICIONALES PARA COBERTURA
# ============================================================

test_that("cie11_search con API real busca hipertension", {
  skip_on_cran()
  skip_if_not_installed("httr2")
  skip_if_offline()

  api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  skip_if(is.na(api_key) || api_key == "",
          "ICD_API_KEY no configurada para tests reales")

  resultado <- cie11_search("hipertension arterial", max_results = 3)

  expect_s3_class(resultado, "tbl_df")
  expect_lte(nrow(resultado), 3)
  if (nrow(resultado) > 0) {
    expect_true(all(c("codigo", "titulo", "capitulo") %in% names(resultado)))
  }
})


test_that("cie11_search con API real busca cancer", {
  skip_on_cran()
  skip_if_not_installed("httr2")
  skip_if_offline()

  api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  skip_if(is.na(api_key) || api_key == "",
          "ICD_API_KEY no configurada para tests reales")

  resultado <- cie11_search("neoplasia maligna", lang = "es", max_results = 5)

  expect_s3_class(resultado, "tbl_df")
  expect_type(resultado$codigo, "character")
  expect_type(resultado$capitulo, "character")
})


test_that("cie11_search con API real verifica columna capitulo", {
  skip_on_cran()
  skip_if_not_installed("httr2")
  skip_if_offline()

  api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  skip_if(is.na(api_key) || api_key == "",
          "ICD_API_KEY no configurada para tests reales")

  resultado <- cie11_search("neumonia", max_results = 5)

  expect_true("capitulo" %in% names(resultado))
  if (nrow(resultado) > 0) {
    # Capitulos deben ser strings no vacios o NA
    expect_type(resultado$capitulo, "character")
  }
})


test_that("cie11_search con API real acepta max_results=1", {
  skip_on_cran()
  skip_if_not_installed("httr2")
  skip_if_offline()

  api_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  skip_if(is.na(api_key) || api_key == "",
          "ICD_API_KEY no configurada para tests reales")

  resultado <- cie11_search("asma", max_results = 1)

  expect_s3_class(resultado, "tbl_df")
  expect_lte(nrow(resultado), 1)
})

# test-api-mock.R
# Pruebas de API CIE-11 con mocks y validacion de parametros

# ============================================================
# PRUEBAS DE VALIDACION DE PARAMETROS cie11_search()
# ============================================================

# ============================================================
# PRUEBAS DE MANEJO DE ERRORES
# ============================================================

test_that("cie11_search maneja error de conexion gracefully", {
  skip_if_not_installed("httr2")

  # Con credenciales invalidas, debe dar warning y retornar tibble vacio
  # (no debe crashear)
  expect_warning(
    resultado <- cie11_search("diabetes", api_key = "invalid:credentials"),
    "Error API CIE-11"
  )

  # Debe retornar tibble vacio
  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

test_that("cie11_search retorna tibble vacio en error", {
  skip_if_not_installed("httr2")

  # Cualquier error debe retornar estructura correcta
  suppressWarnings({
    resultado <- cie11_search("test", api_key = "bad:key")
  })

  expect_s3_class(resultado, "tbl_df")
  expect_true("codigo" %in% names(resultado))
  expect_true("titulo" %in% names(resultado))
  expect_true("capitulo" %in% names(resultado))
  expect_equal(nrow(resultado), 0)
})

# ============================================================
# PRUEBAS DE FORMATO DE API KEY
# ============================================================

test_that("cie11_search rechaza API key sin separador", {
  skip_if_not_installed("httr2")

  # Sin ":"
  expect_error(
    cie11_search("diabetes", api_key = "singletoken"),
    "client_id:client_secret"
  )

  # Vacio
  expect_error(
    cie11_search("diabetes", api_key = ""),
    "client_id:client_secret"
  )
})

test_that("cie11_search acepta API key con formato correcto",
{
  skip_if_not_installed("httr2")

  # Formato correcto (fallara en autenticacion, no en formato)
  # El error no debe mencionar "client_id:client_secret"
  error_msg <- tryCatch({
    suppressWarnings(cie11_search("diabetes", api_key = "id:secret"))
    "no_error"
  }, error = function(e) {
    e$message
  }, warning = function(w) {
    "warning"
  })

  # Si hay error, no debe ser de formato
  if (error_msg != "no_error" && error_msg != "warning") {
    expect_false(grepl("client_id:client_secret", error_msg))
  }
})

# ============================================================
# PRUEBAS DE VARIABLE DE ENTORNO
# ============================================================

test_that("cie11_search usa ICD_API_KEY de environment", {
  skip_if_not_installed("httr2")

  # Guardar valor original
  old_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  on.exit({
    if (is.na(old_key)) {
      Sys.unsetenv("ICD_API_KEY")
    } else {
      Sys.setenv(ICD_API_KEY = old_key)
    }
  })

  # Setear key de prueba
  Sys.setenv(ICD_API_KEY = "env:key")

  # Debe usar la key del environment (y fallar en autenticacion)
  expect_warning(
    resultado <- cie11_search("diabetes"),
    "Error API CIE-11"
  )

  expect_s3_class(resultado, "tbl_df")
})

test_that("cie11_search prefiere argumento sobre environment", {
  skip_if_not_installed("httr2")

  # Guardar valor original
  old_key <- Sys.getenv("ICD_API_KEY", unset = NA)
  on.exit({
    if (is.na(old_key)) {
      Sys.unsetenv("ICD_API_KEY")
    } else {
      Sys.setenv(ICD_API_KEY = old_key)
    }
  })

  # Setear key en environment
  Sys.setenv(ICD_API_KEY = "env:key")

  # Usar key de argumento (diferente)
  expect_warning(
    resultado <- cie11_search("diabetes", api_key = "arg:key"),
    "Error API CIE-11"
  )

  # La funcion debe usar arg:key, no env:key
  # (ambas fallaran, pero verificamos que acepta el argumento)
  expect_s3_class(resultado, "tbl_df")
})

# ============================================================
# PRUEBAS DE DEPENDENCIA HTTR2
# ============================================================

test_that("cie11_search informa sobre httr2 faltante", {
  # Este test solo funciona si httr2 NO esta instalado
  # Lo saltamos si httr2 esta disponible
  skip_if(requireNamespace("httr2", quietly = TRUE),
          "httr2 esta instalado")

  expect_error(
    cie11_search("diabetes", api_key = "test:test"),
    "httr2"
  )
})

# ============================================================
# PRUEBAS CON HTTP MOCKING (local_mocked_bindings)
# Cubren lineas 41-95 de cie-api.R: OAuth token + search + parsing
# ============================================================

test_that("cie11_search retorna resultados con mock HTTP exitoso", {
  skip_if_not_installed("httr2")

  call_count <- 0L

  local_mocked_bindings(
    req_perform = function(req, ...) {
      call_count <<- call_count + 1L
      structure(list(call_id = call_count), class = "httr2_response")
    },
    resp_body_json = function(resp, ...) {
      if (resp$call_id == 1L) {
        # Token response
        list(access_token = "mock_token_abc123")
      } else {
        # Search response con resultados
        list(destinationEntities = data.frame(
          theCode = c("5A00", "5A01", "5A02"),
          title = c(
            "<em class='found'>Diabetes</em> mellitus tipo 1",
            "<em class='found'>Diabetes</em> mellitus tipo 2",
            "Otra <em class='found'>diabetes</em> mellitus"
          ),
          chapter = c("05", "05", "05"),
          stringsAsFactors = FALSE
        ))
      }
    },
    .package = "httr2"
  )

  resultado <- cie11_search("diabetes", api_key = "test_id:test_secret")

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 3)
  expect_equal(resultado$codigo, c("5A00", "5A01", "5A02"))
  # Verificar que HTML tags fueron limpiados

  expect_false(grepl("<em", resultado$titulo[1]))
  expect_true(grepl("Diabetes mellitus tipo 1", resultado$titulo[1]))
  expect_equal(resultado$capitulo, c("05", "05", "05"))
})

test_that("cie11_search respeta max_results con mock", {
  skip_if_not_installed("httr2")

  call_count <- 0L

  local_mocked_bindings(
    req_perform = function(req, ...) {
      call_count <<- call_count + 1L
      structure(list(call_id = call_count), class = "httr2_response")
    },
    resp_body_json = function(resp, ...) {
      if (resp$call_id == 1L) {
        list(access_token = "mock_token")
      } else {
        list(destinationEntities = data.frame(
          theCode = c("5A00", "5A01", "5A02", "5A03", "5A04"),
          title = paste("Resultado", 1:5),
          chapter = rep("05", 5),
          stringsAsFactors = FALSE
        ))
      }
    },
    .package = "httr2"
  )

  resultado <- cie11_search("diabetes", api_key = "id:secret", max_results = 2)

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 2)
})

test_that("cie11_search retorna tibble vacio sin destinationEntities", {
  skip_if_not_installed("httr2")

  call_count <- 0L

  local_mocked_bindings(
    req_perform = function(req, ...) {
      call_count <<- call_count + 1L
      structure(list(call_id = call_count), class = "httr2_response")
    },
    resp_body_json = function(resp, ...) {
      if (resp$call_id == 1L) {
        list(access_token = "mock_token")
      } else {
        # Sin destinationEntities
        list(error = FALSE, errorMessage = "")
      }
    },
    .package = "httr2"
  )

  expect_message(
    resultado <- cie11_search("xyznonexistent", api_key = "id:secret"),
    "Sin resultados"
  )

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
  expect_true(all(c("codigo", "titulo", "capitulo") %in% names(resultado)))
})

test_that("cie11_search limpia HTML tags correctamente con mock", {
  skip_if_not_installed("httr2")

  call_count <- 0L

  local_mocked_bindings(
    req_perform = function(req, ...) {
      call_count <<- call_count + 1L
      structure(list(call_id = call_count), class = "httr2_response")
    },
    resp_body_json = function(resp, ...) {
      if (resp$call_id == 1L) {
        list(access_token = "mock_token")
      } else {
        list(destinationEntities = data.frame(
          theCode = "BA00",
          title = paste0(
            "<em class='found'>Hipertensi\u00f3n</em> ",
            "<em class='found'>arterial</em> esencial"),
          chapter = "11",
          stringsAsFactors = FALSE
        ))
      }
    },
    .package = "httr2"
  )

  resultado <- cie11_search("hipertension", api_key = "id:secret")

  expect_equal(nrow(resultado), 1)
  expect_equal(resultado$titulo[1], "Hipertensi\u00f3n arterial esencial")
  expect_false(grepl("<em", resultado$titulo[1]))
})

test_that("cie11_search maneja JSON inesperado sin entities", {
  skip_if_not_installed("httr2")

  call_count <- 0L

  local_mocked_bindings(
    req_perform = function(req, ...) {
      call_count <<- call_count + 1L
      structure(list(call_id = call_count), class = "httr2_response")
    },
    resp_body_json = function(resp, ...) {
      if (resp$call_id == 1L) {
        list(access_token = "mock_token")
      } else {
        # JSON sin destinationEntities ni error — estructura inesperada
        list(status = "ok", data = list())
      }
    },
    .package = "httr2"
  )

  expect_message(
    resultado <- cie11_search("unexpected", api_key = "id:secret"),
    "Sin resultados"
  )

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

test_that("cie11_search retorna tibble vacio con destinationEntities vacio", {
  skip_if_not_installed("httr2")

  call_count <- 0L

  local_mocked_bindings(
    req_perform = function(req, ...) {
      call_count <<- call_count + 1L
      structure(list(call_id = call_count), class = "httr2_response")
    },
    resp_body_json = function(resp, ...) {
      if (resp$call_id == 1L) {
        list(access_token = "mock_token")
      } else {
        # destinationEntities presente pero vacio
        list(destinationEntities = list())
      }
    },
    .package = "httr2"
  )

  expect_message(
    resultado <- cie11_search("nada", api_key = "id:secret"),
    "Sin resultados"
  )

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

# Environment interno para connection pooling
# La conexion se crea lazy (al primer uso), no al cargar el paquete
.ciecl_env <- new.env(parent = emptyenv())
.ciecl_env$con <- NULL
.ciecl_env$db_path <- NULL

.onUnload <- function(libpath) {
  if (!is.null(.ciecl_env$con)) {
    try(DBI::dbDisconnect(.ciecl_env$con), silent = TRUE)
    .ciecl_env$con <- NULL
    .ciecl_env$db_path <- NULL
  }
}

#' Normalizar codigos CIE-10 a formato con punto
#'
#' @description
#' Convierte codigos CIE-10 de diferentes formatos al
#' formato estandar (con punto).
#' Maneja multiples variaciones de entrada comunes en datos clinicos.
#' 
#' @details
#' La normalizacion incluye:
#' \itemize{
#'   \item Conversion a mayusculas
#'   \item Eliminacion de espacios (inicio, fin e internos)
#'   \item Eliminacion de simbolos daga y asterisco (codificacion dual)
#'   \item Conversion de guiones a puntos (I10-0 -> I10.0)
#'   \item Eliminacion de puntos iniciales (.I10 -> I10)
#'   \item Correccion de puntos multiples (E..11 -> E.11)
#'   \item Eliminacion de sufijo X en codigos cortos (I10X -> I10)
#'   \item Preservacion de X en codigos largos (placeholder 7o caracter)
#'   \item Agregado de punto en posicion correcta (E110 -> E11.0)
#' }
#' 
#' El sistema de daga/asterisco indica codificacion dual donde la daga
#' marca la enfermedad subyacente y el asterisco la manifestacion.
#' Ambos simbolos se eliminan para normalizacion.
#' 
#' @param codigos Character vector de codigos en cualquier formato
#' @param buscar_db Logical, buscar codigo en base de datos
#'   si no se encuentra exacto (default TRUE)
#' @return Character vector con codigos normalizados al formato con punto
#' @family validacion
#' @seealso \code{\link{cie_validate_vector}},
#'   \code{\link{cie_expand}}, \code{\link{cie_lookup}}
#' @export
#' @examples
#' cie_normalizar("E110")     # Retorna "E11.0"
#' cie_normalizar("E11")      # Retorna "E11" (categoria)
#' cie_normalizar("I10X")     # Retorna "I10" (elimina X)
#' cie_normalizar("E 11 0")   # Retorna "E11.0" (espacios internos)
#' cie_normalizar("I10-0")    # Retorna "I10.0" (guion a punto)
#' cie_normalizar("A17.0\u2020") # Retorna "A17.0" (elimina daga)
#' cie_normalizar("G01*")     # Retorna "G01" (elimina asterisco)
#' cie_normalizar(c("E110", "I10X", "Z00"))  # Vectorizado
cie_normalizar <- function(codigos, buscar_db = TRUE) {
  # Manejar NULL y vectores vacios

  if (is.null(codigos) || length(codigos) == 0) {
    return(character(0))
  }
  
  # Convertir factores a character
  if (is.factor(codigos)) {
    codigos <- as.character(codigos)
  }
  
  # Manejar vector todo NA
  if (all(is.na(codigos))) {
    return(rep(NA_character_, length(codigos)))
  }
  
  # Normalizar a mayusculas y trim
  codigos_norm <- stringr::str_trim(toupper(codigos))
  
  # === LIMPIEZA DE CARACTERES ESPECIALES ===
  # Eliminar simbolos de codificacion dual (daga U+2020, asterisco)
  # Daga: enfermedad subyacente, Asterisco: manifestacion
  codigos_norm <- stringr::str_replace_all(codigos_norm, "[\u2020\u00B7*]", "")
  
  # Convertir guiones a puntos (comun en sistemas que no aceptan puntos)
  codigos_norm <- stringr::str_replace_all(codigos_norm, "-", ".")
  
  # Eliminar espacios internos (E 11 0 -> E110)
  codigos_norm <- stringr::str_replace_all(codigos_norm, "\\s+", "")
  
  # Eliminar puntos iniciales (.I10 -> I10)
  codigos_norm <- stringr::str_replace_all(codigos_norm, "^\\.+", "")
  
  # Corregir puntos multiples consecutivos (E..11 -> E.11)
  codigos_norm <- stringr::str_replace_all(codigos_norm, "\\.{2,}", ".")
  
  # === MANEJO DE SUFIJO X ===
  # Eliminar X final solo en codigos cortos (<=5 chars)
  # Preservar X en codigos largos donde es placeholder obligatorio (ej. S72X01A)
  codigos_norm <- ifelse(
    nchar(codigos_norm) <= 5 & stringr::str_detect(codigos_norm, "X$"),
    stringr::str_remove(codigos_norm, "X$"),
    codigos_norm
  )
  
  # === FORMATO CON PUNTO ===
  # Agregar punto si no lo tiene (E110 -> E11.0)
  # Solo para codigos de 4+ caracteres sin punto: [A-Z]\d{3,}
  codigos_norm <- ifelse(
    stringr::str_detect(codigos_norm, "^[A-Z]\\d{3,}$") &
      !stringr::str_detect(codigos_norm, "\\."),
    stringr::str_replace(codigos_norm, "^([A-Z]\\d{2})(\\d.*)$", "\\1.\\2"),
    codigos_norm
  )
  
  if (buscar_db) {
    # Verificar que existan en la base de datos
    con <- get_cie10_db()

    codigos_db <- DBI::dbGetQuery(
      con, "SELECT DISTINCT codigo FROM cie10"
    )$codigo

    # Verificacion vectorizada contra DB
    no_na <- !is.na(codigos_norm)
    en_db <- codigos_norm %in% codigos_db
    es_3_dig <- no_na & !en_db & nchar(codigos_norm) == 3 &
      stringr::str_detect(codigos_norm, "^[A-Z]\\d{2}$")
    cod_con_0 <- paste0(codigos_norm, "0")
    fallback_ok <- es_3_dig & (cod_con_0 %in% codigos_db)

    resultado <- codigos_norm
    resultado[fallback_ok] <- cod_con_0[fallback_ok]

    return(resultado)
  } else {
    return(codigos_norm)
  }
}

#' Validar vector de codigos CIE-10 formato
#'
#' @param codigos Character vector codigos (ej. c("E11.0", "Z00.0"))
#' @param strict Logical, validar existencia en DB (default FALSE)
#' @return Logical vector de la misma longitud que `codigos`. TRUE si el
#'   codigo tiene formato CIE-10 valido (y existe en DB si `strict = TRUE`).
#' @family validacion
#' @seealso \code{\link{cie_normalizar}}, \code{\link{cie_expand}}
#' @export
#' @examples
#' cie_validate_vector(c("E11.0", "INVALIDO", "Z00"))
cie_validate_vector <- function(codigos, strict = FALSE) {
  # Regex CIE-10 Chile: acepta formato MINSAL (E110) y estandar (E11.0)
  # MINSAL: [A-Z]\d{2}\d? (3-4 chars: E11, E110)
  # Estandar: [A-Z]\d{2}\.\d{1,2} (con punto: E11.0, E11.00)
  patron <- "^[A-Z]\\d{2}(\\d|\\.\\d{1,2})?$"

  # Normalizar antes de validar formato (I10X -> I10, N10X -> N10)
  codigos_norm <- cie_normalizar(codigos, buscar_db = FALSE)

  # Manejar NAs: retornar FALSE para NAs
  validos_formato <- ifelse(
    is.na(codigos_norm),
    FALSE,
    stringr::str_detect(toupper(codigos_norm), patron)
  )
  
  if (strict) {
    # Validar existencia en DB (conexion pooled)
    con <- get_cie10_db()
    
    codigos_db <- DBI::dbGetQuery(
      con, "SELECT DISTINCT codigo FROM cie10"
    )$codigo
    
    validos_db <- codigos_norm %in% codigos_db
    resultado <- validos_formato & validos_db

    invalidos <- codigos[!resultado & !is.na(codigos)]
    if (length(invalidos) > 0) {
      warning(
        "Codigos no encontrados en DB MINSAL: ",
        paste(invalidos, collapse = ", ")
      )
    }
    
    return(resultado)
  } else {
    return(validos_formato)
  }
}

#' Expandir codigo jerarquico (ej. E11 -> E11.0-E11.9)
#'
#' @param codigo String codigo padre (ej. "E11")
#' @return Character vector con todos los codigos hijos del codigo padre.
#'   Vector vacio si el codigo no existe en la base de datos.
#' @family validacion
#' @seealso \code{\link{cie_normalizar}}, \code{\link{cie_lookup}}
#' @export
#' @examples
#' cie_expand("E11")
cie_expand <- function(codigo) {
  # Manejar NA o cadena vacia
  if (length(codigo) == 0 || is.na(codigo) ||
      nchar(stringr::str_trim(codigo)) == 0) {
    return(character(0))
  }

  hijos <- cie_lookup(codigo, expandir = TRUE)$codigo
  return(hijos)
}

# Tests de rendimiento a gran escala
# Activar con: Sys.setenv(CIECL_LARGE_TESTS = "true")

skip_on_cran()
skip_if_not(
  identical(Sys.getenv("CIECL_LARGE_TESTS"), "true"),
  message = "Tests de escala desactivados (CIECL_LARGE_TESTS != 'true')"
)

# --- Setup comun ---

data("cie10_cl", package = "ciecl", envir = environment())
todos_codigos <- cie10_cl$codigo
set.seed(42)

top_500 <- sample(todos_codigos, min(500, length(todos_codigos)))
resto <- setdiff(todos_codigos, top_500)

generar_codigos <- function(n, prop_top = 0.8) {
  n_top <- round(n * prop_top)
  n_resto <- n - n_top
  c(
    sample(top_500, n_top, replace = TRUE),
    sample(resto, n_resto, replace = TRUE)
  )
}

# --- Tests ---

test_that("Patron GITTESIS 1M registros: lookup + join < 10s", {
  codigos_sim <- generar_codigos(1e6)
  df_sim <- dplyr::tibble(id = seq_len(1e6), codigo = codigos_sim)

  t <- system.time({
    unicos <- unique(codigos_sim)
    resultado <- cie_lookup(unicos)
    df_final <- dplyr::left_join(df_sim, resultado, by = "codigo")
  })

  expect_true(t["elapsed"] < 10,
    label = sprintf("GITTESIS pattern: %.2fs (limite 10s)", t["elapsed"]))
  expect_equal(nrow(df_final), 1e6)
  expect_true("descripcion" %in% names(df_final))
})

test_that("cie_validate_vector 1M codigos < 10s", {
  codigos_sim <- generar_codigos(1e6)

  t <- system.time({
    resultado <- cie_validate_vector(codigos_sim)
  })

  expect_true(t["elapsed"] < 10,
    label = sprintf("validate_vector 1M: %.2fs (limite 10s)", t["elapsed"]))
  expect_length(resultado, 1e6)
  expect_type(resultado, "logical")
})

test_that("cie_normalizar 500K codigos < 5s", {
  codigos_sim <- generar_codigos(5e5)
  # Agregar variaciones realistas
  codigos_sim[1:500] <- gsub("\\.", "", codigos_sim[1:500])
  codigos_sim[501:1000] <- paste0(" ", codigos_sim[501:1000], " ")

  t <- system.time({
    resultado <- cie_normalizar(codigos_sim, buscar_db = TRUE)
  })

  expect_true(t["elapsed"] < 5,
    label = sprintf("normalizar 500K: %.2fs (limite 5s)", t["elapsed"]))
  expect_length(resultado, 5e5)
  # Verificar que normalizo correctamente
  expect_false(any(grepl("^\\s|\\s$", resultado, perl = TRUE), na.rm = TRUE))
})

test_that("cie_map_comorbid 100K codigos vectorizado", {
  codigos_sim <- generar_codigos(1e5)

  t <- system.time({
    resultado <- cie_map_comorbid(codigos_sim)
  })

  expect_true(t["elapsed"] < 5,
    label = sprintf("map_comorbid 100K: %.2fs (limite 5s)", t["elapsed"]))
  expect_equal(nrow(resultado), 1e5)
  expect_true(all(c("codigo", "categoria") %in% names(resultado)))
  # Verificar que hay categorias diversas
  expect_true(length(unique(resultado$categoria)) > 1)
})

test_that("cie_comorbid 10K pacientes < 60s", {
  skip_if_not_installed("comorbidity")

  n_pac <- 1e4
  n_diag <- n_pac * 5
  df_sim <- dplyr::tibble(
    id_pac = rep(seq_len(n_pac), each = 5),
    diag = generar_codigos(n_diag)
  )

  t <- system.time({
    resultado <- cie_comorbid(df_sim, id = "id_pac", code = "diag", map = "charlson")
  })

  expect_true(t["elapsed"] < 60,
    label = sprintf("comorbid 10K pac: %.2fs (limite 60s)", t["elapsed"]))
  expect_equal(nrow(resultado), n_pac)
  expect_true("score_charlson" %in% names(resultado))
})

# test-integration.R
# Pruebas de integracion que verifican flujos de trabajo completos

# ============================================================
# FLUJOS DE TRABAJO TIPICOS DE USUARIO
# ============================================================

test_that("flujo: buscar termino -> obtener codigos -> validar", {
  skip_on_cran()

  # 1. Usuario busca termino medico
  resultados_busqueda <- cie_search("diabetes mellitus", threshold = 0.70, max_results = 10)
  expect_gt(nrow(resultados_busqueda), 0)

  # 2. Obtiene codigos de los resultados
  codigos_encontrados <- resultados_busqueda$codigo

  # 3. Valida que los codigos son correctos
  validacion <- cie_validate_vector(codigos_encontrados)
  expect_true(all(validacion), info = "Todos los codigos de busqueda deben ser validos")

  # 4. Obtiene detalles de cada codigo
  detalles <- cie_lookup(codigos_encontrados)
  expect_equal(nrow(detalles), length(unique(codigos_encontrados)))
})

test_that("flujo: buscar categoria -> expandir -> calcular comorbilidad", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  # 1. Buscar categoria general de diabetes
  resultado_e11 <- cie_lookup("E11")
  expect_equal(nrow(resultado_e11), 1)

  # 2. Expandir para obtener todos los subcigodos
  hijos_e11 <- cie_expand("E11")
  expect_gt(length(hijos_e11), 5)

  # 3. Crear datos de paciente simulados
  set.seed(123)
  datos_pacientes <- data.frame(
    id_paciente = rep(1:5, each = 2),
    codigo_cie = sample(hijos_e11, 10, replace = TRUE)
  )

  # 4. Calcular comorbilidades
  resultado_comorbid <- cie_comorbid(
    datos_pacientes,
    id = "id_paciente",
    code = "codigo_cie",
    map = "charlson"
  )

  expect_s3_class(resultado_comorbid, "tbl_df")
  expect_true("score_charlson" %in% names(resultado_comorbid))
})

test_that("flujo: normalizar codigos -> buscar -> mapear comorbilidad", {
  skip_on_cran()

  # 1. Codigos en formato mixto (como vendrian de datos reales)
  codigos_raw <- c("E110", "I509", "C509", "e11.0", " Z00 ")

  # 2. Normalizar codigos
  codigos_norm <- cie_normalizar(codigos_raw, buscar_db = FALSE)
  expect_equal(length(codigos_norm), length(codigos_raw))

  # 3. Buscar detalles
  suppressMessages({
    detalles <- cie_lookup(codigos_norm)
  })
  expect_gt(nrow(detalles), 0)

  # 4. Mapear a categorias de comorbilidad
  mapa <- cie_map_comorbid(codigos_norm)
  expect_equal(nrow(mapa), length(codigos_norm))
  expect_true("Diabetes" %in% mapa$categoria)
})

test_that("flujo: SQL personalizado -> procesamiento -> validacion", {
  skip_on_cran()

  # 1. Query SQL personalizada para obtener codigos por capitulo
  codigos_cap4 <- cie10_sql("
    SELECT codigo, descripcion
    FROM cie10
    WHERE codigo LIKE 'E%'
    LIMIT 50
  ")
  expect_s3_class(codigos_cap4, "tbl_df")
  expect_gt(nrow(codigos_cap4), 0)

  # 2. Validar codigos obtenidos
  validacion <- cie_validate_vector(codigos_cap4$codigo)
  expect_true(all(validacion))

  # 3. Obtener detalles completos
  detalles <- cie_lookup(codigos_cap4$codigo, descripcion_completa = TRUE)
  expect_true("descripcion_completa" %in% names(detalles))
})

# ============================================================
# PRUEBAS DE CONSISTENCIA ENTRE FUNCIONES
# ============================================================

test_that("cie_lookup y cie10_sql retornan mismos datos", {
  skip_on_cran()

  # Obtener codigo via cie_lookup
  resultado_lookup <- cie_lookup("E11.0")

  # Obtener mismo codigo via SQL directo
  resultado_sql <- cie10_sql("SELECT * FROM cie10 WHERE codigo = 'E11.0'")

  # Deben ser equivalentes
  expect_equal(resultado_lookup$codigo, resultado_sql$codigo)
  expect_equal(resultado_lookup$descripcion, resultado_sql$descripcion)
})

test_that("cie_expand y cie_lookup expandir dan mismos resultados", {
  skip_on_cran()

  # Via cie_expand
  hijos_expand <- cie_expand("E11")

  # Via cie_lookup con expandir=TRUE
  hijos_lookup <- cie_lookup("E11", expandir = TRUE)$codigo

  # Deben ser iguales
  expect_setequal(hijos_expand, hijos_lookup)
})

test_that("cie_normalizar y cie_lookup son coherentes", {
  skip_on_cran()

  # Codigo sin punto
  codigo_raw <- "E110"

  # Normalizar
  codigo_norm <- cie_normalizar(codigo_raw, buscar_db = FALSE)
  expect_equal(codigo_norm, "E11.0")

  # Buscar con codigo raw y normalizado
  resultado_raw <- cie_lookup(codigo_raw)
  resultado_norm <- cie_lookup(codigo_norm)

  # Deben dar mismo resultado
  expect_equal(resultado_raw$codigo, resultado_norm$codigo)
})

test_that("cie_validate_vector y cie_lookup son coherentes", {
  skip_on_cran()

  codigos <- c("E11.0", "INVALIDO", "Z00")

  # Validar
  validacion <- cie_validate_vector(codigos)
  expect_equal(validacion, c(TRUE, FALSE, TRUE))

  # Buscar (solo los validos deben retornar)
  suppressMessages({
    resultado <- cie_lookup(codigos)
  })

  # Solo codigos validos en resultado
  expect_true(all(resultado$codigo %in% codigos[validacion]))
})

# ============================================================
# PRUEBAS DE ESCENARIOS REALES
# ============================================================

test_that("escenario: analisis de egreso hospitalario", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  # Simular datos de egresos hospitalarios
  set.seed(42)
  egresos <- data.frame(
    id_egreso = 1:20,
    codigo_principal = sample(c("E11.0", "I50.9", "C50.9", "J18.9", "K70.3"), 20, replace = TRUE),
    codigo_secundario = sample(c("E11.9", "I10", "Z86.7", "N18.9", "F32.9"), 20, replace = TRUE)
  )

  # 1. Validar codigos principales
  val_principal <- cie_validate_vector(egresos$codigo_principal)
  expect_true(all(val_principal))

  # 2. Validar codigos secundarios
  val_secundario <- cie_validate_vector(egresos$codigo_secundario)
  expect_true(all(val_secundario))

  # 3. Preparar datos para comorbilidad (formato largo)
  datos_largo <- data.frame(
    id = rep(egresos$id_egreso, 2),
    codigo = c(egresos$codigo_principal, egresos$codigo_secundario)
  )

  # 4. Calcular comorbilidades
  comorbilidades <- cie_comorbid(datos_largo, id = "id", code = "codigo", map = "charlson")
  expect_s3_class(comorbilidades, "tbl_df")
  expect_equal(nrow(comorbilidades), 20)
})

test_that("escenario: busqueda de codigos para estudio", {
  skip_on_cran()

  # Investigador busca codigos relacionados con su estudio
  # 1. Buscar por termino
  resultados_diabetes <- cie_search("diabetes mellitus tipo 2", threshold = 0.65)

  # 2. Buscar categoria general
  categoria_e11 <- cie_lookup("E11", expandir = TRUE)

  # 3. Combinar resultados
  codigos_estudio <- unique(c(resultados_diabetes$codigo, categoria_e11$codigo))

  # 4. Validar todos
  validacion <- cie_validate_vector(codigos_estudio)
  expect_true(all(validacion))

  # 5. Obtener descripciones completas
  descripciones <- cie_lookup(codigos_estudio, descripcion_completa = TRUE)
  expect_true("descripcion_completa" %in% names(descripciones))
})

test_that("escenario: limpieza de datos con codigos sucios", {
  skip_on_cran()

  # Datos con codigos en diferentes formatos (escenario real)
  codigos_sucios <- c(
    "E110",     # Sin punto - valido
    "e11.0",    # Minusculas - valido
    " E11.0 ",  # Espacios - valido (se normaliza con trim)
    "E11",      # Categoria - valido
    "DIABETES", # Texto (invalido)
    NA,         # NA - invalido
    "",         # Vacio - invalido
    "E11.0"     # Correcto - valido
  )

  # 1. Validar formato (normaliza internamente: trim, punto, sufijo X)
  formato_ok <- cie_validate_vector(codigos_sucios)
  expect_equal(formato_ok, c(TRUE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, TRUE))

  # 2. Filtrar solo validos
  codigos_validos <- codigos_sucios[formato_ok]

  # 3. Normalizar
  codigos_norm <- cie_normalizar(codigos_validos, buscar_db = FALSE)

  # 4. Buscar en base
  suppressMessages({
    resultado <- cie_lookup(codigos_norm)
  })
  expect_gt(nrow(resultado), 0)
})

# ============================================================
# PRUEBAS DE RENDIMIENTO BASICAS
# ============================================================

test_that("busquedas multiples son razonablemente rapidas", {
  skip_on_cran()

  # Medir tiempo de 100 busquedas simples
  tiempo_inicio <- Sys.time()

  for (i in 1:100) {
    resultado <- cie_lookup("E11.0")
  }

  tiempo_fin <- Sys.time()
  duracion <- as.numeric(difftime(tiempo_fin, tiempo_inicio, units = "secs"))

  # Debe completar en menos de 30 segundos (muy generoso)
  expect_lt(duracion, 30)
})

test_that("validacion de vector grande es rapida", {
  skip_on_cran()

  # Vector de 10000 codigos
  codigos <- rep(c("E11.0", "Z00", "INVALIDO"), 3333)
  codigos <- c(codigos, "E11.0")  # Para llegar a 10000

  tiempo_inicio <- Sys.time()
  resultado <- cie_validate_vector(codigos)
  tiempo_fin <- Sys.time()

  duracion <- as.numeric(difftime(tiempo_fin, tiempo_inicio, units = "secs"))

  # Debe completar en menos de 5 segundos
  expect_lt(duracion, 5)
  expect_length(resultado, 10000)
})

# ============================================================
# PRUEBAS DE INTEROPERABILIDAD CON dplyr
# ============================================================

test_that("resultados funcionan con dplyr::filter", {
  skip_on_cran()

  resultado <- cie_search("diabetes", threshold = 0.70, max_results = 20)

  # Filtrar con dplyr
  filtrado <- resultado %>%
    dplyr::filter(score > 0.80)

  expect_s3_class(filtrado, "tbl_df")
})

test_that("resultados funcionan con dplyr::mutate", {
  skip_on_cran()

  resultado <- cie_lookup(c("E11.0", "E11.1", "E11.2"))

  # Mutar con dplyr
  mutado <- resultado %>%
    dplyr::mutate(codigo_corto = substr(codigo, 1, 3))

  expect_s3_class(mutado, "tbl_df")
  expect_true("codigo_corto" %in% names(mutado))
})

test_that("resultados funcionan con dplyr::group_by y summarise", {
  skip_on_cran()

  resultado <- cie10_sql("SELECT codigo, capitulo FROM cie10 WHERE codigo LIKE 'E1%' LIMIT 100")

  # Agrupar y resumir
  resumen <- resultado %>%
    dplyr::group_by(capitulo) %>%
    dplyr::summarise(n = dplyr::n(), .groups = "drop")

  expect_s3_class(resumen, "tbl_df")
  expect_true("n" %in% names(resumen))
})

# ============================================================
# PRUEBAS DE API CIE-11 (SOLO SI HAY CREDENCIALES)
# ============================================================

test_that("cie11_search falla gracefully sin credenciales", {
  skip_on_cran()
  skip_if_not_installed("httr2")

  # Limpiar credenciales temporalmente
  old_key <- Sys.getenv("ICD_API_KEY")
  Sys.unsetenv("ICD_API_KEY")
  on.exit(if (nchar(old_key) > 0) Sys.setenv(ICD_API_KEY = old_key))

  # Sin credenciales, debe dar error informativo
  expect_error(
    cie11_search("diabetes"),
    regexp = "API key|requerida|OMS",
    ignore.case = TRUE
  )
})

# test-encoding.R
# Pruebas de compatibilidad con diferentes encodings y caracteres especiales

# ============================================================
# PRUEBAS DE ENCODING PARA CARACTERES ESPANOLES
# ============================================================

test_that("cie_search maneja caracteres con tildes", {
  skip_on_cran()


  # Busqueda con tildes en espanol
  resultado <- cie_search("neumonia", threshold = 0.70)
  expect_s3_class(resultado, "tbl_df")
  expect_gt(nrow(resultado), 0)

  # Busqueda con ene
  resultado_ene <- cie_search("rinon", threshold = 0.70)
  expect_s3_class(resultado_ene, "tbl_df")
})

test_that("cie_search encuentra terminos con y sin tildes",
  {
  skip_on_cran()

  # Ambas versiones deben encontrar resultados
  resultado_sin <- cie_search("diabetes", threshold = 0.70)
  resultado_con <- cie_search("diabetis", threshold = 0.70)

  expect_gt(nrow(resultado_sin), 0)
  expect_gt(nrow(resultado_con), 0)
})

test_that("base de datos contiene descripciones con tildes correctas", {
  skip_on_cran()

  # Verificar que las descripciones con tildes estan correctas
  resultado <- cie10_sql("SELECT * FROM cie10 WHERE descripcion LIKE '%neumon%' LIMIT 5")
  expect_s3_class(resultado, "tbl_df")

  # Verificar que no hay caracteres corruptos tipicos de encoding incorrecto
  if (nrow(resultado) > 0) {
    # Buscar caracteres corruptos comunes
    descripciones <- resultado$descripcion
    tiene_corruptos <- any(stringr::str_detect(descripciones, "\ufffd|Ã¡|Ã©|Ã\u00ad|Ã³|Ãº|Ã±"))
    expect_false(tiene_corruptos, info = "Las descripciones no deben tener caracteres corruptos")
  }
})

test_that("cie_lookup maneja codigos sin importar encoding", {
  skip_on_cran()

  # El codigo debe funcionar independientemente del encoding del sistema
  resultado <- cie_lookup("E11.0")
  expect_equal(nrow(resultado), 1)
  expect_s3_class(resultado, "tbl_df")
})

# ============================================================
# PRUEBAS DE CARACTERES ESPECIALES
# ============================================================

test_that("cie_search maneja parentesis y corchetes", {
  skip_on_cran()

  # Texto con parentesis
  expect_no_error({
    suppressMessages(cie_search("diabetes (tipo 2)", threshold = 0.5))
  })

  # Texto con corchetes
  expect_no_error({
    suppressMessages(cie_search("diabetes [mellitus]", threshold = 0.5))
  })
})

test_that("cie_search maneja signos de puntuacion", {
  skip_on_cran()

  # Coma
  expect_no_error({
    suppressMessages(cie_search("diabetes, tipo 2", threshold = 0.5))
  })

  # Punto y coma
  expect_no_error({
    suppressMessages(cie_search("diabetes; insulino", threshold = 0.5))
  })

  # Dos puntos
  expect_no_error({
    suppressMessages(cie_search("diabetes: tipo 2", threshold = 0.5))
  })
})

test_that("cie_search maneja guiones y barras", {
  skip_on_cran()

  # Guion
  resultado <- cie_search("insulino-dependiente", threshold = 0.50)
  expect_s3_class(resultado, "tbl_df")

  # Barra
  expect_no_error({
    suppressMessages(cie_search("diabetes mellitus/tipo", threshold = 0.5))
  })
})

test_that("cie10_sql maneja comillas en queries", {
  skip_on_cran()

  # Query con comillas simples (parametros)
  resultado <- cie10_sql("SELECT * FROM cie10 WHERE codigo = 'E11.0'")
  expect_s3_class(resultado, "tbl_df")

  # Query con LIKE y comillas
  resultado2 <- cie10_sql("SELECT * FROM cie10 WHERE descripcion LIKE '%diabetes%' LIMIT 5")
  expect_s3_class(resultado2, "tbl_df")
})

# ============================================================
# PRUEBAS DE UNICODE
# ============================================================

test_that("cie_search maneja caracteres unicode validos", {
  skip_on_cran()

  # Caracteres unicode basicos deben ser manejados
  expect_no_error({
    suppressMessages(cie_search("diabetes", threshold = 0.70))
  })
})

test_that("base de datos mantiene integridad unicode", {
  skip_on_cran()

  # Verificar que los datos no estan corruptos
  resultado <- cie10_sql("SELECT COUNT(*) as n FROM cie10")
  expect_gt(resultado$n, 5000)

  # Verificar que hay codigos con descripciones no vacias
  resultado2 <- cie10_sql("SELECT COUNT(*) as n FROM cie10 WHERE descripcion IS NOT NULL AND descripcion != ''")
  expect_gt(resultado2$n, 5000)
})

# ============================================================
# PRUEBAS DE CONSISTENCIA ENTRE PLATAFORMAS
# ============================================================

test_that("cie_normalizar es consistente con diferentes inputs", {
  skip_on_cran()

  # Mismos resultados independientemente de mayusculas/minusculas
  resultado_may <- cie_normalizar("E110", buscar_db = FALSE)
  resultado_min <- cie_normalizar("e110", buscar_db = FALSE)

  expect_equal(resultado_may, resultado_min)
})

test_that("cie_validate_vector es case-insensitive", {
  # Debe validar independientemente del case
  expect_true(cie_validate_vector("E11.0"))
  expect_true(cie_validate_vector("e11.0"))
  expect_true(cie_validate_vector("E11.0"))
})

test_that("cie_lookup es case-insensitive", {
  skip_on_cran()

  resultado_may <- cie_lookup("E11.0")
  resultado_min <- cie_lookup("e11.0")
  resultado_mix <- cie_lookup("e11.0")

  expect_equal(resultado_may$codigo, resultado_min$codigo)
  expect_equal(resultado_may$codigo, resultado_mix$codigo)
})

# ============================================================
# PRUEBAS DE LOCALE
# ============================================================

test_that("funciones operan independientemente del locale", {
  skip_on_cran()

  # Guardar locale actual
  old_locale <- Sys.getlocale("LC_COLLATE")

  # Las funciones deben operar correctamente
  resultado <- cie_lookup("E11.0")
  expect_equal(nrow(resultado), 1)

  resultado2 <- cie_search("diabetes", threshold = 0.70)
  expect_gt(nrow(resultado2), 0)

  # Restaurar locale (por si acaso cambio)
  tryCatch(
    Sys.setlocale("LC_COLLATE", old_locale),
    error = function(e) NULL
  )
})

# ============================================================
# PRUEBAS DE COMPARACION STRING
# ============================================================

test_that("cie_search usa comparacion case-insensitive", {
  skip_on_cran()

  resultado_lower <- cie_search("diabetes mellitus", threshold = 0.70)
  resultado_upper <- cie_search("DIABETES MELLITUS", threshold = 0.70)
  resultado_mixed <- cie_search("Diabetes Mellitus", threshold = 0.70)

  # Todos deben encontrar resultados similares
  expect_gt(nrow(resultado_lower), 0)
  expect_gt(nrow(resultado_upper), 0)
  expect_gt(nrow(resultado_mixed), 0)

  # Los codigos encontrados deben ser similares
  expect_true(any(resultado_lower$codigo %in% resultado_upper$codigo))
})

test_that("similitud Jaro-Winkler funciona con tildes", {
  skip_on_cran()

  # La similitud debe funcionar aunque haya diferencias de tildes
  resultado1 <- cie_search("neumonia", threshold = 0.60)
  resultado2 <- cie_search("neumonía", threshold = 0.60)

  # Ambos deben encontrar resultados (aunque no exactamente los mismos)
  expect_s3_class(resultado1, "tbl_df")
  expect_s3_class(resultado2, "tbl_df")
})

# test-edge-cases.R
# Pruebas exhaustivas para edge cases y entradas problematicas

# ============================================================
# PRUEBAS PARA cie_search()
# ============================================================

test_that("cie_search rechaza NA, vectores y tipos invalidos", {
  skip_on_cran()

  # NA debe dar error
  expect_error(cie_search(NA), "texto debe ser un string character no-NA")
  expect_error(cie_search(NA_character_), "texto debe ser un string character no-NA")

  # Vectores de longitud != 1 deben dar error
  expect_error(cie_search(c("diabetes", "cancer")), "texto debe ser un string character no-NA")
  expect_error(cie_search(character(0)), "texto debe ser un string character no-NA")

  # Tipos invalidos deben dar error
  expect_error(cie_search(123), "texto debe ser un string character no-NA")
  expect_error(cie_search(list("diabetes")), "texto debe ser un string character no-NA")
})

test_that("cie_search maneja cadenas muy cortas", {
  skip_on_cran()


  # Texto de 2 caracteres ahora es valido (para siglas como DM, FA, TB)
  # Sigla DM debe expandirse a "diabetes mellitus"
  expect_no_error(suppressMessages(cie_search("DM")))

  # Texto de 1 caracter o vacio debe dar error
  expect_error(cie_search("a"), "Texto minimo 2 caracteres")
  expect_error(cie_search(""), "Texto minimo 2 caracteres")
  expect_error(cie_search(" "), "Texto minimo 2 caracteres")
})

test_that("cie_search maneja threshold invalido", {
  skip_on_cran()

  # Threshold debe estar entre 0 y 1
  # Con threshold muy alto no deberia encontrar nada
  resultado <- cie_search("diabetes mellitus", threshold = 0.99)
  expect_s3_class(resultado, "tbl_df")

  # Con threshold = 0 debe encontrar muchos (todos tienen score >= 0)
  resultado_bajo <- cie_search("diabetes", threshold = 0, max_results = 5)
  expect_s3_class(resultado_bajo, "tbl_df")
})

test_that("cie_search maneja max_results extremos", {
  skip_on_cran()

  # max_results = 1

  resultado <- cie_search("diabetes", threshold = 0.70, max_results = 1)
  expect_lte(nrow(resultado), 1)

  # max_results muy grande
  resultado_grande <- cie_search("diabetes", threshold = 0.60, max_results = 10000)
  expect_s3_class(resultado_grande, "tbl_df")
})

test_that("cie_search maneja caracteres especiales en texto", {
  skip_on_cran()

  # Texto con caracteres especiales SQL (inyeccion)
  expect_no_error({
    suppressMessages(cie_search("diabetes'; DROP TABLE cie10;--", threshold = 0.5))
  })

  # Texto con comillas
  expect_no_error({
    suppressMessages(cie_search("diabetes \"tipo 1\"", threshold = 0.5))
  })

  # Texto con tildes y caracteres especiales espanol
  expect_no_error({
    resultado <- cie_search("diarrea infecciosa", threshold = 0.70)
  })
})

test_that("cie_search maneja espacios multiples", {
  skip_on_cran()

  # Texto con espacios multiples debe funcionar igual
  resultado1 <- cie_search("diabetes mellitus", threshold = 0.80)
  resultado2 <- cie_search("diabetes   mellitus", threshold = 0.80)

  # Ambos deben retornar resultados
  expect_gt(nrow(resultado1), 0)
  expect_gt(nrow(resultado2), 0)
})

test_that("cie_search maneja campo invalido", {
  skip_on_cran()

  # Campo invalido debe usar default o dar error
  expect_error(cie_search("diabetes", campo = "inexistente"))
})

# ============================================================
# PRUEBAS PARA cie_lookup()
# ============================================================

test_that("cie_lookup maneja NA en entrada", {
  skip_on_cran()

  # NA como codigo
  suppressMessages({
    resultado <- cie_lookup(NA_character_)
  })
  expect_equal(nrow(resultado), 0)
})

test_that("cie_lookup maneja vector vacio", {
  skip_on_cran()

  # Vector vacio
  resultado <- cie_lookup(character(0))
  expect_equal(nrow(resultado), 0)
})

test_that("cie_lookup maneja cadena vacia", {
  skip_on_cran()

  suppressMessages({
    resultado <- cie_lookup("")
  })
  expect_equal(nrow(resultado), 0)
})

test_that("cie_lookup maneja codigo con espacios", {
  skip_on_cran()

  # Codigo con espacios debe ser normalizado
  resultado1 <- cie_lookup("E11.0")
  resultado2 <- cie_lookup(" E11.0 ")

  expect_equal(nrow(resultado1), nrow(resultado2))
  expect_equal(resultado1$codigo, resultado2$codigo)
})

test_that("cie_lookup maneja minusculas", {
  skip_on_cran()

  # Codigo en minusculas debe funcionar
  resultado1 <- cie_lookup("E11.0")
  resultado2 <- cie_lookup("e11.0")

  expect_equal(nrow(resultado1), nrow(resultado2))
  expect_equal(resultado1$codigo, resultado2$codigo)
})

test_that("cie_lookup maneja rangos invalidos", {
  skip_on_cran()

  # Rango invertido
  suppressWarnings(
    resultado <- cie_lookup("E14-E10")
  )
  # Puede no encontrar resultados pero no debe crashear
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_lookup maneja vector con NAs mezclados", {
  skip_on_cran()

  # Vector con NAs mezclados
  codigos <- c("E11.0", NA, "Z00", NA)
  suppressMessages({
    resultado <- cie_lookup(codigos)
  })

  # Debe procesar los codigos validos
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_lookup maneja codigo con caracteres SQL peligrosos", {
  skip_on_cran()

  # Intento de SQL injection - debe ser rechazado con mensaje, no error
  codigos_peligrosos <- c(
    "E11'; DROP TABLE cie10;--",
    "E11.0 OR 1=1",
    "E11%"
  )

  for (cod in codigos_peligrosos) {
    # Debe retornar tibble vacio sin crashear
    suppressMessages({
      resultado <- cie_lookup(cod)
    })
    expect_s3_class(resultado, "tbl_df")
    expect_equal(nrow(resultado), 0)
  }

  # E11* se normaliza a E11 (asterisco = codificacion dual, se elimina)
  # por lo tanto encuentra resultados legitimamente
  suppressMessages({
    resultado_ast <- cie_lookup("E11*")
  })
  expect_s3_class(resultado_ast, "tbl_df")
  expect_gt(nrow(resultado_ast), 0)
})

test_that("cie_lookup expandir con codigo inexistente", {
  skip_on_cran()

  # Expandir codigo que no existe
  suppressMessages({
    resultado <- cie_lookup("ZZZ", expandir = TRUE)
  })
  expect_equal(nrow(resultado), 0)
})

# ============================================================
# PRUEBAS PARA cie_normalizar()
# ============================================================

test_that("cie_normalizar maneja NA", {
  skip_on_cran()

  # NA como entrada
  resultado <- cie_normalizar(NA_character_, buscar_db = FALSE)
  expect_true(is.na(resultado))
})

test_that("cie_normalizar maneja vector con NAs", {
  skip_on_cran()

  codigos <- c("E110", NA, "I100")
  resultado <- cie_normalizar(codigos, buscar_db = FALSE)

  expect_equal(resultado[1], "E11.0")
  expect_true(is.na(resultado[2]))
  expect_equal(resultado[3], "I10.0")
})

test_that("cie_normalizar maneja cadena vacia", {
  skip_on_cran()

  resultado <- cie_normalizar("", buscar_db = FALSE)
  expect_equal(resultado, "")
})

test_that("cie_normalizar maneja codigos ya normalizados", {
  skip_on_cran()

  # Codigos ya con punto no deben cambiar
  codigos <- c("E11.0", "I10.0", "Z00.0")
  resultado <- cie_normalizar(codigos, buscar_db = FALSE)
  expect_equal(resultado, codigos)
})

test_that("cie_normalizar maneja codigos de 3 caracteres", {
  skip_on_cran()

  # Codigos de categoria (3 chars) no deben modificarse
  resultado <- cie_normalizar("E11", buscar_db = FALSE)
  expect_equal(resultado, "E11")
})

test_that("cie_normalizar maneja codigos largos", {
  skip_on_cran()

  # Codigos de 5 digitos (E11.00 formato)
  resultado <- cie_normalizar("E1100", buscar_db = FALSE)
  expect_equal(resultado, "E11.00")
})

# ============================================================
# PRUEBAS PARA cie_validate_vector()
# ============================================================

test_that("cie_validate_vector maneja NA", {
  # NA debe ser FALSE
  resultado <- cie_validate_vector(NA_character_)
  expect_false(resultado)
})

test_that("cie_validate_vector maneja vector vacio", {
  resultado <- cie_validate_vector(character(0))
  expect_length(resultado, 0)
})

test_that("cie_validate_vector maneja cadena vacia", {
  resultado <- cie_validate_vector("")
  expect_false(resultado)
})

test_that("cie_validate_vector acepta formatos validos", {
  # Todos los formatos validos CIE-10
  codigos_validos <- c(
    "E11",      # Categoria (3 chars)
    "E11.0",    # Subcategoria con punto
    "E11.00",   # Subcategoria con 2 decimales
    "E110",     # Formato MINSAL sin punto
    "Z00",
    "A00.0",
    "B99.9"
  )

  resultado <- cie_validate_vector(codigos_validos)
  expect_true(all(resultado))
})

test_that("cie_validate_vector rechaza formatos invalidos", {
  codigos_invalidos <- c(
    "INVALIDO",   # Texto
    "123",        # Solo numeros
    "E1",         # Muy corto
    "EE11",       # Letra duplicada
    "E11.111",    # Demasiados decimales
    "1E11",       # Numero al inicio
    ""            # Vacio
  )

  resultado <- cie_validate_vector(codigos_invalidos)
  expect_true(all(!resultado))
})

test_that("cie_validate_vector normaliza antes de validar", {
  # Punto duplicado se normaliza a valido (E11..0 -> E11.0)
  expect_true(cie_validate_vector("E11..0"))

  # Minusculas se normalizan
  expect_true(cie_validate_vector("e11.0"))
})

test_that("cie_validate_vector strict mode", {
  skip_on_cran()

  # Modo estricto verifica existencia en DB
  codigos <- c("E11.0", "Z00", "XXXX")

  expect_warning({
    resultado <- cie_validate_vector(codigos, strict = TRUE)
  })

  # Los dos primeros deben ser validos, el tercero no
  expect_equal(resultado[1:2], c(TRUE, TRUE))
  expect_false(resultado[3])
})

# ============================================================
# PRUEBAS PARA cie_expand()
# ============================================================

test_that("cie_expand maneja codigo vacio", {
  skip_on_cran()

  suppressMessages({
    resultado <- cie_expand("")
  })
  expect_length(resultado, 0)
})

test_that("cie_expand maneja NA", {
  skip_on_cran()

  suppressMessages({
    resultado <- cie_expand(NA_character_)
  })
  expect_length(resultado, 0)
})

test_that("cie_expand maneja codigo inexistente", {
  skip_on_cran()

  suppressMessages({
    resultado <- cie_expand("ZZZ")
  })
  expect_length(resultado, 0)
})

test_that("cie_expand retorna hijos correctos", {
  skip_on_cran()

  # E11 debe tener multiples hijos
  hijos <- cie_expand("E11")

  # Todos deben empezar con E11
  expect_true(all(stringr::str_starts(hijos, "E11")))

  # Debe haber al menos E11.0
  expect_true("E11.0" %in% hijos || any(stringr::str_detect(hijos, "^E11\\.0")))
})

# ============================================================
# PRUEBAS PARA cie_map_comorbid()
# ============================================================

test_that("cie_map_comorbid maneja vector vacio", {
  resultado <- cie_map_comorbid(character(0))
  expect_equal(nrow(resultado), 0)
  expect_true("codigo" %in% names(resultado))
  expect_true("categoria" %in% names(resultado))
})

test_that("cie_map_comorbid maneja NA", {
  resultado <- cie_map_comorbid(NA_character_)
  expect_equal(nrow(resultado), 1)
  expect_equal(resultado$categoria, "Otra")
})

test_that("cie_map_comorbid categoriza correctamente multiples codigos", {
  codigos <- c(
    "E11.0",   # Diabetes
    "I50.9",   # Insuficiencia cardiaca
    "I21.0",   # Infarto miocardio
    "C50.9",   # Neoplasia maligna
    "J44.0",   # EPOC
    "N18.0",   # Enfermedad renal cronica
    "F32.0",   # Trastornos mentales
    "Z00.0"    # Otra (no mapeada)
  )

  resultado <- cie_map_comorbid(codigos)

  expect_equal(resultado$categoria[resultado$codigo == "E11.0"], "Diabetes")
  expect_equal(resultado$categoria[resultado$codigo == "I50.9"], "Insuficiencia cardiaca")
  expect_equal(resultado$categoria[resultado$codigo == "I21.0"], "Infarto miocardio")
  expect_equal(resultado$categoria[resultado$codigo == "C50.9"], "Neoplasia maligna")
  expect_equal(resultado$categoria[resultado$codigo == "J44.0"], "EPOC")
  expect_equal(resultado$categoria[resultado$codigo == "N18.0"], "Enfermedad renal cronica")
  expect_equal(resultado$categoria[resultado$codigo == "F32.0"], "Trastornos mentales")
  expect_equal(resultado$categoria[resultado$codigo == "Z00.0"], "Otra")
})

# ============================================================
# PRUEBAS PARA cie_comorbid()
# ============================================================

test_that("cie_comorbid rechaza dataframe vacio", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df_vacio <- data.frame(
    id = character(0),
    diag = character(0)
  )

  # El paquete comorbidity no acepta datos vacios, esto es comportamiento esperado
  expect_error(
    cie_comorbid(df_vacio, id = "id", code = "diag", map = "charlson"),
    "non-missing data"
  )
})

test_that("cie_comorbid detecta columnas inexistentes", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    paciente = c(1, 2),
    codigo = c("E11.0", "I50.9")
  )

  # Columna id incorrecta
  expect_error(
    cie_comorbid(df, id = "id_paciente", code = "codigo"),
    "no existen en data"
  )

  # Columna code incorrecta
  expect_error(
    cie_comorbid(df, id = "paciente", code = "diagnostico"),
    "no existen en data"
  )
})

test_that("cie_comorbid funciona con map elixhauser", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 2),
    diag = c("E11.0", "I50.9", "C50.9")
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "elixhauser")
  expect_s3_class(resultado, "tbl_df")
  expect_false("score_charlson" %in% names(resultado))
})

test_that("cie_comorbid maneja codigos con NA", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 2, 2),
    diag = c("E11.0", NA, "I50.9", "C50.9")
  )

  # No debe crashear con NAs (warning esperado por NA values)
  expect_no_error({
    suppressWarnings({
      resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
    })
  })
})

# ============================================================
# PRUEBAS PARA cie10_sql()
# ============================================================

test_that("cie10_sql bloquea queries UPDATE", {
  expect_error(
    cie10_sql("UPDATE cie10 SET codigo = 'X' WHERE codigo = 'E11.0'"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea queries DELETE", {
  expect_error(
    cie10_sql("DELETE FROM cie10 WHERE codigo = 'E11.0'"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea queries INSERT", {
  expect_error(
    cie10_sql("INSERT INTO cie10 (codigo) VALUES ('TEST')"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea queries ALTER", {
  expect_error(
    cie10_sql("ALTER TABLE cie10 ADD COLUMN test TEXT"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql maneja queries con espacios", {
  skip_on_cran()

  # Query con espacios al inicio
  resultado <- cie10_sql("   SELECT COUNT(*) as n FROM cie10")
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie10_sql maneja queries complejas", {
  skip_on_cran()

  # Query con JOIN, GROUP BY, ORDER BY
  query <- "
    SELECT capitulo, COUNT(*) as n
    FROM cie10
    WHERE codigo LIKE 'E%'
    GROUP BY capitulo
    ORDER BY n DESC
    LIMIT 5
  "

  resultado <- cie10_sql(query)
  expect_s3_class(resultado, "tbl_df")
  expect_true("capitulo" %in% names(resultado))
  expect_true("n" %in% names(resultado))
})

test_that("cie10_sql maneja queries con LIKE", {
  skip_on_cran()

  resultado <- cie10_sql("SELECT * FROM cie10 WHERE codigo LIKE 'E11%' LIMIT 5")
  expect_s3_class(resultado, "tbl_df")
  expect_true(all(stringr::str_starts(resultado$codigo, "E11")))
})

test_that("cie10_sql bloquea keywords peligrosos", {
  # DROP (detectado como keyword peligroso)
  expect_error(
    cie10_sql("SELECT * FROM cie10; DROP TABLE cie10;--"),
    "keyword no permitido"
  )

  # Multiples statements sin keyword peligroso
  expect_error(
    cie10_sql("SELECT * FROM cie10; SELECT * FROM cie10"),
    "Multiples statements"
  )

  # ATTACH (SQLite specific attack)
  expect_error(
    cie10_sql("SELECT * FROM cie10 WHERE 1=1 ATTACH DATABASE"),
    "keyword no permitido"
  )

  # PRAGMA (SQLite metadata)
  expect_error(
    cie10_sql("SELECT * FROM cie10 WHERE 1=1 PRAGMA table_info"),
    "keyword no permitido"
  )
})

test_that("cie10_sql permite queries legitimas con strings especiales", {
  skip_on_cran()

  # Query con string que contiene ;
  resultado <- cie10_sql("SELECT * FROM cie10 WHERE descripcion LIKE '%a;b%' LIMIT 1")
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_normalizar convierte formatos correctamente", {
  skip_on_cran()
  
  # Con punto (ya normalizado)
  expect_equal(cie_normalizar("E11.0", buscar_db = FALSE), "E11.0")
  
  # Sin punto (agrega punto)
  expect_equal(cie_normalizar("E110", buscar_db = FALSE), "E11.0")
  
  # Con sufijo X en codigos cortos (elimina X)
  expect_equal(cie_normalizar("I10X", buscar_db = FALSE), "I10")
  expect_equal(cie_normalizar("J00X", buscar_db = FALSE), "J00")
  expect_equal(cie_normalizar("B54X", buscar_db = FALSE), "B54")
  expect_equal(cie_normalizar("i10x", buscar_db = FALSE), "I10")  # minusculas
  
  # Codigos largos con X placeholder (preserva X)
  # En trauma/lesiones, X es placeholder obligatorio del 7o caracter
  expect_equal(cie_normalizar("S72X01A", buscar_db = FALSE), "S72X01A")
  expect_equal(cie_normalizar("T84X0XA", buscar_db = FALSE), "T84X0XA")
  
  # Vectorizado
  codigos <- c("E11.0", "I10.0", "Z00")
  resultado <- cie_normalizar(codigos, buscar_db = FALSE)
  expect_equal(resultado, c("E11.0", "I10.0", "Z00"))
  
  # Vectorizado con X (cortos y largos)
  codigos_x <- c("I10X", "J00X", "E110", "S72X01A")
  resultado_x <- cie_normalizar(codigos_x, buscar_db = FALSE)
  expect_equal(resultado_x, c("I10", "J00", "E11.0", "S72X01A"))
})

test_that("cie_normalizar maneja caracteres especiales", {
  skip_on_cran()
  
  # Espacios internos (error comun de captura)
  expect_equal(cie_normalizar("E 11 0", buscar_db = FALSE), "E11.0")
  expect_equal(cie_normalizar("I 10", buscar_db = FALSE), "I10")
  expect_equal(cie_normalizar(" E11.0 ", buscar_db = FALSE), "E11.0")  # trim
  
  # Guiones en lugar de puntos
  expect_equal(cie_normalizar("I10-0", buscar_db = FALSE), "I10.0")
  expect_equal(cie_normalizar("E11-0", buscar_db = FALSE), "E11.0")
  
  # Puntos multiples consecutivos
  expect_equal(cie_normalizar("E..11.0", buscar_db = FALSE), "E.11.0")
  expect_equal(cie_normalizar("I10..0", buscar_db = FALSE), "I10.0")
  
  # Puntos iniciales
  expect_equal(cie_normalizar(".I10", buscar_db = FALSE), "I10")
  expect_equal(cie_normalizar("..E11.0", buscar_db = FALSE), "E11.0")
  
  # Sistema daga/asterisco (codificacion dual CIE-10)
  expect_equal(cie_normalizar("A17.0\u2020", buscar_db = FALSE), "A17.0")  # daga
  expect_equal(cie_normalizar("G01*", buscar_db = FALSE), "G01")           # asterisco
  expect_equal(cie_normalizar("A52.1\u2020", buscar_db = FALSE), "A52.1")  # parkinsonismo sifilitico
  
  # Vectorizado con caracteres especiales
  codigos_especiales <- c("E 11 0", "I10-0", "A17.0\u2020", "G01*")
  resultado <- cie_normalizar(codigos_especiales, buscar_db = FALSE)
  expect_equal(resultado, c("E11.0", "I10.0", "A17.0", "G01"))
})

test_that("cie_validate_vector detecta formatos invalidos", {
  validos <- cie_validate_vector(c("E11.0", "Z00", "INVALIDO"))
  expect_equal(validos, c(TRUE, TRUE, FALSE))
})

test_that("cie_validate_vector acepta codigos con sufijo X DEIS", {
  expect_true(cie_validate_vector("I10X"))
  expect_true(cie_validate_vector("N10X"))
  expect_equal(cie_validate_vector(c("I10X", "N40X", "INVALIDO")),
               c(TRUE, TRUE, FALSE))
})

test_that("cie_expand genera hijos correctos", {
  skip_on_cran()

  hijos <- cie_expand("E11")
  expect_true(length(hijos) > 0)
  expect_true(all(stringr::str_starts(hijos, "E11")))
})

# ============================================================
# PRUEBAS ADICIONALES cie_normalizar() con buscar_db=TRUE
# ============================================================

test_that("cie_normalizar con buscar_db=TRUE verifica existencia", {
  skip_on_cran()

  # Codigo que existe en la base
  resultado <- cie_normalizar("E11.0", buscar_db = TRUE)
  expect_equal(resultado, "E11.0")

  # Codigo de categoria (3 digitos)
  resultado_cat <- cie_normalizar("E11", buscar_db = TRUE)
  # Puede agregar 0 si E110 existe, o mantenerse E11
  expect_true(resultado_cat %in% c("E11", "E110"))
})

test_that("cie_normalizar con buscar_db=TRUE es vectorizado", {
  skip_on_cran()

  codigos <- c("E11.0", "I10", "Z00")
  resultado <- cie_normalizar(codigos, buscar_db = TRUE)

  expect_length(resultado, 3)
  expect_true("E11.0" %in% resultado)
})

test_that("cie_normalizar con buscar_db=TRUE maneja NAs", {
  skip_on_cran()

  codigos <- c(NA, "E11.0")

  resultado <- cie_normalizar(codigos, buscar_db = TRUE)
  expect_length(resultado, 2)
  expect_true(is.na(resultado[1]))
  expect_equal(resultado[2], "E11.0")
})


test_that("cie_normalizar maneja codigo inexistente con buscar_db=TRUE", {
  skip_on_cran()

  # Codigo que no existe (retorna el codigo normalizado)
  resultado <- cie_normalizar("X99.9", buscar_db = TRUE)
  expect_equal(resultado, "X99.9")
})

# ============================================================
# PRUEBAS ADICIONALES cie_validate_vector() con strict=TRUE
# ============================================================

test_that("cie_validate_vector con strict=TRUE verifica existencia en DB", {
  skip_on_cran()

  # Codigos validos e invalidos
  codigos <- c("E11.0", "I10", "X99.9", "INVALIDO")

  expect_warning(
    resultado <- cie_validate_vector(codigos, strict = TRUE),
    "no encontrados"
  )

  # E11.0 e I10 deben estar en la DB
  expect_true(resultado[1])  # E11.0
  expect_true(resultado[2])  # I10
})

test_that("cie_validate_vector maneja NA con strict=TRUE", {
  skip_on_cran()

  codigos <- c("E11.0", NA, "I10")

  suppressWarnings({
    resultado <- cie_validate_vector(codigos, strict = TRUE)
  })

  expect_false(resultado[2])  # NA es FALSE
})

# ============================================================
# PRUEBAS ADICIONALES cie_expand()
# ============================================================

test_that("cie_expand maneja codigo vacio", {
  resultado <- cie_expand("")
  expect_length(resultado, 0)
})

test_that("cie_expand maneja NA", {
  resultado <- cie_expand(NA)
  expect_length(resultado, 0)
})

test_that("cie_expand maneja NULL", {
  resultado <- cie_expand(NULL)
  expect_length(resultado, 0)
})

test_that("cie_expand maneja codigo con subcategorias", {
  skip_on_cran()

  # E11 tiene subcategorias E11.0, E11.1, etc.
  hijos <- cie_expand("E11")
  expect_gt(length(hijos), 5)

  # Todos deben empezar con E11
  expect_true(all(grepl("^E11", hijos)))
})

# ============================================================
# PRUEBAS ADICIONALES COBERTURA - cie_normalizar()
# ============================================================

test_that("cie_normalizar maneja minusculas y mayusculas mezcladas", {
  skip_on_cran()

  expect_equal(cie_normalizar("e11.0", buscar_db = FALSE), "E11.0")
  expect_equal(cie_normalizar("E11.0", buscar_db = FALSE), "E11.0")
  expect_equal(cie_normalizar("e11", buscar_db = FALSE), "E11")
})

test_that("cie_normalizar maneja codigos con punto medio unicode", {
  skip_on_cran()

  # Punto medio (middot) a veces usado en sistemas
  # Unicode U+00B7 es eliminado por la funcion
  expect_equal(cie_normalizar("E11\u00B70", buscar_db = FALSE), "E11.0")
})

test_that("cie_normalizar maneja vector con NAs", {
  skip_on_cran()

  codigos <- c("E11.0", NA, "I10")
  resultado <- cie_normalizar(codigos, buscar_db = FALSE)

  expect_length(resultado, 3)
  expect_true(is.na(resultado[2]))
})

test_that("cie_normalizar buscar_db=TRUE con codigo de 3 digitos que tiene extension", {
  skip_on_cran()

  # I10 es codigo sin subcategorias directas
  resultado <- cie_normalizar("I10", buscar_db = TRUE)
  # Debe mantenerse como I10 o agregar 0 si existe I100
  expect_true(nchar(resultado) >= 3)
})

test_that("cie_normalizar buscar_db=TRUE preserva codigo ya normalizado", {
  skip_on_cran()

  # E11.0 existe en la base
  resultado <- cie_normalizar("E11.0", buscar_db = TRUE)
  expect_equal(resultado, "E11.0")
})

test_that("cie_normalizar buscar_db=TRUE con multiples codigos invalidos", {
  skip_on_cran()

  codigos <- c("XX9.9", "YY8.8", "ZZ7.7")
  resultado <- cie_normalizar(codigos, buscar_db = TRUE)

  # Debe retornar los codigos normalizados aunque no existan
  expect_length(resultado, 3)
})

# ============================================================
# PRUEBAS ADICIONALES COBERTURA - cie_validate_vector()
# ============================================================

test_that("cie_validate_vector maneja vector vacio", {
  resultado <- cie_validate_vector(character(0))
  expect_length(resultado, 0)
})

test_that("cie_validate_vector maneja solo NAs", {
  resultado <- cie_validate_vector(c(NA, NA, NA))
  expect_equal(resultado, c(FALSE, FALSE, FALSE))
})

test_that("cie_validate_vector strict=TRUE con todos codigos validos", {
  skip_on_cran()

  codigos <- c("E11.0", "I10")

  # No debe dar warning si todos existen
  resultado <- cie_validate_vector(codigos, strict = TRUE)

  expect_true(all(resultado))
})

test_that("cie_validate_vector strict=TRUE con mezcla validos e invalidos", {
  skip_on_cran()

  codigos <- c("E11.0", "XXXX", "I10", "YYYY")

  expect_warning(
    resultado <- cie_validate_vector(codigos, strict = TRUE),
    "no encontrados"
  )

  expect_equal(resultado[1], TRUE)   # E11.0 valido
  expect_equal(resultado[2], FALSE)  # XXXX invalido formato
  expect_equal(resultado[3], TRUE)   # I10 valido
  expect_equal(resultado[4], FALSE)  # YYYY invalido formato
})

test_that("cie_validate_vector acepta formato sin punto", {
  # E110 es formato valido MINSAL
  resultado <- cie_validate_vector(c("E110", "I100"))
  expect_equal(resultado, c(TRUE, TRUE))
})

test_that("cie_validate_vector rechaza formatos incorrectos", {
  # Formatos que no cumplen patron CIE-10
  codigos_invalidos <- c(
    "11E",      # letra al final
    "E1",       # muy corto
    "E111111",  # muy largo
    "123",      # solo numeros
    "ABCD"      # solo letras
  )

  resultado <- cie_validate_vector(codigos_invalidos)
  expect_true(all(resultado == FALSE))
})

# ============================================================
# PRUEBAS ADICIONALES COBERTURA - cie_expand()
# ============================================================

test_that("cie_expand con espacio en blanco", {
  resultado <- cie_expand("   ")
  expect_length(resultado, 0)
})

test_that("cie_expand con codigo especifico (E11.0)", {
  skip_on_cran()

  # E11.0 es codigo terminal, no tiene hijos
  hijos <- cie_expand("E11.0")

  # Puede retornar solo E11.0 o vacio
  expect_true(length(hijos) <= 1 || "E11.0" %in% hijos)
})

test_that("cie_expand con codigo de capitulo (A00-B99)", {
  skip_on_cran()

  # A00 es Colera, debe tener subcategorias
  hijos <- cie_expand("A00")

  # Debe tener al menos A00.0, A00.1, A00.9
  expect_gt(length(hijos), 0)
})

test_that("cie_expand retorna character vector", {
  skip_on_cran()

  hijos <- cie_expand("E11")

  expect_type(hijos, "character")
})

# ============================================================
# PRUEBAS DE EDGE CASES ADICIONALES
# ============================================================

test_that("cie_normalizar maneja codigo con espacios multiples", {
  skip_on_cran()

  expect_equal(cie_normalizar("E  11  0", buscar_db = FALSE), "E11.0")
  expect_equal(cie_normalizar("  E11.0  ", buscar_db = FALSE), "E11.0")
})

test_that("cie_normalizar maneja combinacion de caracteres especiales", {
  skip_on_cran()

  # Daga + espacio + guion
  expect_equal(cie_normalizar("A17\u2020 0", buscar_db = FALSE), "A17.0")

  # Asterisco + punto inicial
  expect_equal(cie_normalizar(".G01*", buscar_db = FALSE), "G01")
})

test_that("cie_validate_vector strict=FALSE acepta formato correcto sin DB", {
  # No debe conectar a DB con strict=FALSE
  codigos <- c("E11.0", "I10.0", "Z00")
  resultado <- cie_validate_vector(codigos, strict = FALSE)

  expect_equal(resultado, c(TRUE, TRUE, TRUE))
})

test_that("cie_normalizar con codigos de trauma largos", {
  skip_on_cran()

  # Codigos de lesiones/trauma pueden ser mas largos
  codigos_trauma <- c("S72.001A", "T84.50XA")

  # La funcion debe manejarlos sin error
  resultado <- cie_normalizar(codigos_trauma, buscar_db = FALSE)
  expect_length(resultado, 2)
})

# ============================================================
# PRUEBAS COBERTURA LINEAS 90-94: Extension cod_con_0 para codigos 3 digitos
# ============================================================

test_that("cie_normalizar buscar_db=TRUE extiende codigo 3 digitos con 0", {
  skip_on_cran()

  # E10 es categoria diabetes tipo 1
  # Si E10 no existe pero E100 si, debe retornar E100
  # O si E10 existe, retorna E10
  resultado <- cie_normalizar("E10", buscar_db = TRUE)

  # El resultado debe ser E10 o E100 (si se extendio)
  expect_true(resultado %in% c("E10", "E100"))
  expect_true(nchar(resultado) >= 3)
})

test_that("cie_normalizar buscar_db=TRUE con codigo 3 digitos que no tiene extension", {
  skip_on_cran()

  # Codigo de 3 digitos que existe en la DB
  # I10 es Hipertension esencial (codigo sin subcategorias directas numericas)
  resultado <- cie_normalizar("I10", buscar_db = TRUE)

  # Debe mantenerse I10 ya que existe en la DB
  expect_equal(resultado, "I10")
})

test_that("cie_normalizar buscar_db=TRUE con varios codigos 3 digitos", {
  skip_on_cran()

  # Mezcla de codigos de 3 digitos
  codigos <- c("E10", "E11", "I10", "J00")
  resultado <- cie_normalizar(codigos, buscar_db = TRUE)

  expect_length(resultado, 4)
  # Todos deben empezar con la letra original
  expect_true(all(substr(resultado, 1, 1) == c("E", "E", "I", "J")))
})

test_that("cie_normalizar buscar_db=TRUE codigo 3 digitos inexistente no se extiende", {
  skip_on_cran()

  # Codigo que no existe en la base y tampoco existe con 0
  resultado <- cie_normalizar("X99", buscar_db = TRUE)

  # Debe retornar X99 ya que ni X99 ni X990 existen
  expect_equal(resultado, "X99")
})

test_that("cie_normalizar buscar_db=TRUE ejercita rama cod_con_0", {
  skip_on_cran()

  # Buscar un codigo de 3 digitos donde el codigo base no existe

  # pero la extension con 0 podria existir
  # A09 (Diarrea y gastroenteritis) puede tener A090
  resultado <- cie_normalizar("A09", buscar_db = TRUE)

  # El resultado debe ser valido (A09 o A090)
  expect_true(nchar(resultado) >= 3)
  expect_true(grepl("^A09", resultado))
})

#' Generar tabla HTML interactiva GT de codigo CIE-10
#'
#' @param codigo String codigo (ej. "E11" muestra jerarquia)
#' @return Objeto de clase \code{gt_tbl} (tabla HTML interactiva)
#' @family visualizacion
#' @seealso \code{\link{cie_search}}, \code{\link{cie_lookup}}
#' @export
#' @importFrom dplyr select everything
#' @examples
#' # Requiere paquete gt
#' requireNamespace("gt", quietly = TRUE)
#'
#' \donttest{
#' cie_table("E11")  # Diabetes tipo 2 completo
#' }
cie_table <- function(codigo) {
  # Verificar que gt este instalado
  if (!requireNamespace("gt", quietly = TRUE)) {
    stop("El paquete 'gt' es necesario para esta funci\u00f3n.\n",
         "Inst\u00e1lalo con: install.packages('gt')")
  }
  
  datos <- cie_lookup(codigo, expandir = TRUE)
  
  if (nrow(datos) == 0) {
    stop("Codigo no encontrado: ", codigo)
  }
  
  tabla <- datos %>%
    dplyr::select(codigo, descripcion, inclusion, exclusion) %>%
    gt::gt() %>%
    gt::tab_header(
      title = sprintf("CIE-10 Chile: %s", codigo),
      subtitle = "Fuente: MINSAL/DEIS v2018"
    ) %>%
    gt::cols_label(
      codigo = "Codigo",
      descripcion = "Diagnostico",
      inclusion = "Incluye",
      exclusion = "Excluye"
    ) %>%
    gt::fmt_markdown(columns = dplyr::everything()) %>%
    gt::tab_options(
      table.font.size = 12,
      heading.background.color = "#1f77b4"
    )
  
  return(tabla)
}

# Tests para cie_table (tablas gt)

test_that("cie_table requiere gt instalado",
{
  skip_if_not_installed("gt")

  # Test basico con codigo valido
  tabla <- cie_table("E11")
  expect_s3_class(tabla, "gt_tbl")
})

test_that("cie_table funciona con multiples codigos", {
  skip_if_not_installed("gt")
  skip_on_cran()

  tabla <- cie_table(c("E11", "I10"))
  expect_s3_class(tabla, "gt_tbl")
})

test_that("cie_table maneja codigo invalido", {
  skip_if_not_installed("gt")
  skip_on_cran()

  # Codigo que no existe lanza error
  expect_error(cie_table("XXXXX"), "no encontrado")
})

# ============================================================
# PRUEBAS ADICIONALES cie_table()
# ============================================================

test_that("cie_table genera tabla gt correctamente", {
  skip_if_not_installed("gt")
  skip_on_cran()

  tabla <- cie_table("E11")
  expect_s3_class(tabla, "gt_tbl")
})

test_that("cie_table genera header correcto", {
  skip_if_not_installed("gt")
  skip_on_cran()

  tabla <- cie_table("E11")

  # Verificar que es objeto gt
  expect_s3_class(tabla, "gt_tbl")

  # gt tables tienen estructura interna que podemos verificar
  expect_true("_heading" %in% names(tabla))
})

test_that("cie_table funciona con codigo categoria", {
  skip_if_not_installed("gt")
  skip_on_cran()

  # Categoria sin punto (expande a todos los hijos)
  tabla <- cie_table("I10")
  expect_s3_class(tabla, "gt_tbl")
})

test_that("cie_table funciona con codigo especifico", {
  skip_if_not_installed("gt")
  skip_on_cran()

  # Codigo especifico con punto
  tabla <- cie_table("E11.0")
  expect_s3_class(tabla, "gt_tbl")
})

test_that("cie_table expande jerarquia automaticamente", {
  skip_if_not_installed("gt")
  skip_on_cran()

  # E11 debe expandir a E11.0, E11.1, etc.
  tabla <- cie_table("E11")

  # Verificar que tiene datos (no solo header)
  # El objeto gt tiene _data interno
  expect_true("_data" %in% names(tabla))
  expect_gt(nrow(tabla$`_data`), 1)
})

test_that("cie_table tiene columnas esperadas", {
  skip_if_not_installed("gt")
  skip_on_cran()

  tabla <- cie_table("E11.0")

  # Verificar columnas en los datos internos
  columnas_data <- names(tabla$`_data`)

  expect_true("codigo" %in% columnas_data)
  expect_true("descripcion" %in% columnas_data)
})

test_that("cie_table error sin gt instalado", {
  # Este test solo funciona si gt NO esta instalado
  skip_if(requireNamespace("gt", quietly = TRUE),
          "gt esta instalado")

  expect_error(
    cie_table("E11"),
    "gt"
  )
})

#' @importFrom dplyr filter pull rowwise %>%
#' @importFrom stringr str_detect
#' @importFrom tibble tibble as_tibble tribble
NULL

#' Calcular comorbilidades Charlson/Elixhauser para Chile
#'
#' @param data data.frame con columnas id paciente + codigos CIE-10
#' @param id String nombre columna identificador paciente
#' @param code String nombre columna con codigos CIE-10 (uno por fila)
#' @param map Character, esquema comorbilidad ("charlson" o "elixhauser")
#' @param assign0 Logical, asignar 0 si sin comorbilidad (default TRUE)
#' @return data.frame ancho con scores comorbilidad por paciente
#' @family comorbilidades
#' @seealso \code{\link{cie_map_comorbid}}, \code{\link{cie_normalizar}}
#' @export
#' @examples
#' # Ver documentacion de parametros
#' args(cie_comorbid)
#'
#' \donttest{
#' df <- data.frame(
#'   id_pac = c(1, 1, 2, 2),
#'   diag = c("E11.0", "I21.0", "C50.9", "E10.9")
#' )
#' cie_comorbid(df, id = "id_pac", code = "diag", map = "charlson")
#' }
cie_comorbid <- function(data, id, code, map = c("charlson", "elixhauser"), 
                         assign0 = TRUE) {
  # Verificar que comorbidity este instalado
  if (!requireNamespace("comorbidity", quietly = TRUE)) {
    stop("El paquete 'comorbidity' es necesario para esta funci\u00f3n.\n",
         "Inst\u00e1lalo con: install.packages('comorbidity')")
  }
  
  map <- match.arg(map)

  # Validar columnas existen
  if (!id %in% names(data) || !code %in% names(data)) {
    stop("Columnas '", id, "' o '", code, "' no existen en data")
  }

  # Advertir sobre NAs en columna de codigos
  n_na <- sum(is.na(data[[code]]))
  if (n_na > 0) {
    warning("Columna '", code, "' contiene ", n_na,
            " valores NA que seran ignorados")
    data <- data[!is.na(data[[code]]), ]
  }

  # Advertir sobre codigos vacios
  n_empty <- sum(nchar(trimws(data[[code]])) == 0)
  if (n_empty > 0) {
    warning("Columna '", code, "' contiene ", n_empty,
            " valores vacios que seran ignorados")
    data <- data[nchar(trimws(data[[code]])) > 0, ]
  }

  # Normalizar codigos (elimina sufijo X DEIS, agrega punto, etc.)
  data[[code]] <- cie_normalizar(data[[code]], buscar_db = FALSE)

  # Mapear a nomenclatura comorbidity package
  map_full <- switch(map,
    "charlson" = "charlson_icd10_quan",
    "elixhauser" = "elixhauser_icd10_quan"
  )
  
  # Mapeo Charlson adaptado Chile (usa comorbidity::comorbidity)
  # Nota: version mas reciente de comorbidity no requiere argumento 'icd'
  resultado <- comorbidity::comorbidity(
    x = data,
    id = id,
    code = code,
    map = map_full,
    assign0 = assign0,
    labelled = FALSE
  )
  
  # Score total Charlson (si aplica)
  if (map == "charlson") {
    resultado$score_charlson <- comorbidity::score(
      resultado,
      weights = "charlson",
      assign0 = assign0
    )
  }
  
  return(tibble::as_tibble(resultado))
}

#' Mapeo manual grupos comorbilidad Chile-especifico
#'
#' @description
#' Agrupa codigos CIE-10 chilenos en categorias comorbilidad MINSAL.
#' Basado en Decreto 1301/2016 MINSAL + icd::icd10_map_charlson.
#' 
#' @param codigos Character vector codigos CIE-10
#' @return tibble con codigo + categoria_comorbilidad
#' @family comorbilidades
#' @seealso \code{\link{cie_comorbid}}, \code{\link{cie_normalizar}}
#' @export
#' @examples
#' cie_map_comorbid(c("E11.0", "I50.9", "C50.9"))
cie_map_comorbid <- function(codigos) {
  # Manejar vector vacio
  if (length(codigos) == 0) {
    return(tibble::tibble(
      codigo = character(0),
      categoria = character(0)
    ))
  }

  # Categorizacion vectorizada con case_when
  resultado <- tibble::tibble(
    codigo = codigos,
    categoria = dplyr::case_when(
      is.na(codigos) ~ "Otra",
      stringr::str_detect(codigos, "^E10|^E11") ~ "Diabetes",
      stringr::str_detect(codigos, "^I50") ~ "Insuficiencia cardiaca",
      stringr::str_detect(codigos, "^I21|^I22") ~ "Infarto miocardio",
      stringr::str_detect(codigos, "^C[0-9]{2}") ~ "Neoplasia maligna",
      stringr::str_detect(codigos, "^J4[0-4]") ~ "EPOC",
      stringr::str_detect(codigos, "^N18") ~ "Enfermedad renal cronica",
      stringr::str_detect(codigos, "^F[0-9]{2}") ~ "Trastornos mentales",
      .default = "Otra"
    )
  )

  return(resultado)
}

# test-comorbid-validation.R
# Pruebas de validacion de comorbilidades con datos sinteticos

# ============================================================
# PRUEBAS cie_comorbid() - CHARLSON
# ============================================================

test_that("cie_comorbid calcula Charlson con datos sinteticos", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  # Datos sinteticos con comorbilidades conocidas
  df_charlson <- data.frame(
    id_paciente = c(1, 1, 1, 2, 2, 3, 3, 3, 3),
    codigo_cie = c(
      "E11.0", "I50.9", "N18.3",      # Pac 1: DM2, ICC, ERC
      "C50.9", "C78.0",               # Pac 2: Ca mama + metastasis
      "E10.0", "G20", "J44.1", "I21.0" # Pac 3: DM1, Parkinson, EPOC, IAM
    ),
    stringsAsFactors = FALSE
  )

  resultado <- cie_comorbid(df_charlson, id = "id_paciente",
                            code = "codigo_cie", map = "charlson")

  expect_s3_class(resultado, "tbl_df")
  expect_true("score_charlson" %in% names(resultado))
  expect_equal(nrow(resultado), 3, info = "Debe haber 3 pacientes")

  # Verificar que scores sean >= 0
  expect_true(all(resultado$score_charlson >= 0))
})

test_that("cie_comorbid calcula Elixhauser", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df_elix <- data.frame(
    id = c(1, 1, 2, 2),
    dx = c("E11.0", "I10", "F32.0", "J45.0"),
    stringsAsFactors = FALSE
  )

  resultado <- cie_comorbid(df_elix, id = "id", code = "dx", map = "elixhauser")

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 2)

  # Elixhauser no tiene score_charlson
  expect_false("score_charlson" %in% names(resultado))
})

test_that("cie_comorbid maneja NA en codigos", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df_na <- data.frame(
    id = c(1, 1, 2, 2),
    dx = c("E11.0", NA, "I10", NA),
    stringsAsFactors = FALSE
  )

  # Debe emitir warning por NAs
  expect_warning(
    resultado <- cie_comorbid(df_na, id = "id", code = "dx", map = "charlson"),
    "NA"
  )

  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid maneja codigos vacios", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df_empty <- data.frame(
    id = c(1, 1, 2),
    dx = c("E11.0", "", "I10"),
    stringsAsFactors = FALSE
  )

  # Debe emitir warning por vacios
  expect_warning(
    resultado <- cie_comorbid(df_empty, id = "id", code = "dx", map = "charlson"),
    "vacios"
  )

  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid error con columnas inexistentes", {
  df <- data.frame(id = 1, dx = "E11.0")

  expect_error(
    cie_comorbid(df, id = "paciente", code = "dx", map = "charlson"),
    "no existen"
  )

  expect_error(
    cie_comorbid(df, id = "id", code = "codigo", map = "charlson"),
    "no existen"
  )
})

test_that("cie_comorbid usa assign0 correctamente", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 2),
    dx = c("E11.0", "Z00.0"),  # DM2 y codigo sin comorbilidad
    stringsAsFactors = FALSE
  )

  resultado <- cie_comorbid(df, id = "id", code = "dx",
                            map = "charlson", assign0 = TRUE)

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 2)
})

# ============================================================
# PRUEBAS cie_map_comorbid()
# ============================================================

test_that("cie_map_comorbid categoriza diabetes correctamente", {
  # Solo E10 y E11 estan en el mapeo de cie_map_comorbid
  resultado <- cie_map_comorbid(c("E10.0", "E11.0"))

  expect_equal(nrow(resultado), 2)
  expect_true(all(resultado$categoria == "Diabetes"))
})

test_that("cie_map_comorbid categoriza cardiovascular", {
  codigos <- c("I50.0", "I50.9", "I21.0", "I22.1")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 4)
  # I50 = ICC, I21/I22 = IAM
  expect_true(resultado$categoria[1] == "Insuficiencia cardiaca")
  expect_true(resultado$categoria[3] == "Infarto miocardio")
})

test_that("cie_map_comorbid categoriza neoplasias", {
  codigos <- c("C50.0", "C61", "C34.0", "C18.0")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 4)
  expect_true(all(resultado$categoria == "Neoplasia maligna"))
})

test_that("cie_map_comorbid categoriza respiratorio", {
  codigos <- c("J40", "J44.0", "J44.1")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 3)
  expect_true(all(resultado$categoria == "EPOC"))
})

test_that("cie_map_comorbid categoriza ERC", {
  codigos <- c("N18.1", "N18.3", "N18.5", "N18.9")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 4)
  expect_true(all(resultado$categoria == "Enfermedad renal cronica"))
})

test_that("cie_map_comorbid categoriza trastornos mentales", {
  codigos <- c("F32.0", "F41.0", "F20.0")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 3)
  expect_true(all(resultado$categoria == "Trastornos mentales"))
})

test_that("cie_map_comorbid retorna 'Otra' para codigos no mapeados", {
  codigos <- c("Z00.0", "R10.0", "S62.0")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 3)
  expect_true(all(resultado$categoria == "Otra"))
})

test_that("cie_map_comorbid maneja NA", {
  codigos <- c("E11.0", NA, "I50.9")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 3)
  expect_equal(resultado$categoria[2], "Otra")
})

test_that("cie_map_comorbid maneja vector vacio", {
  resultado <- cie_map_comorbid(character(0))

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
  expect_equal(names(resultado), c("codigo", "categoria"))
})

test_that("cie_map_comorbid maneja codigos mixtos", {
  codigos <- c("E11.0", "I50.9", "C50.9", "J44.0", "N18.3", "Z00.0")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 6)

  # Verificar cada categoria
  expect_equal(resultado$categoria[resultado$codigo == "E11.0"], "Diabetes")
  expect_equal(resultado$categoria[resultado$codigo == "I50.9"], "Insuficiencia cardiaca")
  expect_equal(resultado$categoria[resultado$codigo == "C50.9"], "Neoplasia maligna")
  expect_equal(resultado$categoria[resultado$codigo == "J44.0"], "EPOC")
  expect_equal(resultado$categoria[resultado$codigo == "N18.3"], "Enfermedad renal cronica")
  expect_equal(resultado$categoria[resultado$codigo == "Z00.0"], "Otra")
})

# ============================================================
# PRUEBAS CON MULTIPLES COMORBILIDADES POR PACIENTE
# ============================================================

test_that("cie_comorbid maneja paciente con muchos codigos", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  # Paciente complejo con multiples comorbilidades
  df_complejo <- data.frame(
    id = rep(1, 10),
    dx = c("E11.0", "E11.2", "I10", "I50.0", "N18.3",
           "J44.1", "G20", "F32.0", "C50.9", "I21.0"),
    stringsAsFactors = FALSE
  )

  resultado <- cie_comorbid(df_complejo, id = "id", code = "dx", map = "charlson")

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 1)
  expect_gt(resultado$score_charlson, 0)
})

test_that("cie_comorbid distingue pacientes correctamente", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df_multi <- data.frame(
    id = c(1, 1, 2, 2, 3, 3),
    dx = c("E11.0", "I50.0",   # Pac 1: DM + ICC
           "C50.9", "C78.0",   # Pac 2: Cancer + metastasis
           "Z00.0", "Z01.0"),  # Pac 3: Sin comorbilidades
    stringsAsFactors = FALSE
  )

  resultado <- cie_comorbid(df_multi, id = "id", code = "dx", map = "charlson")

  expect_equal(nrow(resultado), 3)

  # Pac 3 deberia tener score bajo o 0
  score_pac3 <- resultado$score_charlson[resultado$id == 3]
  score_pac2 <- resultado$score_charlson[resultado$id == 2]

  # Cancer con metastasis deberia tener score alto
  expect_gte(score_pac2, score_pac3)
})

# ============================================================
# PRUEBAS DE CONSISTENCIA ENTRE FUNCIONES
# ============================================================

test_that("cie_map_comorbid y cie_comorbid son consistentes", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  codigos <- c("E11.0", "I21.0", "C50.9")

  # Mapeo manual
  map_resultado <- cie_map_comorbid(codigos)

  # Las categorias de mapeo deben existir
  expect_true("Diabetes" %in% map_resultado$categoria)
  expect_true("Infarto miocardio" %in% map_resultado$categoria)
  expect_true("Neoplasia maligna" %in% map_resultado$categoria)
})

test_that("cie_comorbid retorna columnas consistentes", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(id = 1, dx = "E11.0", stringsAsFactors = FALSE)

  charlson <- cie_comorbid(df, id = "id", code = "dx", map = "charlson")
  elix <- cie_comorbid(df, id = "id", code = "dx", map = "elixhauser")

  # Ambos deben tener columna id
  expect_true("id" %in% names(charlson))
  expect_true("id" %in% names(elix))

  # Solo Charlson tiene score_charlson
  expect_true("score_charlson" %in% names(charlson))
  expect_false("score_charlson" %in% names(elix))
})

# ============================================================
# PRUEBAS cie_comorbid()
# ============================================================

# ------------------------------------------------------------------------------
# Pruebas basicas de Charlson
# ------------------------------------------------------------------------------

test_that("cie_comorbid calcula Charlson", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 2),
    diag = c("E11.0", "I50.9", "C50.9")
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  expect_true("score_charlson" %in% names(resultado))
})

test_that("cie_comorbid Charlson con diabetes tipo 2", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "E11.0"  # Diabetes mellitus tipo 2
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  expect_true("score_charlson" %in% names(resultado))
  # Diabetes sin complicaciones = 1 punto en Charlson
  expect_gte(resultado$score_charlson[1], 0)
})

test_that("cie_comorbid Charlson con diabetes tipo 1", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "E10.9"  # Diabetes mellitus tipo 1
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  expect_true("score_charlson" %in% names(resultado))
})

test_that("cie_comorbid Charlson con infarto miocardio", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "I21.0"  # Infarto agudo miocardio
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  expect_true("score_charlson" %in% names(resultado))
})

test_that("cie_comorbid Charlson con cancer", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 2),
    diag = c("C50.9", "C34.9")  # Cancer mama, cancer pulmon
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 2)
})

test_that("cie_comorbid Charlson con insuficiencia cardiaca", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "I50.9"  # Insuficiencia cardiaca
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
})

# ------------------------------------------------------------------------------
# Pruebas basicas de Elixhauser
# ------------------------------------------------------------------------------

test_that("cie_comorbid calcula Elixhauser", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 2),
    diag = c("E11.0", "I50.9", "J44.9")
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "elixhauser")
  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 2)
})

test_that("cie_comorbid Elixhauser con hipertension", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "I10"  # Hipertension esencial
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "elixhauser")
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid Elixhauser con EPOC", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "J44.9"  # EPOC
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "elixhauser")
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid Elixhauser con obesidad", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "E66.9"  # Obesidad
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "elixhauser")
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid Elixhauser con depresion", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "F32.9"  # Episodio depresivo
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "elixhauser")
  expect_s3_class(resultado, "tbl_df")
})

# ------------------------------------------------------------------------------
# Pruebas de manejo de NA y vectores vacios
# ------------------------------------------------------------------------------

test_that("cie_comorbid maneja NA en codigos", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 2),
    diag = c("E11.0", NA, "I50.9")
  )

  expect_warning(
    resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson"),
    "NA"
  )
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid maneja strings vacios", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 2),
    diag = c("E11.0", "", "I50.9")
  )

  expect_warning(
    resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson"),
    "vacios"
  )
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid maneja espacios en blanco", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 2),
    diag = c("E11.0", "   ", "I50.9")
  )

  expect_warning(
    resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson"),
    "vacios"
  )
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid maneja multiples NA", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 1, 2),
    diag = c("E11.0", NA, NA, "I50.9")
  )

  expect_warning(
    resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson"),
    "2 valores NA"
  )
  expect_s3_class(resultado, "tbl_df")
})

# ------------------------------------------------------------------------------
# Pruebas de codigos invalidos y no-CIE
# ------------------------------------------------------------------------------

test_that("cie_comorbid con codigos no reconocidos", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1),
    diag = c("E11.0", "XXXXX")  # Codigo invalido
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid con mezcla validos e invalidos", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 1, 2, 2),
    diag = c("E11.0", "INVALIDO", "I50.9", "C50.9", "NOVALIDO")
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 2)
})

test_that("cie_comorbid con solo codigos invalidos", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 2),
    diag = c("INVALIDO1", "INVALIDO2")
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  # Score deberia ser 0 para codigos no reconocidos
  expect_true(all(resultado$score_charlson == 0))
})

test_that("cie_comorbid con numeros como codigos", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 2),
    diag = c("12345", "99999")
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
})

# ------------------------------------------------------------------------------
# Pruebas de edge cases
# ------------------------------------------------------------------------------

test_that("cie_comorbid con un solo codigo", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "E11.0"
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 1)
})

test_that("cie_comorbid con un solo paciente multiples diagnosticos", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = rep(1, 10),
    diag = c("E11.0", "I50.9", "C50.9", "J44.9", "I10",
             "N18.9", "F32.9", "E66.9", "I21.0", "K70.3")
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 1)
  expect_gt(resultado$score_charlson[1], 0)
})

test_that("cie_comorbid con muchos pacientes", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  # Crear dataset con 100 pacientes
  n_pacientes <- 100
  df <- data.frame(
    id = rep(1:n_pacientes, each = 3),
    diag = rep(c("E11.0", "I50.9", "C50.9"), n_pacientes)
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), n_pacientes)
})

test_that("cie_comorbid con codigos duplicados mismo paciente", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 1),
    diag = c("E11.0", "E11.0", "E11.0")  # Mismo codigo repetido
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 1)
})

# ------------------------------------------------------------------------------
# Pruebas de scores conocidos
# ------------------------------------------------------------------------------

test_that("cie_comorbid score sin comorbilidades es 0", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "Z00.0"  # Examen general (no es comorbilidad)
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_equal(resultado$score_charlson[1], 0)
})

test_that("cie_comorbid con SIDA tiene score alto", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "B24"  # VIH/SIDA
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  # SIDA = 6 puntos en Charlson original
  expect_gte(resultado$score_charlson[1], 0)
})

test_that("cie_comorbid pacientes con diferentes cargas comorbidas", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 2, 2, 2, 3, 3, 3, 3, 3),
    diag = c(
      "Z00.0",                    # Paciente 1: sin comorbilidades
      "E11.0", "I50.9", "J44.9",  # Paciente 2: 3 comorbilidades
      "E11.0", "I50.9", "C50.9", "N18.9", "B24"  # Paciente 3: 5 comorbilidades
    )
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  expect_equal(nrow(resultado), 3)

  # Paciente 3 deberia tener mayor score que paciente 2, y este mayor que 1
  expect_lte(resultado$score_charlson[1], resultado$score_charlson[2])
})

test_that("cie_comorbid suma correctamente comorbilidades multiples", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = c(1, 1, 1, 1),
    diag = c("E11.0", "I50.9", "C50.9", "J44.9")
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson")
  # Con 4 comorbilidades, score debe ser > 0
  expect_gt(resultado$score_charlson[1], 0)
})

# ------------------------------------------------------------------------------
# Pruebas de validacion de parametros
# ------------------------------------------------------------------------------

test_that("cie_comorbid error si columna id no existe", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    patient = 1,
    diag = "E11.0"
  )

  expect_error(
    cie_comorbid(df, id = "id", code = "diag", map = "charlson"),
    "no existen"
  )
})

test_that("cie_comorbid error si columna code no existe", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diagnosis = "E11.0"
  )

  expect_error(
    cie_comorbid(df, id = "id", code = "diag", map = "charlson"),
    "no existen"
  )
})

test_that("cie_comorbid error si comorbidity no instalado", {
  skip_on_cran()
  skip_if(requireNamespace("comorbidity", quietly = TRUE),
          "comorbidity esta instalado")

  df <- data.frame(
    id = 1,
    diag = "E11.0"
  )

  expect_error(
    cie_comorbid(df, id = "id", code = "diag", map = "charlson"),
    "comorbidity"
  )
})

test_that("cie_comorbid acepta nombres de columna personalizados", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    paciente_id = c(1, 1, 2),
    codigo_cie = c("E11.0", "I50.9", "C50.9")
  )

  resultado <- cie_comorbid(df, id = "paciente_id", code = "codigo_cie", map = "charlson")
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid assign0 = FALSE", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "E11.0"
  )

  resultado <- cie_comorbid(df, id = "id", code = "diag", map = "charlson", assign0 = FALSE)
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_comorbid map default es charlson", {
  skip_on_cran()
  skip_if_not_installed("comorbidity")

  df <- data.frame(
    id = 1,
    diag = "E11.0"
  )

  # Sin especificar map, debe usar charlson
  resultado <- cie_comorbid(df, id = "id", code = "diag")
  expect_true("score_charlson" %in% names(resultado))
})

# ============================================================
# PRUEBAS cie_map_comorbid()
# ============================================================

# ------------------------------------------------------------------------------
# Pruebas basicas de mapeo
# ------------------------------------------------------------------------------

test_that("cie_map_comorbid categoriza correctamente", {
  codigos <- c("E11.0", "I21.0", "INVALIDO")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 3)
  expect_equal(resultado$categoria[1], "Diabetes")
})

test_that("cie_map_comorbid retorna tibble", {
  codigos <- c("E11.0", "I50.9")
  resultado <- cie_map_comorbid(codigos)

  expect_s3_class(resultado, "tbl_df")
  expect_true("codigo" %in% names(resultado))
  expect_true("categoria" %in% names(resultado))
})

test_that("cie_map_comorbid maneja vector vacio", {
  resultado <- cie_map_comorbid(character(0))

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
  expect_true("codigo" %in% names(resultado))
  expect_true("categoria" %in% names(resultado))
})

# ------------------------------------------------------------------------------
# Pruebas de mapeo Charlson
# ------------------------------------------------------------------------------

test_that("cie_map_comorbid categoriza diabetes E10", {
  resultado <- cie_map_comorbid("E10.9")
  expect_equal(resultado$categoria[1], "Diabetes")
})

test_that("cie_map_comorbid categoriza diabetes E11", {
  resultado <- cie_map_comorbid("E11.0")
  expect_equal(resultado$categoria[1], "Diabetes")
})

test_that("cie_map_comorbid categoriza insuficiencia cardiaca", {
  resultado <- cie_map_comorbid("I50.9")
  expect_equal(resultado$categoria[1], "Insuficiencia cardiaca")
})

test_that("cie_map_comorbid categoriza infarto miocardio I21", {
  resultado <- cie_map_comorbid("I21.0")
  expect_equal(resultado$categoria[1], "Infarto miocardio")
})

test_that("cie_map_comorbid categoriza infarto miocardio I22", {
  resultado <- cie_map_comorbid("I22.0")
  expect_equal(resultado$categoria[1], "Infarto miocardio")
})

test_that("cie_map_comorbid categoriza neoplasia maligna", {
  codigos <- c("C50.9", "C34.9", "C18.9")
  resultado <- cie_map_comorbid(codigos)
  expect_true(all(resultado$categoria == "Neoplasia maligna"))
})

test_that("cie_map_comorbid categoriza EPOC J44", {
  resultado <- cie_map_comorbid("J44.9")
  expect_equal(resultado$categoria[1], "EPOC")
})

test_that("cie_map_comorbid categoriza EPOC J40", {
  resultado <- cie_map_comorbid("J40")
  expect_equal(resultado$categoria[1], "EPOC")
})

test_that("cie_map_comorbid categoriza EPOC J41 bronquitis cronica", {
  resultado <- cie_map_comorbid("J41.0")
  expect_equal(resultado$categoria[1], "EPOC")
})

test_that("cie_map_comorbid categoriza EPOC J42 bronquitis cronica NE", {
  resultado <- cie_map_comorbid("J42")
  expect_equal(resultado$categoria[1], "EPOC")
})

test_that("cie_map_comorbid categoriza EPOC J43 enfisema", {
  resultado <- cie_map_comorbid("J43.9")
  expect_equal(resultado$categoria[1], "EPOC")
})

test_that("cie_map_comorbid categoriza todo espectro EPOC J40-J44", {
  codigos <- c("J40", "J41.0", "J42", "J43.9", "J44.1")
  resultado <- cie_map_comorbid(codigos)
  expect_true(all(resultado$categoria == "EPOC"))
})

test_that("cie_map_comorbid categoriza enfermedad renal cronica", {
  resultado <- cie_map_comorbid("N18.9")
  expect_equal(resultado$categoria[1], "Enfermedad renal cronica")
})

test_that("cie_map_comorbid categoriza trastornos mentales", {
  codigos <- c("F32.9", "F41.9", "F20.0")
  resultado <- cie_map_comorbid(codigos)
  expect_true(all(resultado$categoria == "Trastornos mentales"))
})

# ------------------------------------------------------------------------------
# Pruebas de codigos sin categoria
# ------------------------------------------------------------------------------

test_that("cie_map_comorbid asigna Otra a codigos no mapeados", {
  resultado <- cie_map_comorbid("Z00.0")
  expect_equal(resultado$categoria[1], "Otra")
})

test_that("cie_map_comorbid asigna Otra a codigos invalidos", {
  resultado <- cie_map_comorbid("INVALIDO")
  expect_equal(resultado$categoria[1], "Otra")
})

test_that("cie_map_comorbid asigna Otra a NA", {
  resultado <- cie_map_comorbid(NA_character_)
  expect_equal(resultado$categoria[1], "Otra")
})

test_that("cie_map_comorbid mezcla categorias y Otra", {
  codigos <- c("E11.0", "Z00.0", "I50.9", "INVALIDO")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(resultado$categoria[1], "Diabetes")
  expect_equal(resultado$categoria[2], "Otra")
  expect_equal(resultado$categoria[3], "Insuficiencia cardiaca")
  expect_equal(resultado$categoria[4], "Otra")
})

# ------------------------------------------------------------------------------
# Pruebas de validacion de output
# ------------------------------------------------------------------------------

test_that("cie_map_comorbid output tiene columnas correctas", {
  resultado <- cie_map_comorbid(c("E11.0", "I50.9"))

  expect_equal(names(resultado), c("codigo", "categoria"))
  expect_type(resultado$codigo, "character")
  expect_type(resultado$categoria, "character")
})

test_that("cie_map_comorbid preserva orden de entrada", {
  codigos <- c("C50.9", "E11.0", "I50.9", "J44.9")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(resultado$codigo, codigos)
})

test_that("cie_map_comorbid con un solo codigo", {
  resultado <- cie_map_comorbid("E11.0")

  expect_equal(nrow(resultado), 1)
  expect_equal(resultado$codigo[1], "E11.0")
  expect_equal(resultado$categoria[1], "Diabetes")
})

test_that("cie_map_comorbid con muchos codigos", {
  codigos <- paste0("E11.", 0:99)
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 100)
  expect_true(all(resultado$categoria == "Diabetes"))
})

test_that("cie_map_comorbid con codigos repetidos", {
  codigos <- c("E11.0", "E11.0", "E11.0")
  resultado <- cie_map_comorbid(codigos)

  expect_equal(nrow(resultado), 3)
  expect_true(all(resultado$codigo == "E11.0"))
  expect_true(all(resultado$categoria == "Diabetes"))
})

# ------------------------------------------------------------------------------
# Pruebas adicionales
# ------------------------------------------------------------------------------

test_that("cie_map_comorbid es case-sensitive", {
  # Los codigos CIE-10 son mayusculas
  resultado <- cie_map_comorbid("e11.0")  # minuscula
  expect_equal(resultado$categoria[1], "Otra")  # No deberia matchear
})

test_that("cie_map_comorbid con codigos sin punto", {
  resultado <- cie_map_comorbid("E110")
  expect_equal(resultado$categoria[1], "Diabetes")
})

test_that("cie_map_comorbid con codigos de 3 caracteres", {
  resultado <- cie_map_comorbid(c("E11", "I50", "C50"))

  expect_equal(resultado$categoria[1], "Diabetes")
  expect_equal(resultado$categoria[2], "Insuficiencia cardiaca")
  expect_equal(resultado$categoria[3], "Neoplasia maligna")
})

#' @importFrom stringr str_trim str_replace_all str_detect str_extract str_split
#' @importFrom dplyr select mutate filter all_of any_of bind_rows distinct %>%
#' @importFrom tibble as_tibble
NULL

#' Normalizar texto removiendo tildes y caracteres especiales
#'
#' @param texto Character vector a normalizar
#' @return Character vector sin tildes ni caracteres especiales
#' @keywords internal
#' @noRd
normalizar_tildes <- function(texto) {
  # Manejar vector vacio o NA
  if (length(texto) == 0) return(character(0))

  # Usar chartr() que es mas rapido para sustituciones multiples
  # Caracteres con tilde -> sin tilde
  chartr(
    paste0(
      "\u00e1\u00e9\u00ed\u00f3\u00fa\u00fc\u00f1",
      "\u00c1\u00c9\u00cd\u00d3\u00da\u00dc\u00d1"
    ),
    "aeiouunAEIOUUN",
    texto
  )
}

#' Diccionario de siglas medicas comunes en Chile
#'
#' @return Named list con siglas como keys y terminos de busqueda como values
#' @keywords internal
#' @noRd
get_siglas_medicas <- function() {
  # Retorna lista con categoria incluida: list(sigla = list(termino, categoria))
  list(
    # Cardiovasculares
    "iam" = list(
      termino = "infarto agudo miocardio",
      categoria = "cardiovascular"
    ),
    "iamcest" = list(
      termino = "infarto agudo miocardio",
      categoria = "cardiovascular"
    ),
    "iamsest" = list(
      termino = "infarto agudo miocardio",
      categoria = "cardiovascular"
    ),
    "sca" = list(
      termino = "sindrome coronario agudo",
      categoria = "cardiovascular"
    ),
    "hta" = list(
      termino = "hipertension arterial",
      categoria = "cardiovascular"
    ),
    "aha" = list(
      termino = "hipertension arterial",
      categoria = "cardiovascular"
    ),
    "icc" = list(
      termino = "insuficiencia cardiaca",
      categoria = "cardiovascular"
    ),
    "ic" = list(
      termino = "insuficiencia cardiaca",
      categoria = "cardiovascular"
    ),
    "fa" = list(
      termino = "fibrilacion auricular",
      categoria = "cardiovascular"
    ),
    "tep" = list(
      termino = "embolia pulmonar",
      categoria = "cardiovascular"
    ),
    "tvp" = list(
      termino = "trombosis venosa profunda",
      categoria = "cardiovascular"
    ),
    "eap" = list(
      termino = "edema agudo pulmon",
      categoria = "cardiovascular"
    ),
    "acv" = list(
      termino = "accidente cerebrovascular",
      categoria = "cardiovascular"
    ),
    "ave" = list(
      termino = "accidente vascular encefalico",
      categoria = "cardiovascular"
    ),
    "ait" = list(
      termino = "isquemico transitorio",
      categoria = "cardiovascular"
    ),

    # Respiratorias
    "tbc" = list(
      termino = "tuberculosis",
      categoria = "respiratoria"
    ),
    "tb" = list(
      termino = "tuberculosis",
      categoria = "respiratoria"
    ),
    "epoc" = list(
      termino = "enfermedad pulmonar obstructiva cronica",
      categoria = "respiratoria"
    ),
    "asma" = list(
      termino = "asma",
      categoria = "respiratoria"
    ),
    "nac" = list(
      termino = "neumonia",
      categoria = "respiratoria"
    ),
    "ira" = list(
      termino = "infeccion respiratoria aguda",
      categoria = "respiratoria"
    ),
    "sdra" = list(
      termino = "sindrome distres respiratorio",
      categoria = "respiratoria"
    ),
    "covid" = list(
      termino = "covid",
      categoria = "respiratoria"
    ),
    "sars" = list(
      termino = "coronavirus",
      categoria = "respiratoria"
    ),

    # Metabolicas/Endocrinas
    "dm" = list(
      termino = "diabetes mellitus",
      categoria = "metabolica"
    ),
    "dm1" = list(
      termino = "diabetes mellitus tipo 1",
      categoria = "metabolica"
    ),
    "dm2" = list(
      termino = "diabetes mellitus tipo 2",
      categoria = "metabolica"
    ),
    "dbt" = list(
      termino = "diabetes",
      categoria = "metabolica"
    ),
    "hipo" = list(
      termino = "hipotiroidismo",
      categoria = "metabolica"
    ),
    "hiper" = list(
      termino = "hipertiroidismo",
      categoria = "metabolica"
    ),
    "erc" = list(
      termino = "enfermedad renal cronica",
      categoria = "metabolica"
    ),
    "irc" = list(
      termino = "insuficiencia renal cronica",
      categoria = "metabolica"
    ),
    "ira_renal" = list(
      termino = "insuficiencia renal aguda",
      categoria = "metabolica"
    ),
    "lra" = list(
      termino = "lesion renal aguda",
      categoria = "metabolica"
    ),

    # Gastrointestinales
    "hda" = list(
      termino = "hemorragia digestiva alta",
      categoria = "gastrointestinal"
    ),
    "hdb" = list(
      termino = "hemorragia digestiva baja",
      categoria = "gastrointestinal"
    ),
    "rge" = list(
      termino = "reflujo gastroesofagico",
      categoria = "gastrointestinal"
    ),
    "erge" = list(
      termino = "reflujo gastroesofagico",
      categoria = "gastrointestinal"
    ),
    "eii" = list(
      termino = "enfermedad inflamatoria intestinal",
      categoria = "gastrointestinal"
    ),
    "cu" = list(
      termino = "colitis ulcerosa",
      categoria = "gastrointestinal"
    ),
    "ec" = list(
      termino = "enfermedad crohn",
      categoria = "gastrointestinal"
    ),
    "dhc" = list(
      termino = "dano hepatico cronico",
      categoria = "gastrointestinal"
    ),
    "cirrosis" = list(
      termino = "cirrosis",
      categoria = "gastrointestinal"
    ),

    # Infecciosas
    "vih" = list(
      termino = "vih",
      categoria = "infecciosa"
    ),
    "sida" = list(
      termino = "sida",
      categoria = "infecciosa"
    ),
    "its" = list(
      termino = "infeccion transmision sexual",
      categoria = "infecciosa"
    ),
    "ets" = list(
      termino = "enfermedad transmision sexual",
      categoria = "infecciosa"
    ),
    "itu" = list(
      termino = "infeccion tracto urinario",
      categoria = "infecciosa"
    ),
    "ivu" = list(
      termino = "infeccion vias urinarias",
      categoria = "infecciosa"
    ),
    "meningitis" = list(
      termino = "meningitis",
      categoria = "infecciosa"
    ),
    "sepsis" = list(
      termino = "sepsis",
      categoria = "infecciosa"
    ),

    # Oncologicas
    "ca" = list(
      termino = "carcinoma",
      categoria = "oncologica"
    ),
    "neo" = list(
      termino = "neoplasia",
      categoria = "oncologica"
    ),
    "lma" = list(
      termino = "leucemia mieloide aguda",
      categoria = "oncologica"
    ),
    "lmc" = list(
      termino = "leucemia mieloide cronica",
      categoria = "oncologica"
    ),
    "lla" = list(
      termino = "leucemia linfoblastica aguda",
      categoria = "oncologica"
    ),
    "llc" = list(
      termino = "leucemia linfocitica cronica",
      categoria = "oncologica"
    ),
    "lnh" = list(
      termino = "linfoma no hodgkin",
      categoria = "oncologica"
    ),
    "lh" = list(
      termino = "linfoma hodgkin",
      categoria = "oncologica"
    ),
    "mm" = list(
      termino = "mieloma multiple",
      categoria = "oncologica"
    ),

    # Reumatologicas
    "ar" = list(
      termino = "artritis reumatoide",
      categoria = "reumatologica"
    ),
    "les" = list(
      termino = "lupus eritematoso",
      categoria = "reumatologica"
    ),
    "fm" = list(
      termino = "fibromialgia",
      categoria = "reumatologica"
    ),
    "ea" = list(
      termino = "espondilitis anquilosante",
      categoria = "reumatologica"
    ),

    # Neurologicas
    "epi" = list(
      termino = "epilepsia",
      categoria = "neurologica"
    ),
    "parkinson" = list(
      termino = "parkinson",
      categoria = "neurologica"
    ),
    "alzheimer" = list(
      termino = "alzheimer",
      categoria = "neurologica"
    ),
    "em" = list(
      termino = "esclerosis multiple",
      categoria = "neurologica"
    ),
    "ela" = list(
      termino = "esclerosis lateral amiotrofica",
      categoria = "neurologica"
    ),
    "cefalea" = list(
      termino = "cefalea",
      categoria = "neurologica"
    ),
    "migrana" = list(
      termino = "migrana",
      categoria = "neurologica"
    ),

    # Psiquiatricas
    "tdah" = list(
      termino = "deficit atencion hiperactividad",
      categoria = "psiquiatrica"
    ),
    "toc" = list(
      termino = "obsesivo compulsivo",
      categoria = "psiquiatrica"
    ),
    "tag" = list(
      termino = "ansiedad generalizada",
      categoria = "psiquiatrica"
    ),
    "tept" = list(
      termino = "estres postraumatico",
      categoria = "psiquiatrica"
    ),
    "edm" = list(
      termino = "depresion mayor",
      categoria = "psiquiatrica"
    ),
    "tab" = list(
      termino = "trastorno bipolar",
      categoria = "psiquiatrica"
    ),

    # Traumatologicas
    "tec" = list(
      termino = "traumatismo craneoencefalico",
      categoria = "traumatologica"
    ),
    "fx" = list(
      termino = "fractura",
      categoria = "traumatologica"
    ),
    "lca" = list(
      termino = "ligamento cruzado anterior",
      categoria = "traumatologica"
    ),

    # Pediatricas
    "sbo" = list(
      termino = "sindrome bronquial obstructivo",
      categoria = "pediatrica"
    ),
    "eda" = list(
      termino = "enfermedad diarreica aguda",
      categoria = "pediatrica"
    ),
    "gea" = list(
      termino = "gastroenteritis aguda",
      categoria = "pediatrica"
    ),

    # Gineco-obstetricias
    "sop" = list(
      termino = "sindrome ovario poliquistico",
      categoria = "gineco_obstetrica"
    ),
    "epi_gineco" = list(
      termino = "enfermedad pelvica inflamatoria",
      categoria = "gineco_obstetrica"
    ),
    "hie" = list(
      termino = "hipertension embarazo",
      categoria = "gineco_obstetrica"
    ),
    "pe" = list(
      termino = "preeclampsia",
      categoria = "gineco_obstetrica"
    ),
    "dpp" = list(
      termino = "desprendimiento prematuro placenta",
      categoria = "gineco_obstetrica"
    ),
    "rciu" = list(
      termino = "restriccion crecimiento intrauterino",
      categoria = "gineco_obstetrica"
    )
  )
}

#' Expandir siglas medicas a terminos de busqueda
#'
#' @param texto Texto que puede contener siglas
#' @return Texto con siglas expandidas o NULL si no es sigla
#' @keywords internal
#' @noRd
expandir_sigla <- function(texto) {
  siglas <- get_siglas_medicas()
  texto_lower <- tolower(stringr::str_trim(texto))

  if (texto_lower %in% names(siglas)) {
    return(siglas[[texto_lower]]$termino)
  }

  return(NULL)
}

#' Obtener codigo CIE-10 desde sigla medica
#'
#' @param sigla Character sigla medica (ej. "IAM", "DM2")
#' @return Character vector con codigos CIE-10 o NULL
#' @keywords internal
#' @noRd
sigla_to_codigo <- function(sigla) {
  siglas <- get_siglas_medicas()
  sigla_lower <- tolower(stringr::str_trim(sigla))

  if (!(sigla_lower %in% names(siglas))) {
    return(NULL)
  }

  termino <- siglas[[sigla_lower]]$termino
  resultado <- cie_search(termino, solo_fuzzy = TRUE, verbose = FALSE)

  if (nrow(resultado) > 0) {
    return(resultado$codigo[1])
  }

  return(NULL)
}

#' Extraer codigo CIE-10 de texto con ruido
#'
#' @param texto Character vector que puede contener prefijos/sufijos
#' @return Character vector con codigo CIE-10 extraido o original
#' @keywords internal
#' @noRd
extract_cie_from_text <- function(texto) {
  # Fix #2: Patron estricto para extraer codigo CIE-10:
  # letra + 2-3 digitos + punto opcional + 0-2 digitos
  # Solo extrae si esta rodeado de no-alfanumericos o en extremos
  patron <- "(?:^|[^A-Z0-9])([A-Z][0-9]{2}[0-9]?\\.?[0-9X]{0,2})(?:$|[^A-Z0-9])"
  
  extraido <- stringr::str_extract(toupper(texto), patron)
  
  # Extraer grupo capturado (quitar prefijos/sufijos)
  if (!is.na(extraido) && extraido != "") {
    extraido <- gsub("^[^A-Z]+|[^A-Z0-9]+$", "", extraido)
  }
  
  # Si se extrajo algo, usarlo; si no, devolver original
  resultado <- ifelse(
    !is.na(extraido) & extraido != "",
    extraido,
    texto
  )
  
  return(resultado)
}

#' Listar siglas medicas soportadas
#'
#' Muestra todas las siglas medicas que pueden usarse en cie_search().
#'
#' @param categoria Character opcional, filtrar por categoria. Valores validos:
#'   "cardiovascular", "respiratoria", "metabolica", "gastrointestinal",
#'   "infecciosa", "oncologica", "reumatologica", "neurologica",
#'   "psiquiatrica", "traumatologica", "pediatrica", "gineco_obstetrica".
#'   Si es NULL (default), retorna todas las siglas.
#' @return tibble con columnas: sigla, termino_busqueda, categoria
#' @family busqueda
#' @seealso \code{\link{cie_search}}, \code{\link{cie_lookup}}
#' @export
#' @examples
#' # Ver todas las siglas
#' cie_siglas()
#'
#' # Filtrar por categoria
#' cie_siglas("cardiovascular")
#' cie_siglas("oncologica")
#'
#' # Buscar una sigla especifica
#' cie_siglas() |> dplyr::filter(sigla == "iam")
cie_siglas <- function(categoria = NULL) {
  siglas <- get_siglas_medicas()

  resultado <- tibble::tibble(
    sigla = names(siglas),
    termino_busqueda = vapply(siglas, function(x) x$termino, character(1)),
    categoria = vapply(siglas, function(x) x$categoria, character(1))
  )

  # Filtrar por categoria si se especifica
  if (!is.null(categoria)) {
    categoria <- tolower(categoria)
    categorias_validas <- unique(resultado$categoria)

    if (!categoria %in% categorias_validas) {
      warning("Categoria '", categoria, "' no encontrada. ",
              "Categorias validas: ",
              paste(categorias_validas, collapse = ", "))
      return(resultado[0, ])
    }

    resultado <- resultado[resultado$categoria == categoria, ]
  }

  return(resultado)
}

#' Busqueda difusa (fuzzy) de terminos medicos CIE-10
#'
#' Busca en descripciones CIE-10 usando multiples estrategias:
#' 1. Expansion de siglas medicas (IAM, TBC, DM, etc.)
#' 2. Busqueda exacta por subcadena (mas rapida)
#' 3. Busqueda fuzzy con Jaro-Winkler (tolera typos)
#'
#' La busqueda es tolerante a tildes: "neumonia" encuentra "neumonia".
#' Soporta siglas medicas comunes: "IAM" busca "infarto agudo miocardio".
#'
#' @param texto String termino medico en espanol o sigla
#'   (ej. "diabetes", "IAM", "TBC")
#' @param threshold Numeric entre 0 y 1, umbral similitud
#'   Jaro-Winkler (default 0.70)
#' @param max_results Integer, maximo resultados a retornar (default 50)
#' @param campo Character, campo busqueda ("descripcion" o "inclusion")
#' @param solo_fuzzy Logical, usar solo busqueda fuzzy
#'   sin busqueda exacta (default FALSE)
#' @param verbose Logical, mostrar mensajes informativos
#'   (default TRUE). Usar FALSE en scripts.
#' @return tibble ordenado por score descendente (1.0 = coincidencia exacta).
#'   Si el texto corresponde a una sigla medica, se expande
#'   automaticamente antes de buscar.
#' @family busqueda
#' @seealso \code{\link{cie_lookup}},
#'   \code{\link{cie_siglas}}, \code{\link{cie10_sql}}
#' @export
#' @importFrom stringdist stringsim
#' @importFrom dplyr mutate filter arrange desc slice_head select everything %>%
#' @examples
#' # Busqueda basica
#' cie_search("diabetes")
#'
#' \donttest{
#' cie_search("neumonia")
#'
#' # Busqueda por siglas medicas
#' cie_search("IAM")
#' cie_search("DM2")
#'
#' # Tolerante a tildes y typos
#' cie_search("diabetis")
#'
#' # Buscar en inclusiones
#' cie_search("bacteriana", campo = "inclusion")
#' }
cie_search <- function(texto, threshold = 0.70, max_results = 50,
                       campo = c("descripcion", "inclusion"),
                       solo_fuzzy = FALSE, verbose = TRUE) {
  campo <- match.arg(campo)

  # Validacion de parametros
  if (!is.character(texto) || length(texto) != 1 || is.na(texto)) {
    stop("texto debe ser un string character no-NA de longitud 1")
  }
  if (threshold < 0 || threshold > 1) {
    stop("threshold debe estar entre 0 y 1")
  }
  if (max_results < 1) {
    stop("max_results debe ser >= 1")
  }

  texto_limpio <- stringr::str_trim(texto)

  # Permitir siglas de 2 caracteres (DM, TB, FA, etc.)
  if (nchar(texto_limpio) < 2) {
    stop("Texto minimo 2 caracteres")
  }

  # Verificar si es una sigla medica y expandirla
  sigla_expandida <- expandir_sigla(texto_limpio)
  texto_busqueda <- if (!is.null(sigla_expandida)) {
    if (verbose) {
      message(
        "i Sigla detectada: ",
        toupper(texto_limpio), " -> ", sigla_expandida
      )
    }
    sigla_expandida
  } else {
    texto_limpio
  }

  # Conexion pooled (no cerrar, gestionada por .ciecl_env)
  con <- get_cie10_db()

  # Normalizar texto de busqueda (minusculas + sin tildes)
  texto_norm <- tolower(texto_busqueda)
  texto_sin_tildes <- normalizar_tildes(texto_norm)

  # Dividir en palabras para FTS5
  palabras <- unlist(stringr::str_split(texto_sin_tildes, "\\s+"))
  palabras <- palabras[nchar(palabras) >= 2]

  # Pre-filtrar usando FTS5 para velocidad
  query_params <- NULL
  if (length(palabras) > 0) {
    # Sanitizar palabras para FTS5 (prevenir SQL injection)
    # Solo permitir alfanumericos y acentos normalizados
    palabras_fts <- gsub("[^a-z0-9]", "", palabras)
    palabras_fts <- palabras_fts[nchar(palabras_fts) >= 2]

    if (length(palabras_fts) > 0) {
      # Construir query FTS5: palabra1* OR palabra2*
      texto_fts <- paste0(palabras_fts, "*", collapse = " OR ")
      query_params <- list(texto_fts)

      if (campo == "descripcion") {
        query_sql <- "
          SELECT c.codigo, c.descripcion, c.categoria
          FROM cie10 c
          WHERE c.rowid IN (SELECT rowid FROM cie10_fts WHERE cie10_fts MATCH ?)
        "
      } else {
        query_sql <- sprintf("
          SELECT c.codigo, c.descripcion, c.categoria, c.%s
          FROM cie10 c
          WHERE c.rowid IN (SELECT rowid FROM cie10_fts WHERE cie10_fts MATCH ?)
        ", campo)
      }
    } else {
      # Sin palabras validas tras sanitizar, cargar todo
      if (campo == "descripcion") {
        query_sql <- "SELECT codigo, descripcion, categoria FROM cie10"
      } else {
        query_sql <- sprintf(
          "SELECT codigo, descripcion, categoria, %s FROM cie10",
          campo)
      }
    }
  } else {
    # Sin palabras validas, cargar todo (fallback)
    if (campo == "descripcion") {
      query_sql <- "SELECT codigo, descripcion, categoria FROM cie10"
    } else {
      query_sql <- sprintf(
        "SELECT codigo, descripcion, categoria, %s FROM cie10",
        campo)
    }
  }

  base <- DBI::dbGetQuery(con, query_sql, params = query_params) %>%
    tibble::as_tibble()

  # Si FTS5 no retorno resultados, intentar carga completa para fuzzy
  if (nrow(base) == 0 && length(palabras) > 0) {
    if (campo == "descripcion") {
      query_sql <- "SELECT codigo, descripcion, categoria FROM cie10"
    } else {
      query_sql <- sprintf(
        "SELECT codigo, descripcion, categoria, %s FROM cie10",
        campo)
    }
    base <- DBI::dbGetQuery(con, query_sql) %>%
      tibble::as_tibble()
  }

  # Normalizar texto de la base (minusculas + sin tildes)
  base_texto <- tolower(stringr::str_trim(base[[campo]]))
  base_texto[is.na(base_texto)] <- ""
  base_texto_sin_tildes <- normalizar_tildes(base_texto)

  # ESTRATEGIA 1: Busqueda exacta por subcadena (mas rapida y precisa)
  if (!solo_fuzzy) {
    # Buscar coincidencias exactas (subcadena)
    matches_exactos <- stringr::str_detect(base_texto_sin_tildes,
                                           stringr::fixed(texto_sin_tildes))

    if (any(matches_exactos)) {
      resultado_exacto <- base[matches_exactos, ] %>%
        dplyr::mutate(score = 1.0) %>%
        dplyr::slice_head(n = max_results) %>%
        dplyr::select(codigo, descripcion, score, dplyr::everything())

      return(resultado_exacto)
    }
  }

  # ESTRATEGIA 2: Busqueda fuzzy palabra por palabra
  # palabras ya fue definido arriba para FTS5
  palabras_fuzzy <- palabras[nchar(palabras) >= 3]

  if (length(palabras_fuzzy) > 0) {
    # Calcular score basado en cuantas palabras coinciden
    scores_palabras <- vapply(seq_along(base_texto_sin_tildes), function(i) {
      texto_base <- base_texto_sin_tildes[i]
      # Contar palabras que aparecen en la descripcion
      matches <- vapply(palabras_fuzzy, function(p) {
        stringr::str_detect(texto_base, stringr::fixed(p))
      }, logical(1))
      sum(matches) / length(palabras_fuzzy)
    }, numeric(1))

    # Si hay coincidencias parciales de palabras
    if (any(scores_palabras > 0)) {
      resultado <- base %>%
        dplyr::mutate(score = scores_palabras) %>%
        dplyr::filter(score > 0) %>%
        dplyr::arrange(dplyr::desc(score)) %>%
        dplyr::slice_head(n = max_results) %>%
        dplyr::select(codigo, descripcion, score, dplyr::everything())

      if (nrow(resultado) > 0) {
        return(resultado)
      }
    }
  }

  # ESTRATEGIA 3: Fuzzy matching con Jaro-Winkler (para typos)
  # Calcular similitud de cada palabra del texto con palabras de la descripcion
  scores_fuzzy <- vapply(seq_along(base_texto_sin_tildes), function(i) {
    texto_base <- base_texto_sin_tildes[i]
    palabras_base <- unlist(stringr::str_split(texto_base, "\\s+"))
    palabras_base <- palabras_base[nchar(palabras_base) >= 3]

    if (length(palabras_base) == 0) return(0)

    # Para cada palabra del texto buscar la mejor coincidencia en la descripcion
    best_scores <- vapply(palabras_fuzzy, function(p) {
      if (length(palabras_base) == 0) return(0)
      max(stringdist::stringsim(p, palabras_base, method = "jw"))
    }, numeric(1))

    mean(best_scores)
  }, numeric(1))

  # Filtrar + ordenar resultados fuzzy
  resultado <- base %>%
    dplyr::mutate(score = scores_fuzzy) %>%
    dplyr::filter(score >= threshold) %>%
    dplyr::arrange(dplyr::desc(score)) %>%
    dplyr::slice_head(n = max_results) %>%
    dplyr::select(codigo, descripcion, score, dplyr::everything())

  if (nrow(resultado) == 0 && verbose) {
    message("x Sin coincidencias >= threshold ", threshold)
  }

  return(resultado)
}

#' Busqueda exacta por codigo CIE-10
#'
#' @param codigo Character vector de codigos
#'   (ej. "E11", "E11.0", c("E11.0", "Z00"))
#'   o rango (ej. "E10-E14"). Acepta vectores.
#'   Soporta formatos: con punto (E11.0),
#'   sin punto (E110), o solo categoria (E11).
#' @param expandir Logical, expandir jerarquia completa (default FALSE)
#' @param normalizar Logical, normalizar formato de codigos
#'   automaticamente (default TRUE)
#' @param descripcion_completa Logical, agregar columna descripcion_completa
#'   con formato "CODIGO - DESCRIPCION" (default FALSE)
#' @param extract Logical, extraer codigo CIE-10 de texto con
#'   prefijos/sufijos (default FALSE).
#'   IMPORTANTE: Solo usar con codigo ESCALAR (longitud 1).
#'   Ejemplo: "CIE:E11.0" -> "E11.0", "E11.0-confirmado" -> "E11.0".
#'   Para vectores multiples usar extract=FALSE (default).
#' @param check_siglas Logical, buscar siglas medicas comunes (default FALSE).
#'   Ejemplo: "IAM" -> I21.0 (Infarto agudo miocardio)
#' @return tibble con codigo(s) matcheado(s)
#' @family busqueda
#' @seealso \code{\link{cie_search}},
#'   \code{\link{cie_normalizar}}, \code{\link{cie_expand}}
#' @export
#' @examples
#' # Busqueda directa por codigo
#' cie_lookup("E11.0")
#'
#' \donttest{
#' cie_lookup("E110")        # Sin punto
#' cie_lookup("E11")         # Solo categoria
#' cie_lookup("E11", expandir = TRUE)  # Todos E11.x
#' # Vectorizado - multiples codigos y formatos
#' cie_lookup(c("E11.0", "Z00", "I10"))
#' # Con descripcion completa
#' cie_lookup("E110", descripcion_completa = TRUE)
#' # Extraer codigo de texto con ruido (solo codigo escalar)
#' cie_lookup("CIE:E11.0", extract = TRUE)
#' cie_lookup("E11.0-confirmado", extract = TRUE)
#' # Buscar por siglas medicas
#' cie_lookup("IAM", check_siglas = TRUE)
#' cie_lookup("DM2", check_siglas = TRUE)
#' }
cie_lookup <- function(codigo, expandir = FALSE, normalizar = TRUE,
                       descripcion_completa = FALSE, extract = FALSE,
                       check_siglas = FALSE) {
  # Manejar vector vacio

  if (length(codigo) == 0) {
    return(cie10_empty_tibble(add_descripcion_completa = descripcion_completa))
  }

  # Filtrar NAs antes de procesar
  codigo_sin_na <- codigo[!is.na(codigo)]
  if (length(codigo_sin_na) == 0) {
    return(cie10_empty_tibble(add_descripcion_completa = descripcion_completa))
  }

  # Normalizar entrada
  codigo_input <- stringr::str_trim(toupper(codigo_sin_na))
  # Fix #1: Normalizar espacios en codigos
  codigo_input <- gsub("\\s+", "", codigo_input)
  
  # Extraer codigo de texto con ruido (prefijos/sufijos)
  if (extract) {
    codigo_input <- extract_cie_from_text(codigo_input)
  }
  
  # Buscar siglas medicas
  if (check_siglas) {
    codigo_input <- vapply(codigo_input, function(x) {
      codigo_sigla <- sigla_to_codigo(x)
      if (!is.null(codigo_sigla)) {
        return(codigo_sigla)
      }
      return(x)
    }, character(1), USE.NAMES = FALSE)
  }
  
  # Normalizar formato si se solicita
  # (elimina sufijo X DEIS, agrega punto, etc.)
  # Preservar rangos (guion entre codigos) antes de normalizar,
  # ya que cie_normalizar convierte guiones a puntos
  if (normalizar) {
    es_rango_input <- stringr::str_detect(
      codigo_input,
      "^[A-Z][0-9.]{2,}-[A-Z][0-9.]{2,}$"
    )
    codigo_norm <- ifelse(
      es_rango_input,
      codigo_input,
      cie_normalizar(codigo_input, buscar_db = FALSE)
    )
  } else {
    codigo_norm <- codigo_input
  }
  
  # Si es vector de multiples codigos, procesar con query vectorizada
  if (length(codigo_norm) > 1) {
    # Optimizacion: usar unique() y query batch
    codigo_unique <- unique(codigo_norm)

    # Separar codigos normales de rangos (contienen "-")
    es_rango <- stringr::str_detect(codigo_unique, "-")
    codigos_normales <- codigo_unique[!es_rango]
    codigos_rango <- codigo_unique[es_rango]

    resultado <- cie10_empty_tibble()

    # Query batch para codigos normales (sin rangos)
    if (length(codigos_normales) > 0) {
      # Sanitizar codigos (solo alfanumericos y punto)
      codigos_safe <- codigos_normales[
        stringr::str_detect(
          codigos_normales, "^[A-Za-z0-9.]+$"
        )
      ]

      if (length(codigos_safe) > 0) {
        con <- get_cie10_db()

        if (expandir) {
          # Expandir: LIKE parametrizado por cada codigo
          placeholders <- paste(
            rep("codigo LIKE ?", length(codigos_safe)),
            collapse = " OR "
          )
          query <- sprintf(
            "SELECT * FROM cie10 WHERE %s ORDER BY codigo",
            placeholders
          )
          params <- paste0(codigos_safe, "%")
          resultado <- DBI::dbGetQuery(
            con, query, params = as.list(params)
          ) %>% tibble::as_tibble()
        } else {
          # Exacto: IN clause parametrizada
          placeholders <- paste(rep("?", length(codigos_safe)),
                                collapse = ",")
          query <- sprintf(
            "SELECT * FROM cie10 WHERE codigo IN (%s)",
            placeholders
          )
          resultado <- DBI::dbGetQuery(
            con, query, params = as.list(codigos_safe)
          ) %>% tibble::as_tibble()
        }
      }
    }

    # Procesar rangos individualmente (poco comun)
    if (length(codigos_rango) > 0) {
      resultados_rango <- lapply(codigos_rango, function(cod) {
        cie_lookup_single(cod, expandir = expandir)
      })
      resultado <- dplyr::bind_rows(
        resultado, dplyr::bind_rows(resultados_rango)
      )
    }

    # Eliminar duplicados
    resultado <- dplyr::distinct(resultado)
  } else {
    # Codigo unico - usar funcion interna
    resultado <- cie_lookup_single(codigo_norm, expandir = expandir)
  }
  
  # Agregar columna descripcion_completa si se solicita
  if (descripcion_completa) {
    if (nrow(resultado) > 0) {
      resultado <- resultado %>%
        dplyr::mutate(
          descripcion_completa = paste0(codigo, " - ", descripcion)
        )
    } else {
      # Asegurar que la columna existe incluso cuando el resultado esta vacio
      # Necesitamos usar tibble::add_column()
      # para mantener la estructura de tibble
      resultado <- resultado %>%
        tibble::add_column(descripcion_completa = character(0))
    }
  }
  
  return(resultado)
}

#' Busqueda interna de un solo codigo CIE-10
#' @keywords internal
#' @noRd
cie_lookup_single <- function(codigo_norm, expandir = FALSE) {
  # Asegurar que codigo_norm es un escalar (longitud 1)
  if (length(codigo_norm) != 1) {
    stop("cie_lookup_single() solo acepta un codigo a la vez")
  }

  # Manejar NA
  if (is.na(codigo_norm)) {
    return(cie10_empty_tibble())
  }

  # Manejar cadena vacia
  if (nchar(stringr::str_trim(codigo_norm)) == 0) {
    return(cie10_empty_tibble())
  }

  # Sanitizar entrada para prevenir SQL injection
  # Solo permitir caracteres validos para codigos CIE-10:
  # letras, numeros, punto, guion
  if (!stringr::str_detect(codigo_norm, "^[A-Za-z0-9.\\-]+$")) {
    message("x Codigo con caracteres invalidos: ", codigo_norm)
    return(cie10_empty_tibble())
  }

  # Conexion pooled (queries parametrizadas, previene SQL injection)
  con <- get_cie10_db()

  if (expandir) {
    # Buscar jerarquia completa (E11 -> E11.x)
    query <- "SELECT * FROM cie10 WHERE codigo LIKE ? ORDER BY codigo"
    resultado <- DBI::dbGetQuery(
      con, query,
      params = list(paste0(codigo_norm, "%"))
    )
  } else if (stringr::str_detect(codigo_norm, "-")) {
    # Rango (ej. "E10-E14")
    partes <- stringr::str_split(codigo_norm, "-")[[1]]
    inicio <- partes[1]
    fin <- partes[2]

    # Advertir y corregir rangos invertidos (ej. "E14-E10" -> "E10-E14")
    if (inicio > fin) {
      warning(paste0("Rango invertido detectado: '", codigo_norm, "'. ",
                   "Corrigiendo a '", fin, "-", inicio, "'"))
      temp <- inicio
      inicio <- fin
      fin <- temp
    }

    query <- "SELECT * FROM cie10 WHERE codigo BETWEEN ? AND ? ORDER BY codigo"
    resultado <- DBI::dbGetQuery(con, query, params = list(inicio, fin))
  } else {
    # Exacto
    query <- "SELECT * FROM cie10 WHERE codigo = ?"
    resultado <- DBI::dbGetQuery(con, query, params = list(codigo_norm))
  }

  resultado <- tibble::as_tibble(resultado)

  # Fix #3: Validar estrictamente codigos invalidos
  # Solo codigos CIE-10 validos existen en la base
  if (nrow(resultado) == 0) {
    message("x Codigo no encontrado: ", codigo_norm)
    return(cie10_empty_tibble())
  }

  return(resultado)
}

#' Guia de funciones de busqueda CIE-10
#'
#' Muestra tabla comparativa de cuando usar cada funcion de busqueda.
#'
#' @return tibble con guia comparativa de funciones de busqueda
#' @family busqueda
#' @seealso \code{\link{cie_search}},
#'   \code{\link{cie_lookup}}, \code{\link{cie_siglas}}
#' @export
#' @examples
#' cie_guia_busqueda()
cie_guia_busqueda <- function() {
  guia <- data.frame(
    `Tengo...` = c(
      "Codigo exacto (E11.0)",
      "Codigo sin punto (e110)",
      "Codigo con espacios (E 11.0)",
      "Codigo con prefijos/sufijos",
      "Codigo categoria (E11)",
      "Descripcion (diabetes)",
      "Sigla medica (IAM, DM2)",
      "No se que tengo"
    ),
    `Usar funcion` = c(
      "cie_lookup()",
      "cie_lookup()",
      "cie_lookup()",
      "cie_lookup(extract = TRUE)",
      "cie_lookup() o cie_lookup(expandir = TRUE)",
      "cie_search()",
      "cie_lookup(check_siglas = TRUE)",
      "cie_search() con threshold bajo"
    ),
    `Ejemplo` = c(
      'cie_lookup("E11.0")',
      'cie_lookup("e110")',
      'cie_lookup("E 11.0")',
      'cie_lookup("CIE:E11.0", extract = TRUE)',
      'cie_lookup("E11")',
      'cie_search("diabetes")',
      'cie_lookup("IAM", check_siglas = TRUE)',
      'cie_search("dolor", threshold = 0.5)'
    ),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  return(tibble::as_tibble(guia))
}

test_that("cie_search encuentra diabetes con fuzzy", {
  skip_on_cran()
  
  # Buscar diabetes (sin typo para mayor certeza)
  resultado <- cie_search("diabetes mellitus", threshold = 0.70)
  expect_gt(nrow(resultado), 0)
  # Codigos E10x-E14x (diabetes sin punto decimal)
  expect_true(any(stringr::str_detect(resultado$codigo, "^E1[0-4]")))
})

test_that("cie_lookup codigo exacto funciona", {
  skip_on_cran()
  
  resultado <- cie_lookup("E11.0")
  expect_equal(nrow(resultado), 1)
  expect_equal(resultado$codigo, "E11.0")
})

test_that("cie_lookup expansion jerarquica funciona", {
  skip_on_cran()
  
  hijos <- cie_lookup("E11", expandir = TRUE)
  expect_gt(nrow(hijos), 5)  # E11.0, E11.1, ...
  expect_true(all(stringr::str_starts(hijos$codigo, "E11")))
})

test_that("cie_lookup acepta vectores de multiples codigos", {
  skip_on_cran()
  
  # Vector con multiples codigos
  codigos <- c("E11.0", "Z00", "I10")
  resultado <- cie_lookup(codigos)
  
  # Debe retornar resultados para al menos algunos codigos
  expect_gt(nrow(resultado), 0)
  
  # Los codigos en el resultado deben estar en nuestra lista original
  # (puede haber mas si algun codigo tiene expansion)
  expect_true(any(resultado$codigo %in% codigos))
})

test_that("cie_lookup vectorizado maneja codigos invalidos", {
  skip_on_cran()
  
  # Mezcla de codigos validos e invalidos
  codigos <- c("E11.0", "INVALIDO", "Z00")
  
  # Capturar mensajes de advertencia
  suppressMessages({
    resultado <- cie_lookup(codigos)
  })
  
  # Debe retornar solo los codigos validos
  expect_gt(nrow(resultado), 0)
  expect_true(all(resultado$codigo %in% c("E11.0", "Z00")))
})

test_that("cie_lookup vectorizado elimina duplicados", {
  skip_on_cran()
  
  # Vector con codigos duplicados
  codigos <- c("E11.0", "E11.0", "Z00")
  resultado <- cie_lookup(codigos)
  
  # No debe haber duplicados en resultado
  expect_equal(nrow(resultado), length(unique(resultado$codigo)))
})

test_that("cie_lookup acepta multiples formatos de codigo", {
  skip_on_cran()
  
  # Probar con punto
  resultado_punto <- cie_lookup("E11.0")
  expect_equal(nrow(resultado_punto), 1)
  expect_equal(resultado_punto$codigo, "E11.0")
  
  # Probar sin punto (debe normalizar)
  resultado_sin_punto <- cie_lookup("E110")
  expect_equal(nrow(resultado_sin_punto), 1)
  expect_equal(resultado_sin_punto$codigo, "E11.0")
  
  # Ambos deben dar el mismo resultado
  expect_equal(resultado_punto$codigo, resultado_sin_punto$codigo)
})

test_that("cie_lookup puede generar descripcion_completa", {
  skip_on_cran()
  
  # Sin descripcion_completa
  resultado_normal <- cie_lookup("E11.0")
  expect_false("descripcion_completa" %in% names(resultado_normal))
  
  # Con descripcion_completa
  resultado_completo <- cie_lookup("E11.0", descripcion_completa = TRUE)
  expect_true("descripcion_completa" %in% names(resultado_completo))
  expect_true(stringr::str_detect(resultado_completo$descripcion_completa, "E11\\.0 - "))
})
test_that("cie_lookup con descripcion_completa mantiene columna en dataframe vacio", {
  skip_on_cran()

  # Bug fix: Cuando todos los codigos son invalidos, debe mantener la columna descripcion_completa
  suppressMessages({
    resultado_vacio <- cie_lookup(codigo = c("XXXX", "YYYY", "ZZZZ"), descripcion_completa = TRUE)
  })

  # El dataframe debe estar vacio
  expect_equal(nrow(resultado_vacio), 0)

  # Pero debe tener 11 columnas incluyendo descripcion_completa
  expect_equal(ncol(resultado_vacio), 11)
  expect_true("descripcion_completa" %in% names(resultado_vacio))

  # Verificar nombres de todas las columnas esperadas
  columnas_esperadas <- c("codigo", "descripcion", "categoria", "seccion",
                          "capitulo_nombre", "inclusion", "exclusion", "capitulo",
                          "es_daga", "es_cruz", "descripcion_completa")
  expect_equal(names(resultado_vacio), columnas_esperadas)
})

# ============================================================
# PRUEBAS PARA cie_siglas()
# ============================================================

test_that("cie_siglas retorna todas las siglas", {
  resultado <- cie_siglas()

  expect_s3_class(resultado, "tbl_df")
  expect_true("sigla" %in% names(resultado))
  expect_true("termino_busqueda" %in% names(resultado))
  expect_true("categoria" %in% names(resultado))
  expect_gt(nrow(resultado), 50)  # Al menos 50 siglas
})

test_that("cie_siglas contiene siglas comunes", {
  resultado <- cie_siglas()

  # Verificar siglas comunes
  siglas_esperadas <- c("iam", "dm", "hta", "epoc", "tbc")
  expect_true(all(siglas_esperadas %in% resultado$sigla))
})

test_that("cie_siglas no tiene duplicados", {
  resultado <- cie_siglas()

  # No debe haber siglas duplicadas
  expect_equal(nrow(resultado), length(unique(resultado$sigla)))
})

test_that("cie_siglas filtra por categoria", {
  # Filtrar cardiovasculares
  cardio <- cie_siglas("cardiovascular")
  expect_gt(nrow(cardio), 5)
  expect_true(all(cardio$categoria == "cardiovascular"))

  # Filtrar oncologicas
  onco <- cie_siglas("oncologica")
  expect_gt(nrow(onco), 3)
  expect_true(all(onco$categoria == "oncologica"))

  # Categoria invalida debe dar warning y retornar vacio
  expect_warning(invalida <- cie_siglas("inexistente"), "no encontrada")
  expect_equal(nrow(invalida), 0)
})

# ============================================================
# PRUEBAS PARA rangos en cie_lookup()
# ============================================================

test_that("cie_lookup maneja rangos invertidos", {
  skip_on_cran()
  
  # Rango invertido debe corregirse automaticamente (warning se emite)
  suppressWarnings(
    resultado <- cie_lookup("E14-E10")
  )
  
  # Debe retornar resultados (el rango corregido funciona)
  expect_gt(nrow(resultado), 0)
  
  # Los codigos deben estar en el rango correcto (E10-E14)
  expect_true(all(stringr::str_detect(resultado$codigo, "^E1[0-4]")))
})

test_that("cie_lookup rango normal funciona sin warning", {
  skip_on_cran()

  # Rango normal no debe emitir warning
  expect_no_warning(
    resultado <- cie_lookup("E10-E14")
  )

  expect_gt(nrow(resultado), 0)
})

test_that("cie_search verbose=FALSE suprime mensajes", {
  skip_on_cran()

  # Con verbose=FALSE no debe emitir mensajes
  expect_silent(
    suppressWarnings(cie_search("xyzabc123", verbose = FALSE))
  )
})

# ============================================================
# PRUEBAS ADICIONALES DE EDGE CASES
# ============================================================

test_that("cie_search con threshold 0 retorna muchos resultados", {
  skip_on_cran()

  resultado <- cie_search("diabetes", threshold = 0.0, max_results = 100)

  expect_s3_class(resultado, "tbl_df")
  # Con threshold 0, deberia retornar el maximo solicitado
  expect_lte(nrow(resultado), 100)
})

test_that("cie_search con threshold 1 es muy restrictivo", {
  skip_on_cran()

  # Threshold 1.0 requiere match perfecto
  suppressMessages({
    resultado <- cie_search("xyz123abc", threshold = 1.0, verbose = FALSE)
  })

  expect_s3_class(resultado, "tbl_df")
  # Probablemente 0 resultados para texto aleatorio
  expect_gte(nrow(resultado), 0)
})

test_that("cie_search con max_results=1 retorna maximo 1 resultado", {
  skip_on_cran()

  resultado <- cie_search("diabetes", threshold = 0.5, max_results = 1)

  expect_s3_class(resultado, "tbl_df")
  expect_lte(nrow(resultado), 1)
})

test_that("cie_search con campo inclusion funciona", {
  skip_on_cran()

  resultado <- cie_search("bacteriana", campo = "inclusion",
                          threshold = 0.7, max_results = 20)

  expect_s3_class(resultado, "tbl_df")
  # Puede o no encontrar resultados
  expect_true(nrow(resultado) >= 0)
})

test_that("cie_search valida parametros incorrectos", {
  skip_on_cran()

  # threshold fuera de rango
  expect_error(
    cie_search("diabetes", threshold = 1.5),
    "threshold"
  )

  expect_error(
    cie_search("diabetes", threshold = -0.1),
    "threshold"
  )

  # max_results invalido
  expect_error(
    cie_search("diabetes", max_results = 0),
    "max_results"
  )

  # texto no character
  expect_error(
    cie_search(123),
    "texto"
  )

  # texto NA
  expect_error(
    cie_search(NA_character_),
    "texto"
  )

  # texto muy corto
  expect_error(
    cie_search("a"),
    "2 caracteres"
  )
})

test_that("cie_search acepta siglas de 2 caracteres", {
  skip_on_cran()

  # DM = diabetes mellitus
  resultado <- cie_search("dm", threshold = 0.7, verbose = FALSE)

  expect_s3_class(resultado, "tbl_df")
  expect_gt(nrow(resultado), 0)
})

test_that("cie_search busca con typos usando fuzzy", {
  skip_on_cran()

  # "diabetis" tiene typo
  resultado <- cie_search("diabetis", threshold = 0.7, verbose = FALSE)

  expect_s3_class(resultado, "tbl_df")
  # Fuzzy deberia encontrar "diabetes"
  expect_gt(nrow(resultado), 0)
})

test_that("cie_search normaliza tildes", {
  skip_on_cran()

  # Buscar con y sin tilde
  resultado_sin <- cie_search("neumonia", threshold = 0.7, verbose = FALSE)
  resultado_con <- cie_search("neumon\u00eda", threshold = 0.7, verbose = FALSE)

  expect_s3_class(resultado_sin, "tbl_df")
  expect_s3_class(resultado_con, "tbl_df")

  # Ambas busquedas deberian encontrar resultados similares
  expect_gt(nrow(resultado_sin), 0)
  expect_gt(nrow(resultado_con), 0)
})

# ============================================================
# PRUEBAS ADICIONALES cie_lookup()
# ============================================================

test_that("cie_lookup con extract=TRUE extrae codigo de texto", {
  skip_on_cran()

  # Codigo con prefijo
  resultado <- cie_lookup("CIE:E11.0", extract = TRUE)
  expect_equal(nrow(resultado), 1)
  expect_equal(resultado$codigo, "E11.0")

  # Codigo con sufijo
  resultado2 <- cie_lookup("I10-confirmado", extract = TRUE)
  expect_equal(nrow(resultado2), 1)
  expect_equal(resultado2$codigo, "I10")
})

test_that("cie_lookup con check_siglas=TRUE convierte siglas", {
  skip_on_cran()

  # IAM deberia convertirse a I21.x
  resultado <- cie_lookup("IAM", check_siglas = TRUE)

  # Puede encontrar o no, pero no debe dar error
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_lookup maneja rangos validos", {
  skip_on_cran()

  # Rango normal
  resultado <- cie_lookup("E10-E14")

  expect_s3_class(resultado, "tbl_df")
  expect_gt(nrow(resultado), 10)

  # Todos los codigos deben estar en el rango
  expect_true(all(grepl("^E1[0-4]", resultado$codigo)))
})

test_that("cie_lookup con normalizar=FALSE preserva formato", {
  skip_on_cran()

  # Sin normalizacion, E110 no se convierte a E11.0
  suppressMessages({
    resultado <- cie_lookup("E110", normalizar = FALSE)
  })

  # Puede encontrar o no dependiendo del formato en DB
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_lookup maneja vector con NAs", {
  skip_on_cran()

  codigos <- c("E11.0", NA, "Z00", NA)

  suppressMessages({
    resultado <- cie_lookup(codigos)
  })

  expect_s3_class(resultado, "tbl_df")
  # Solo deberia encontrar E11.0 y Z00
  expect_lte(nrow(resultado), 2)
})

test_that("cie_lookup maneja vector vacio", {
  resultado <- cie_lookup(character(0))

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

test_that("cie_lookup maneja vector solo NAs", {
  resultado <- cie_lookup(c(NA, NA, NA))

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

# ============================================================
# PRUEBAS cie_guia_busqueda()
# ============================================================

test_that("cie_guia_busqueda retorna data.frame", {
  resultado <- cie_guia_busqueda()

  expect_s3_class(resultado, "data.frame")
  expect_gt(nrow(resultado), 5)
  expect_true("Tengo..." %in% names(resultado))
  expect_true("Usar funcion" %in% names(resultado))
  expect_true("Ejemplo" %in% names(resultado))
})

test_that("cie_guia_busqueda contiene casos comunes", {
  resultado <- cie_guia_busqueda()

  # Verificar que menciona casos importantes
  tengo <- resultado[["Tengo..."]]

  expect_true(any(grepl("exacto", tengo, ignore.case = TRUE)))
  expect_true(any(grepl("Descripcion", tengo, ignore.case = TRUE)))
  expect_true(any(grepl("Sigla", tengo, ignore.case = TRUE)))
})

test_that("cie_guia_busqueda tiene 8 filas", {
  resultado <- cie_guia_busqueda()
  expect_equal(nrow(resultado), 8)
})

test_that("cie_guia_busqueda tiene 3 columnas exactas", {
  resultado <- cie_guia_busqueda()
  expect_equal(ncol(resultado), 3)
  expect_equal(names(resultado), c("Tengo...", "Usar funcion", "Ejemplo"))
})

test_that("cie_guia_busqueda ejemplos mencionan funciones ciecl", {
  resultado <- cie_guia_busqueda()

  ejemplos <- resultado[["Ejemplo"]]

  # Verificar que menciona las funciones principales
  expect_true(any(grepl("cie_lookup", ejemplos)))
  expect_true(any(grepl("cie_search", ejemplos)))
})

test_that("cie_guia_busqueda todas las columnas son character", {
  resultado <- cie_guia_busqueda()

  expect_type(resultado[["Tengo..."]], "character")
  expect_type(resultado[["Usar funcion"]], "character")
  expect_type(resultado[["Ejemplo"]], "character")
})

# ============================================================
# PRUEBAS ADICIONALES cie_search() - COBERTURA
# ============================================================

test_that("cie_search con solo_fuzzy=TRUE omite busqueda exacta", {
  skip_on_cran()

  # Con solo_fuzzy, usa directamente fuzzy matching
  resultado <- cie_search("diabetes mellitus", solo_fuzzy = TRUE,
                          threshold = 0.7, verbose = FALSE)

  expect_s3_class(resultado, "tbl_df")
  expect_gt(nrow(resultado), 0)
})

test_that("cie_search detecta sigla y muestra mensaje", {
  skip_on_cran()

  # IAM debe detectarse como sigla
  expect_message(
    resultado <- cie_search("IAM", threshold = 0.7, verbose = TRUE),
    "Sigla detectada"
  )

  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_search busca en inclusion correctamente", {
  skip_on_cran()

  resultado <- cie_search("aguda", campo = "inclusion",
                          threshold = 0.6, max_results = 10, verbose = FALSE)

  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_search con texto largo encuentra matches parciales", {
  skip_on_cran()

  # Busqueda de multiples palabras
  resultado <- cie_search("diabetes tipo dos insulina",
                          threshold = 0.5, max_results = 20, verbose = FALSE)

  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_search fuzzy encuentra typos", {
  skip_on_cran()

  # Typos intencionales
  resultado <- cie_search("hipertencion", threshold = 0.6, verbose = FALSE)

  expect_s3_class(resultado, "tbl_df")
  # Deberia encontrar hipertension
  expect_gt(nrow(resultado), 0)
})

# ============================================================
# PRUEBAS ADICIONALES cie_lookup() - COBERTURA
# ============================================================

test_that("cie_lookup con extract=TRUE y check_siglas=TRUE combinados", {
  skip_on_cran()

  # Combinar ambas opciones
  resultado <- cie_lookup("CIE:E11.0", extract = TRUE, check_siglas = TRUE)

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 1)
})

test_that("cie_lookup maneja rango de codigos", {
  skip_on_cran()

  resultado <- cie_lookup("I20-I25")

  expect_s3_class(resultado, "tbl_df")
  expect_gt(nrow(resultado), 0)
})

test_that("cie_lookup con normalizar procesa codigo sin punto", {
  skip_on_cran()

  # E110 -> E11.0
  resultado <- cie_lookup("E110", normalizar = TRUE)

  expect_equal(nrow(resultado), 1)
  expect_equal(resultado$codigo, "E11.0")
})

test_that("cie_lookup con normalizar elimina sufijo X DEIS", {
  skip_on_cran()

  resultado <- cie_lookup("N10X", normalizar = TRUE)
  expect_equal(nrow(resultado), 1)
  expect_equal(resultado$codigo, "N10")

  # Vector con multiples codigos X
  res2 <- cie_lookup(c("I10X", "N40X"), normalizar = TRUE)
  expect_equal(nrow(res2), 2)
})

test_that("cie_lookup maneja codigos especiales daga/asterisco", {
  skip_on_cran()

  # Buscar codigo con daga si existe
  # Primero verificar que el paquete normaliza correctamente
  resultado <- cie_lookup("A17.0")

  expect_s3_class(resultado, "tbl_df")
})

# ============================================================
# PRUEBAS COBERTURA - mensaje "Sin coincidencias" y sigla_to_codigo NULL
# ============================================================

test_that("cie_search muestra mensaje Sin coincidencias con verbose=TRUE", {
  skip_on_cran()

  # Buscar texto que no deberia tener coincidencias
  expect_message(
    resultado <- cie_search("xyznonexistent12345abc", threshold = 0.95, verbose = TRUE),
    "Sin coincidencias"
  )

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

test_that("cie_search no muestra mensaje con verbose=FALSE cuando no hay resultados", {
  skip_on_cran()

  # No debe mostrar mensaje con verbose=FALSE
  expect_silent({
    resultado <- cie_search("xyznonexistent12345abc", threshold = 0.95, verbose = FALSE)
  })

  expect_equal(nrow(resultado), 0)
})

test_that("cie_lookup con check_siglas=TRUE y sigla invalida no convierte", {
  skip_on_cran()

  # Sigla que no existe en el diccionario
  suppressMessages({
    resultado <- cie_lookup("XYZ", check_siglas = TRUE)
  })

  # Debe retornar vacio porque XYZ no es codigo CIE ni sigla valida
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_lookup con check_siglas=TRUE y sigla valida convierte", {
  skip_on_cran()

  # EPOC es sigla valida
  resultado <- cie_lookup("EPOC", check_siglas = TRUE)

  expect_s3_class(resultado, "tbl_df")
  # Puede encontrar o no, pero no debe crashear
})

test_that("sigla_to_codigo retorna NULL para sigla inexistente", {
  skip_on_cran()

  # Acceder funcion interna
  resultado <- ciecl:::sigla_to_codigo("XYZINVALIDA")

  expect_null(resultado)
})

test_that("sigla_to_codigo retorna codigo para sigla valida", {
  skip_on_cran()

  # IAM es sigla de infarto agudo miocardio
  resultado <- ciecl:::sigla_to_codigo("iam")

  # Puede retornar codigo o NULL si no encuentra match fuzzy
  expect_true(is.null(resultado) || is.character(resultado))
})

test_that("expandir_sigla retorna NULL para texto no-sigla", {
  skip_on_cran()

  resultado <- ciecl:::expandir_sigla("diabetes")

  expect_null(resultado)
})

test_that("expandir_sigla retorna termino para sigla valida", {
  skip_on_cran()

  resultado <- ciecl:::expandir_sigla("IAM")

  expect_equal(resultado, "infarto agudo miocardio")
})

test_that("cie_search con busqueda que no encuentra matches exactos ni fuzzy", {
  skip_on_cran()

  # Threshold muy alto para texto que no matchea bien
  suppressMessages({
    resultado <- cie_search("qwrtyuiopasdfghjklzxcvbnm", threshold = 0.99, verbose = FALSE)
  })

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

test_that("cie_search fuzzy fallback cuando no hay matches exactos", {
  skip_on_cran()

  # Usar solo_fuzzy para forzar path fuzzy
  resultado <- cie_search("diabetes mellitus tipo", solo_fuzzy = TRUE,
                          threshold = 0.5, max_results = 10, verbose = FALSE)

  expect_s3_class(resultado, "tbl_df")
  # Deberia encontrar algo via fuzzy
  expect_gt(nrow(resultado), 0)
})

test_that("normalizar_tildes con vector vacio retorna vacio", {
  resultado <- ciecl:::normalizar_tildes(character(0))

  expect_length(resultado, 0)
  expect_type(resultado, "character")
})

test_that("normalizar_tildes convierte caracteres acentuados", {
  resultado <- ciecl:::normalizar_tildes("neumonía agúda")

  expect_equal(resultado, "neumonia aguda")
})

test_that("extract_cie_from_text extrae codigo de texto con ruido", {
  skip_on_cran()

  # Con prefijo
  resultado1 <- ciecl:::extract_cie_from_text("CIE:E11.0")
  expect_equal(resultado1, "E11.0")

  # Con sufijo
  resultado2 <- ciecl:::extract_cie_from_text("I10-confirmado")
  expect_equal(resultado2, "I10")

  # Sin ruido
  resultado3 <- ciecl:::extract_cie_from_text("E11.0")
  expect_equal(resultado3, "E11.0")
})

# test-utils-internal.R
# Pruebas de funciones internas (no exportadas)

# ============================================================
# PRUEBAS normalizar_tildes()
# ============================================================

test_that("normalizar_tildes remueve tildes correctamente", {
  # Acceder a funcion interna
  normalizar_tildes <- ciecl:::normalizar_tildes

  # Vocales minusculas con tilde
  expect_equal(normalizar_tildes("caf\u00e9"), "cafe")
  expect_equal(normalizar_tildes("\u00e1rbol"), "arbol")
  expect_equal(normalizar_tildes("ri\u00f1\u00f3n"), "rinon")
  expect_equal(normalizar_tildes("ma\u00eds"), "mais")
  expect_equal(normalizar_tildes("bah\u00eda"), "bahia")

  # Vocales mayusculas con tilde
  expect_equal(normalizar_tildes("\u00c1RBOL"), "ARBOL")
  expect_equal(normalizar_tildes("ESPA\u00d1A"), "ESPANA")

  # Dieresis
  expect_equal(normalizar_tildes("ping\u00fcino"), "pinguino")
  expect_equal(normalizar_tildes("PING\u00dcINO"), "PINGUINO")
})

test_that("normalizar_tildes maneja vector vacio", {
  normalizar_tildes <- ciecl:::normalizar_tildes

  resultado <- normalizar_tildes(character(0))
  expect_length(resultado, 0)
  expect_type(resultado, "character")
})

test_that("normalizar_tildes maneja texto sin tildes", {
  normalizar_tildes <- ciecl:::normalizar_tildes

  # Texto ya normalizado
  expect_equal(normalizar_tildes("diabetes"), "diabetes")
  expect_equal(normalizar_tildes("INFARTO"), "INFARTO")
})

test_that("normalizar_tildes es vectorizado", {
  normalizar_tildes <- ciecl:::normalizar_tildes

  entrada <- c("caf\u00e9", "ri\u00f1\u00f3n", "normal")
  esperado <- c("cafe", "rinon", "normal")

  expect_equal(normalizar_tildes(entrada), esperado)
})

# ============================================================
# PRUEBAS get_siglas_medicas()
# ============================================================

test_that("get_siglas_medicas retorna lista completa", {
  get_siglas_medicas <- ciecl:::get_siglas_medicas

  siglas <- get_siglas_medicas()

  expect_type(siglas, "list")
  expect_gt(length(siglas), 50)

  # Verificar estructura de cada entrada
  for (sigla in names(siglas)) {
    expect_true("termino" %in% names(siglas[[sigla]]))
    expect_true("categoria" %in% names(siglas[[sigla]]))
  }
})

test_that("get_siglas_medicas contiene siglas comunes", {
  get_siglas_medicas <- ciecl:::get_siglas_medicas

  siglas <- get_siglas_medicas()

  # Siglas que deben existir
  esperadas <- c("iam", "dm", "dm2", "hta", "epoc", "tbc", "vih", "icc", "fa")

  for (s in esperadas) {
    expect_true(s %in% names(siglas),
                info = paste("Sigla faltante:", s))
  }
})

test_that("get_siglas_medicas tiene categorias validas", {
  get_siglas_medicas <- ciecl:::get_siglas_medicas

  siglas <- get_siglas_medicas()
  categorias <- unique(vapply(siglas, function(x) x$categoria, character(1)))

  # Categorias esperadas
  esperadas <- c("cardiovascular", "respiratoria", "metabolica",
                 "gastrointestinal", "infecciosa", "oncologica")

  for (cat in esperadas) {
    expect_true(cat %in% categorias,
                info = paste("Categoria faltante:", cat))
  }
})

# ============================================================
# PRUEBAS expandir_sigla()
# ============================================================

test_that("expandir_sigla expande siglas conocidas", {
  expandir_sigla <- ciecl:::expandir_sigla

  # Siglas comunes
  expect_equal(expandir_sigla("iam"), "infarto agudo miocardio")
  expect_equal(expandir_sigla("IAM"), "infarto agudo miocardio")
  expect_equal(expandir_sigla("dm"), "diabetes mellitus")
  expect_equal(expandir_sigla("hta"), "hipertension arterial")
  expect_equal(expandir_sigla("epoc"), "enfermedad pulmonar obstructiva cronica")
})

test_that("expandir_sigla retorna NULL para no-siglas", {
  expandir_sigla <- ciecl:::expandir_sigla

  expect_null(expandir_sigla("diabetes"))
  expect_null(expandir_sigla("xyz123"))
  expect_null(expandir_sigla("NOTASIGLA"))
})

test_that("expandir_sigla maneja espacios", {
  expandir_sigla <- ciecl:::expandir_sigla

  expect_equal(expandir_sigla("  iam  "), "infarto agudo miocardio")
  expect_equal(expandir_sigla(" DM "), "diabetes mellitus")
})

# ============================================================
# PRUEBAS extract_cie_from_text()
# ============================================================

test_that("extract_cie_from_text extrae codigo con prefijos", {
  extract_cie <- ciecl:::extract_cie_from_text

  # Prefijos comunes
  expect_equal(extract_cie("CIE:E11.0"), "E11.0")
  expect_equal(extract_cie("cie-E11.0"), "E11.0")
  expect_equal(extract_cie("DX:I10"), "I10")
})

test_that("extract_cie_from_text extrae codigo con sufijos", {
  extract_cie <- ciecl:::extract_cie_from_text

  # Sufijos comunes
  expect_equal(extract_cie("E11.0-confirmado"), "E11.0")
  expect_equal(extract_cie("I10_principal"), "I10")
  expect_equal(extract_cie("Z00.0#1"), "Z00.0")
})

test_that("extract_cie_from_text maneja codigo limpio", {
  extract_cie <- ciecl:::extract_cie_from_text

  # Codigo sin ruido
  expect_equal(extract_cie("E11.0"), "E11.0")
  expect_equal(extract_cie("I10"), "I10")
  expect_equal(extract_cie("Z00"), "Z00")
})

test_that("extract_cie_from_text maneja minusculas", {
  extract_cie <- ciecl:::extract_cie_from_text

  # Convierte a mayusculas
  expect_equal(extract_cie("e11.0"), "E11.0")
  expect_equal(extract_cie("cie:e11.0"), "E11.0")
})

test_that("extract_cie_from_text maneja texto sin codigo valido", {
  extract_cie <- ciecl:::extract_cie_from_text

  # Retorna original si no encuentra patron
  expect_equal(extract_cie("TEXTO SIN CODIGO"), "TEXTO SIN CODIGO")
  expect_equal(extract_cie("123456"), "123456")
})

# ============================================================
# PRUEBAS cie10_empty_tibble()
# ============================================================

test_that("cie10_empty_tibble retorna tibble vacio con estructura correcta", {
  cie10_empty_tibble <- ciecl:::cie10_empty_tibble

  resultado <- cie10_empty_tibble()

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)

  # Columnas esperadas
  columnas <- c("codigo", "descripcion", "categoria", "seccion",
                "capitulo_nombre", "inclusion", "exclusion", "capitulo",
                "es_daga", "es_cruz")

  expect_true(all(columnas %in% names(resultado)))
})

test_that("cie10_empty_tibble con descripcion_completa agrega columna", {
  cie10_empty_tibble <- ciecl:::cie10_empty_tibble

  resultado <- cie10_empty_tibble(add_descripcion_completa = TRUE)

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
  expect_true("descripcion_completa" %in% names(resultado))
  expect_equal(ncol(resultado), 11)
})

test_that("cie10_empty_tibble tiene tipos correctos", {
  cie10_empty_tibble <- ciecl:::cie10_empty_tibble

  resultado <- cie10_empty_tibble()

  expect_type(resultado$codigo, "character")
  expect_type(resultado$descripcion, "character")
  expect_type(resultado$es_daga, "logical")
  expect_type(resultado$es_cruz, "logical")
})

# ============================================================
# PRUEBAS sigla_to_codigo()
# ============================================================

test_that("sigla_to_codigo convierte siglas a codigos CIE-10", {
  skip_on_cran()  # Requiere DB

  sigla_to_codigo <- ciecl:::sigla_to_codigo

  # IAM debe retornar codigo I21.x
  codigo_iam <- sigla_to_codigo("iam")
  if (!is.null(codigo_iam)) {
    expect_true(grepl("^I2[0-5]", codigo_iam),
                info = paste("IAM deberia dar I2x, dio:", codigo_iam))
  }
})

test_that("sigla_to_codigo retorna NULL para texto normal", {
  skip_on_cran()

  sigla_to_codigo <- ciecl:::sigla_to_codigo

  expect_null(sigla_to_codigo("diabetes"))
  expect_null(sigla_to_codigo("cualquier texto"))
})

# ============================================================
# PRUEBAS cie_lookup_single()
# ============================================================

test_that("cie_lookup_single funciona con codigo valido", {
  skip_on_cran()

  cie_lookup_single <- ciecl:::cie_lookup_single

  resultado <- cie_lookup_single("E11.0")

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 1)
  expect_equal(resultado$codigo, "E11.0")
})

test_that("cie_lookup_single retorna vacio para codigo invalido", {
  skip_on_cran()

  cie_lookup_single <- ciecl:::cie_lookup_single

  suppressMessages({
    resultado <- cie_lookup_single("XXXXX")
  })

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

test_that("cie_lookup_single maneja NA", {
  skip_on_cran()

  cie_lookup_single <- ciecl:::cie_lookup_single

  resultado <- cie_lookup_single(NA_character_)

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

test_that("cie_lookup_single maneja cadena vacia", {
  skip_on_cran()

  cie_lookup_single <- ciecl:::cie_lookup_single

  resultado <- cie_lookup_single("")

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

test_that("cie_lookup_single rechaza caracteres invalidos", {
  skip_on_cran()

  cie_lookup_single <- ciecl:::cie_lookup_single

  # SQL injection attempt
  suppressMessages({
    resultado <- cie_lookup_single("E11'; DROP TABLE cie10;--")
  })

  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 0)
})

test_that("cie_lookup_single expande con patron LIKE", {
  skip_on_cran()

  cie_lookup_single <- ciecl:::cie_lookup_single

  resultado <- cie_lookup_single("E11", expandir = TRUE)

  expect_s3_class(resultado, "tbl_df")
  expect_gt(nrow(resultado), 5)
  expect_true(all(grepl("^E11", resultado$codigo)))
})

test_that("cie_lookup_single maneja rangos", {
  skip_on_cran()

  cie_lookup_single <- ciecl:::cie_lookup_single

  resultado <- cie_lookup_single("E10-E11")

  expect_s3_class(resultado, "tbl_df")
  expect_gt(nrow(resultado), 0)
})

test_that("cie_lookup_single error con vector", {
  cie_lookup_single <- ciecl:::cie_lookup_single

  expect_error(cie_lookup_single(c("E11.0", "I10")),
               "solo acepta un codigo")
})

#' Obtener conexion SQLite pooled CIE-10
#'
#' @description
#' Retorna conexion reutilizable a base SQLite en cache usuario.
#' Si no existe cache, lo construye atomicamente. Si la version no coincide,
#' reconstruye automaticamente.
#' Ubicacion: tools::R_user_dir("ciecl", "data")/cie10.db
#'
#' @return Conexion DBI SQLite activa (pooled)
#' @keywords internal
#' @importFrom DBI dbConnect dbExistsTable dbWriteTable dbDisconnect dbIsValid
#' @importFrom DBI dbExecute dbGetQuery
#' @importFrom RSQLite SQLite
#' @importFrom utils data packageVersion
#' @importFrom dplyr %>%
#' @noRd
get_cie10_db <- function() {
  cache_dir <- tools::R_user_dir("ciecl", "data")
  db_path <- file.path(cache_dir, "cie10.db")

  # Pooling: reutilizar conexion existente si es valida y apunta al mismo path

  if (!is.null(.ciecl_env$con) &&
      inherits(.ciecl_env$con, "SQLiteConnection") &&
      DBI::dbIsValid(.ciecl_env$con) &&
      identical(.ciecl_env$db_path, db_path)) {

    # Failsafe: verificar que FTS5 existe (fix cache parcial)
    if (!DBI::dbExistsTable(.ciecl_env$con, "cie10_fts")) {
      build_fts(.ciecl_env$con)
    }

    # Verificar version del cache
    if (!cache_is_current(.ciecl_env$con)) {
      DBI::dbDisconnect(.ciecl_env$con)
      .ciecl_env$con <- NULL
      .ciecl_env$db_path <- NULL
      build_cache_atomic(cache_dir, db_path)
    } else {
      return(.ciecl_env$con)
    }
  }

  # Cerrar conexion anterior si existe pero es invalida

  if (!is.null(.ciecl_env$con)) {
    try(DBI::dbDisconnect(.ciecl_env$con), silent = TRUE)
    .ciecl_env$con <- NULL
    .ciecl_env$db_path <- NULL
  }

  # Construir cache si no existe
  if (!file.exists(db_path)) {
    build_cache_atomic(cache_dir, db_path)
  }

  # Conectar
  con <- DBI::dbConnect(RSQLite::SQLite(), db_path)

  # Verificar integridad: tabla principal debe existir

  if (!DBI::dbExistsTable(con, "cie10")) {
    DBI::dbDisconnect(con)
    build_cache_atomic(cache_dir, db_path)
    con <- DBI::dbConnect(RSQLite::SQLite(), db_path)
  }

  # Failsafe FTS5
  if (!DBI::dbExistsTable(con, "cie10_fts")) {
    build_fts(con)
  }

  # Verificar version
  if (!cache_is_current(con)) {
    DBI::dbDisconnect(con)
    build_cache_atomic(cache_dir, db_path)
    con <- DBI::dbConnect(RSQLite::SQLite(), db_path)
  }

  # Guardar en pool
  .ciecl_env$con <- con
  .ciecl_env$db_path <- db_path

  return(con)
}

#' Construir cache SQLite atomicamente
#'
#' Construye en archivo temporal y renombra al final.
#' Si falla en cualquier punto, no queda cache parcial.
#'
#' @param cache_dir Directorio del cache
#' @param db_path Path final del archivo .db
#' @keywords internal
#' @noRd
build_cache_atomic <- function(cache_dir, db_path) {
  if (!dir.exists(cache_dir)) {
    dir.create(cache_dir, recursive = TRUE)
  }

  tmp_path <- paste0(db_path, ".tmp")

  # Cleanup de .tmp residuales
  if (file.exists(tmp_path)) {
    file.remove(tmp_path)
  }

  con <- DBI::dbConnect(RSQLite::SQLite(), tmp_path)
  on.exit({
    if (DBI::dbIsValid(con)) DBI::dbDisconnect(con)
  }, add = TRUE)

  tryCatch({
    # Cargar datos
    data(cie10_cl, envir = environment())
    DBI::dbWriteTable(con, "cie10", cie10_cl, overwrite = TRUE)

    # Indices
    DBI::dbExecute(con, "CREATE INDEX idx_codigo ON cie10(codigo)")
    DBI::dbExecute(con, "CREATE INDEX idx_desc ON cie10(descripcion)")

    # FTS5
    build_fts(con)

    # Metadata con version
    DBI::dbExecute(con, "
      CREATE TABLE IF NOT EXISTS cie10_meta (
        key TEXT PRIMARY KEY,
        value TEXT
      )
    ")
    pkg_version <- as.character(utils::packageVersion("ciecl"))
    DBI::dbExecute(
      con,
      "INSERT OR REPLACE INTO cie10_meta
       (key, value) VALUES ('cache_version', ?)",
      params = list(pkg_version)
    )

    # Cerrar antes de renombrar
    DBI::dbDisconnect(con)

    # Atomico: renombrar .tmp -> .db
    if (file.exists(db_path)) {
      file.remove(db_path)
    }
    file.rename(tmp_path, db_path)

    if (interactive()) message("Cache SQLite creado: ", db_path)
  }, error = function(e) {
    # Cleanup en caso de error
    if (DBI::dbIsValid(con)) DBI::dbDisconnect(con)
    if (file.exists(tmp_path)) file.remove(tmp_path)
    stop(
      "Error construyendo cache SQLite: ",
      conditionMessage(e), call. = FALSE
    )
  })
}

#' Construir tabla FTS5 sobre conexion existente
#'
#' @param con Conexion DBI activa
#' @keywords internal
#' @noRd
build_fts <- function(con) {
  DBI::dbExecute(con, "
    CREATE VIRTUAL TABLE IF NOT EXISTS cie10_fts USING fts5(
      codigo, descripcion, inclusion, exclusion,
      content='cie10', content_rowid='rowid'
    )
  ")
  DBI::dbExecute(con, "INSERT INTO cie10_fts(cie10_fts) VALUES('rebuild')")
  if (interactive()) message("Tabla FTS5 creada/reconstruida")
}

#' Verificar si el cache corresponde a la version actual del paquete
#'
#' @param con Conexion DBI activa
#' @return Logical TRUE si version coincide
#' @keywords internal
#' @noRd
cache_is_current <- function(con) {
  if (!DBI::dbExistsTable(con, "cie10_meta")) {
    return(FALSE)
  }

  tryCatch({
    cached_version <- DBI::dbGetQuery(
      con,
      "SELECT value FROM cie10_meta WHERE key = 'cache_version'"
    )
    if (nrow(cached_version) == 0) return(FALSE)

    pkg_version <- as.character(utils::packageVersion("ciecl"))
    return(identical(cached_version$value[1], pkg_version))
  }, error = function(e) {
    return(FALSE)
  })
}

#' Ejecutar consultas SQL sobre CIE-10 Chile
#'
#' @param query String SQL valido SQLite (SELECT/WHERE/JOIN)
#' @param close Deprecated. Ignorado — la conexion es pooled y se
#'   gestiona automaticamente. Sera eliminado en una version futura.
#' @return tibble resultado query
#' @family sql
#' @seealso \code{\link{cie10_clear_cache}}, \code{\link{cie10_disconnect}},
#'   \code{\link{cie_search}}
#' @export
#' @examples
#' # Buscar diabetes
#' cie10_sql("SELECT codigo, descripcion FROM cie10 WHERE codigo LIKE 'E11%'")
#'
#' \donttest{
#' # Contar por capitulo
#' cie10_sql("SELECT capitulo, COUNT(*) n FROM cie10 GROUP BY capitulo")
#' }
cie10_sql <- function(query, close = TRUE) {
  if (!missing(close)) {
    .Deprecated(msg = paste0(
      "El argumento 'close' de cie10_sql() esta deprecado ",
      "y sera eliminado en una version futura.\n",
      "La conexion es pooled y se gestiona automaticamente."
    ))
  }
  # Normalizar query: eliminar espacios y saltos de linea al inicio
  query_norm <- stringr::str_trim(query)

  # Validacion de seguridad: solo SELECT permitido
  if (!stringr::str_detect(query_norm, "(?i)^SELECT")) {
    stop("Solo queries SELECT permitidas (seguridad)")
  }

 # Bloquear keywords peligrosos (case-insensitive)
  keywords_peligrosos <- c(
    "\\bDROP\\b", "\\bDELETE\\b", "\\bUPDATE\\b", "\\bINSERT\\b",
    "\\bALTER\\b", "\\bCREATE\\b", "\\bTRUNCATE\\b", "\\bEXEC\\b",
    "\\bATTACH\\b", "\\bDETACH\\b", "\\bPRAGMA\\b"
  )

  for (keyword in keywords_peligrosos) {
    keyword_found <- stringr::str_detect(
      query_norm, stringr::regex(keyword, ignore_case = TRUE)
    )
    if (keyword_found) {
      stop("Query contiene keyword no permitido (seguridad)")
    }
  }

  # Bloquear multiples statements (;)
  # Remover strings, comentarios de linea (--) y comentarios de bloque (/* */)
  query_sin_strings <- query_norm
  query_sin_strings <- stringr::str_remove_all(query_sin_strings, "'[^']*'")
  query_sin_strings <- stringr::str_remove_all(query_sin_strings, "--[^\n]*")
  query_sin_strings <- stringr::str_remove_all(
    query_sin_strings, "(?s)/\\*.*?\\*/"
  )
  if (stringr::str_detect(query_sin_strings, ";")) {
    stop("Multiples statements SQL no permitidos (seguridad)")
  }

  con <- get_cie10_db()

  resultado <- DBI::dbGetQuery(con, query) %>% tibble::as_tibble()

  return(resultado)
}

#' Limpiar cache SQLite (forzar rebuild)
#'
#' @return No return value, called for side effects (deletes SQLite cache).
#' @family sql
#' @seealso \code{\link{cie10_sql}}, \code{\link{cie10_disconnect}}
#' @export
#' @examples
#' # Ver ubicacion del cache
#' tools::R_user_dir("ciecl", "data")
#'
#' \donttest{
#' cie10_clear_cache()  # Elimina cie10.db local
#' }
cie10_clear_cache <- function() {
  # Cerrar conexion pooled antes de borrar
  if (!is.null(.ciecl_env$con)) {
    try(DBI::dbDisconnect(.ciecl_env$con), silent = TRUE)
    .ciecl_env$con <- NULL
    .ciecl_env$db_path <- NULL
  }

  cache_dir <- tools::R_user_dir("ciecl", "data")
  db_path <- file.path(cache_dir, "cie10.db")
  tmp_path <- paste0(db_path, ".tmp")

  eliminados <- FALSE

  if (file.exists(db_path)) {
    file.remove(db_path)
    eliminados <- TRUE
  }

  # Limpiar .tmp residuales
  if (file.exists(tmp_path)) {
    file.remove(tmp_path)
    eliminados <- TRUE
  }

  if (eliminados) {
    message("Cache SQLite eliminado: ", db_path)
  } else {
    message("i Cache no existe")
  }

  invisible(NULL)
}

#' Cerrar conexion pooled SQLite
#'
#' Cierra la conexion reutilizable al archivo SQLite.
#' Util para liberar el lock del archivo .db.
#'
#' @return No return value, called for side effects.
#' @family sql
#' @seealso \code{\link{cie10_sql}}, \code{\link{cie10_clear_cache}}
#' @export
#' @examples
#' # Verificar si hay conexion activa
#' is.null(ciecl:::.ciecl_env$con)
#'
#' \donttest{
#' cie10_disconnect()
#' }
cie10_disconnect <- function() {
  if (!is.null(.ciecl_env$con)) {
    try(DBI::dbDisconnect(.ciecl_env$con), silent = TRUE)
    .ciecl_env$con <- NULL
    .ciecl_env$db_path <- NULL
  }
  invisible(NULL)
}

test_that("SQLite DB inicializa correctamente", {
  skip_on_cran()

  con <- get_cie10_db()
  expect_s4_class(con, "SQLiteConnection")
  expect_true(DBI::dbExistsTable(con, "cie10"))
})

test_that("cie10_sql ejecuta queries SELECT", {
  skip_on_cran()

  resultado <- cie10_sql("SELECT COUNT(*) AS n FROM cie10")
  expect_s3_class(resultado, "tbl_df")
  expect_gt(resultado$n, 5000)  # Minimo 5k codigos
})

test_that("cie10_sql bloquea queries peligrosas", {
  skip_on_cran()

  expect_error(
    cie10_sql("DROP TABLE cie10"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql emite deprecation warning para argumento close", {
  skip_on_cran()

  expect_warning(
    cie10_sql("SELECT COUNT(*) AS n FROM cie10", close = FALSE),
    "deprecado"
  )
})

# ============================================================
# PRUEBAS ADICIONALES cie10_sql()
# ============================================================

test_that("cie10_sql ejecuta queries SQL con clausulas WHERE/LIKE/GROUP/ORDER", {
  skip_on_cran()

  # WHERE retorna codigo exacto con contenido correcto
  r_where <- cie10_sql("SELECT * FROM cie10 WHERE codigo = 'E11.0'")
  expect_equal(nrow(r_where), 1)
  expect_true(grepl("iabetes", r_where$descripcion, ignore.case = TRUE))

  # LIKE filtra por prefijo
  r_like <- cie10_sql("SELECT * FROM cie10 WHERE codigo LIKE 'E11%' LIMIT 10")
  expect_true(all(grepl("^E11", r_like$codigo)))

  # GROUP BY retorna capitulos
  r_group <- cie10_sql("SELECT capitulo, COUNT(*) as n FROM cie10 GROUP BY capitulo")
  expect_gt(nrow(r_group), 10)

  # ORDER BY ordena correctamente
  r_order <- cie10_sql("SELECT codigo FROM cie10 ORDER BY codigo LIMIT 5")
  expect_equal(r_order$codigo, sort(r_order$codigo))
})

test_that("cie10_sql bloquea ALTER TABLE", {
  skip_on_cran()

  # Bloquea por keyword peligroso o por no ser SELECT
  expect_error(
    cie10_sql("ALTER TABLE cie10 ADD COLUMN test TEXT")
  )
})

test_that("cie10_sql bloquea CREATE TABLE", {
  skip_on_cran()

  expect_error(
    cie10_sql("CREATE TABLE test (id INTEGER)")
  )
})

test_that("cie10_sql bloquea TRUNCATE", {
  skip_on_cran()

  expect_error(
    cie10_sql("TRUNCATE TABLE cie10")
  )
})

test_that("cie10_sql bloquea ATTACH DATABASE", {
  skip_on_cran()

  expect_error(
    cie10_sql("ATTACH DATABASE 'test.db' AS test")
  )
})

test_that("cie10_sql bloquea PRAGMA", {
  skip_on_cran()

  expect_error(
    cie10_sql("PRAGMA table_info(cie10)")
  )
})

test_that("cie10_sql bloquea keywords peligrosos case-insensitive", {
  skip_on_cran()

  expect_error(cie10_sql("SELECT * FROM cie10; dRoP TABLE cie10"))
  expect_error(cie10_sql("SELECT * FROM cie10; Pragma table_info(cie10)"))
  expect_error(cie10_sql("SELECT * FROM cie10; aTTaCH DATABASE 'x' AS y"))
})

test_that("cie10_sql permite DISTINCT", {
  skip_on_cran()

  resultado <- cie10_sql("SELECT DISTINCT capitulo FROM cie10")
  expect_s3_class(resultado, "tbl_df")
  expect_gt(nrow(resultado), 0)
})

test_that("cie10_sql permite subqueries y UNION", {
  skip_on_cran()

  # Subquery en WHERE con IN (SELECT ...)
  r_sub <- cie10_sql("SELECT * FROM cie10 WHERE codigo IN (SELECT codigo FROM cie10 WHERE codigo = 'E11.0')")
  expect_equal(nrow(r_sub), 1)

  # UNION de dos SELECT validos
  r_union <- cie10_sql("SELECT codigo, descripcion FROM cie10 WHERE codigo = 'E11.0' UNION SELECT codigo, descripcion FROM cie10 WHERE codigo = 'I10'")
  expect_gte(nrow(r_union), 1)
})

test_that("cie10_sql permite COUNT con condicion", {
  skip_on_cran()

  resultado <- cie10_sql("SELECT COUNT(*) as n FROM cie10 WHERE codigo LIKE 'E%'")
  expect_s3_class(resultado, "tbl_df")
  expect_gt(resultado$n, 100)
})

test_that("cie10_sql maneja query con saltos de linea", {
  skip_on_cran()

  query <- "
    SELECT
      codigo,
      descripcion
    FROM cie10
    WHERE codigo = 'E11.0'
  "
  resultado <- cie10_sql(query)
  expect_s3_class(resultado, "tbl_df")
  expect_equal(nrow(resultado), 1)
})

# ============================================================
# PRUEBAS get_cie10_db()
# ============================================================

test_that("get_cie10_db retorna conexion DBI valida", {
  skip_on_cran()

  con <- ciecl:::get_cie10_db()
  expect_true(DBI::dbIsValid(con))
  expect_s4_class(con, "SQLiteConnection")
})

test_that("get_cie10_db crea tabla cie10 si no existe", {
  skip_on_cran()

  con <- ciecl:::get_cie10_db()
  expect_true(DBI::dbExistsTable(con, "cie10"))
})

test_that("get_cie10_db tabla tiene indices", {
  skip_on_cran()

  con <- ciecl:::get_cie10_db()
  indices <- DBI::dbGetQuery(con, "SELECT name FROM sqlite_master WHERE type='index'")
  expect_gt(nrow(indices), 0)
})

test_that("get_cie10_db usa directorio cache correcto", {
  skip_on_cran()

  cache_dir <- tools::R_user_dir("ciecl", "data")
  db_path <- file.path(cache_dir, "cie10.db")

  con <- ciecl:::get_cie10_db()
  expect_true(file.exists(db_path))
})

test_that("get_cie10_db tabla tiene columnas esperadas", {
  skip_on_cran()

  con <- ciecl:::get_cie10_db()
  columnas <- DBI::dbListFields(con, "cie10")
  expect_true("codigo" %in% columnas)
  expect_true("descripcion" %in% columnas)
})

# ============================================================
# PRUEBAS cie10_clear_cache()
# ============================================================

test_that("cie10_clear_cache elimina archivo db", {
  skip_on_cran()

  cache_dir <- tools::R_user_dir("ciecl", "data")
  db_path <- file.path(cache_dir, "cie10.db")

  # Asegurar que existe
  ciecl:::get_cie10_db()
  expect_true(file.exists(db_path))

  # Limpiar cache
  suppressMessages(cie10_clear_cache())
  expect_false(file.exists(db_path))
})

test_that("cie10_clear_cache es idempotente", {
  skip_on_cran()

  expect_no_error({
    suppressMessages(cie10_clear_cache())
    suppressMessages(cie10_clear_cache())
  })
})

test_that("cie10_clear_cache emite mensaje apropiado", {
  skip_on_cran()

  # Asegurar que existe cache
  ciecl:::get_cie10_db()

  expect_message(cie10_clear_cache(), "eliminado")
  expect_message(cie10_clear_cache(), "no existe")
})

test_that("cie10_clear_cache retorna invisible NULL", {
  skip_on_cran()

  resultado <- suppressMessages(cie10_clear_cache())
  expect_null(resultado)
})

# ============================================================
# PRUEBAS ADICIONALES COBERTURA - Semicolon dentro de strings
# ============================================================

test_that("cie10_sql permite semicolon dentro de strings", {
  skip_on_cran()

  # Semicolon dentro de comillas simples no debe bloquearse
  resultado <- cie10_sql("SELECT codigo, descripcion FROM cie10 WHERE descripcion LIKE '%tipo;%' LIMIT 1")
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie10_sql bloquea semicolon fuera de strings", {

  skip_on_cran()

  # Multiples statements separados por semicolon (sin keywords peligrosos)
  expect_error(
    cie10_sql("SELECT * FROM cie10; SELECT * FROM cie10"),
    "Multiples statements"
  )
})

test_that("cie10_sql maneja comentarios SQL sin falsos positivos", {
  skip_on_cran()

  # Comentario de linea no genera falso positivo
  resultado <- cie10_sql("SELECT codigo FROM cie10 WHERE codigo = 'E11.0' -- comentario")
  expect_equal(nrow(resultado), 1)

  # Comentario de bloque no genera falso positivo
  resultado2 <- cie10_sql("SELECT codigo /* columna */ FROM cie10 WHERE codigo = 'E11.0'")
  expect_equal(nrow(resultado2), 1)

  # Semicolon dentro de comentario no bloquea
  resultado3 <- cie10_sql("SELECT codigo FROM cie10 WHERE codigo = 'E11.0' -- ;test")
  expect_equal(nrow(resultado3), 1)
})

test_that("cie10_sql bloquea INSERT", {
  skip_on_cran()

  expect_error(
    cie10_sql("INSERT INTO cie10 VALUES ('X99', 'Test')"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea UPDATE", {
  skip_on_cran()

  expect_error(
    cie10_sql("UPDATE cie10 SET descripcion = 'test' WHERE codigo = 'E11.0'"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea DELETE", {
  skip_on_cran()

  expect_error(
    cie10_sql("DELETE FROM cie10 WHERE codigo = 'E11.0'"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea DETACH", {
  skip_on_cran()

  expect_error(
    cie10_sql("DETACH DATABASE main"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea EXEC", {
  skip_on_cran()

  expect_error(
    cie10_sql("EXEC sp_help"),
    "Solo queries SELECT"
  )
})

# ============================================================
# PRUEBAS get_cie10_db() creacion directorio cache
# ============================================================

test_that("get_cie10_db crea directorio cache si no existe", {
  skip_on_cran()

  # Limpiar cache para forzar recreacion
  suppressMessages(cie10_clear_cache())

  cache_dir <- tools::R_user_dir("ciecl", "data")

  # Conectar - debe crear directorio si no existe
  ciecl:::get_cie10_db()

  # Verificar que directorio existe
  expect_true(dir.exists(cache_dir))
})

test_that("get_cie10_db inicializa indices en DB nueva", {
  skip_on_cran()

  # Limpiar cache
  suppressMessages(cie10_clear_cache())

  # Conectar - debe inicializar DB (mensajes solo en interactive)
  con <- ciecl:::get_cie10_db()

  # Verificar indices
  indices <- DBI::dbGetQuery(con, "SELECT name FROM sqlite_master WHERE type='index' AND name LIKE 'idx_%'")
  expect_true("idx_codigo" %in% indices$name)
  expect_true("idx_desc" %in% indices$name)
})

# ============================================================
# PRUEBAS CONNECTION POOLING
# ============================================================

test_that("get_cie10_db reutiliza conexion (pooling)", {
  skip_on_cran()

  con1 <- ciecl:::get_cie10_db()
  con2 <- ciecl:::get_cie10_db()

  # Misma referencia de objeto
  expect_identical(con1, con2)
})

test_that("cie10_clear_cache invalida pool", {
  skip_on_cran()

  env <- getFromNamespace(".ciecl_env", "ciecl")

  con1 <- ciecl:::get_cie10_db()
  suppressMessages(cie10_clear_cache())

  # Pool debe estar vacio
  expect_null(env$con)
  expect_null(env$db_path)

  # Nueva conexion debe ser diferente
  con2 <- ciecl:::get_cie10_db()
  expect_true(DBI::dbIsValid(con2))
})

test_that("cie10_disconnect cierra conexion pooled", {
  skip_on_cran()

  env <- getFromNamespace(".ciecl_env", "ciecl")

  # Asegurar conexion activa
  ciecl:::get_cie10_db()
  expect_false(is.null(env$con))

  # Desconectar
  cie10_disconnect()

  expect_null(env$con)
  expect_null(env$db_path)
})

test_that("cie10_disconnect es idempotente", {
  skip_on_cran()

  expect_no_error({
    cie10_disconnect()
    cie10_disconnect()
  })
})

# ============================================================
# PRUEBAS CACHE VERSIONADO
# ============================================================

test_that("cache incluye tabla cie10_meta con version", {
  skip_on_cran()

  con <- ciecl:::get_cie10_db()

  expect_true(DBI::dbExistsTable(con, "cie10_meta"))

  meta <- DBI::dbGetQuery(con, "SELECT * FROM cie10_meta WHERE key = 'cache_version'")
  expect_equal(nrow(meta), 1)
  expect_equal(meta$value, as.character(utils::packageVersion("ciecl")))
})

test_that("cache_is_current retorna TRUE para cache actual", {
  skip_on_cran()

  con <- ciecl:::get_cie10_db()
  expect_true(ciecl:::cache_is_current(con))
})

test_that("cache_is_current retorna FALSE si no hay tabla meta", {
  skip_on_cran()

  # Crear db temporal sin tabla meta
  tmp <- tempfile(fileext = ".db")
  on.exit(unlink(tmp), add = TRUE)

  con_tmp <- DBI::dbConnect(RSQLite::SQLite(), tmp)
  on.exit(DBI::dbDisconnect(con_tmp), add = TRUE)

  DBI::dbWriteTable(con_tmp, "cie10", data.frame(codigo = "E11.0"))

  expect_false(ciecl:::cache_is_current(con_tmp))
})

test_that("version mismatch fuerza rebuild", {
  skip_on_cran()

  # Obtener conexion valida
  con <- ciecl:::get_cie10_db()

  # Manipular version en meta
  DBI::dbExecute(con, "UPDATE cie10_meta SET value = '0.0.0' WHERE key = 'cache_version'")

  # Desconectar correctamente antes de forzar rebuild
  # (necesario en Windows para liberar lock del archivo)
  cie10_disconnect()

  # get_cie10_db debe detectar version mismatch y reconstruir
  con_new <- ciecl:::get_cie10_db()
  expect_true(DBI::dbIsValid(con_new))

  # Version debe ser correcta ahora
  meta <- DBI::dbGetQuery(con_new, "SELECT value FROM cie10_meta WHERE key = 'cache_version'")
  expect_equal(meta$value, as.character(utils::packageVersion("ciecl")))
})

# ============================================================
# PRUEBAS BUILD ATOMICO
# ============================================================

test_that("build_cache_atomic crea cache completo", {
  skip_on_cran()

  # Limpiar
  suppressMessages(cie10_clear_cache())

  cache_dir <- tools::R_user_dir("ciecl", "data")
  db_path <- file.path(cache_dir, "cie10.db")

  # No debe existir .tmp residual despues de build exitoso
  expect_false(file.exists(paste0(db_path, ".tmp")))

  con <- ciecl:::get_cie10_db()
  expect_true(DBI::dbExistsTable(con, "cie10"))
  expect_true(DBI::dbExistsTable(con, "cie10_fts"))
  expect_true(DBI::dbExistsTable(con, "cie10_meta"))
})

# ============================================================
# PRUEBAS .ciecl_env y .onUnload (zzz.R)
# ============================================================

test_that(".ciecl_env tiene estructura correcta", {
  env <- ciecl:::.ciecl_env
  expect_true(is.environment(env))
  expect_true("con" %in% ls(env, all.names = TRUE))
  expect_true("db_path" %in% ls(env, all.names = TRUE))
})

test_that(".onUnload cierra conexion y limpia env", {
  skip_on_cran()
  ciecl:::get_cie10_db()
  env <- ciecl:::.ciecl_env
  expect_false(is.null(env$con))

  onUnload <- getFromNamespace(".onUnload", "ciecl")
  onUnload(libpath = .libPaths()[1])

  expect_null(env$con)
  expect_null(env$db_path)
})

test_that(".onUnload no falla sin conexion activa", {
  ciecl:::cie10_disconnect()
  onUnload <- getFromNamespace(".onUnload", "ciecl")
  expect_no_error(onUnload(libpath = .libPaths()[1]))
})

# ============================================================
# PRUEBAS BRANCHES ADICIONALES cie-sql.R
# ============================================================

test_that("get_cie10_db reconstruye FTS5 si falta en pooled", {
  skip_on_cran()
  con <- ciecl:::get_cie10_db()
  DBI::dbExecute(con, "DROP TABLE IF EXISTS cie10_fts")
  # Forzar re-check via pooling (misma conexion valida, mismo path)
  con2 <- ciecl:::get_cie10_db()
  tablas <- DBI::dbListTables(con2)
  expect_true(any(grepl("cie10_fts", tablas)))
  ciecl::cie10_disconnect()
})

test_that("get_cie10_db reconstruye si version no coincide en pooled", {
  skip_on_cran()
  con <- ciecl:::get_cie10_db()
  # Modificar version en pooled connection -> trigger rebuild en pooling path
  DBI::dbExecute(con, "UPDATE cie10_meta SET value = '0.0.0' WHERE key = 'cache_version'")
  con2 <- ciecl:::get_cie10_db()
  ver <- DBI::dbGetQuery(con2, "SELECT value FROM cie10_meta WHERE key = 'cache_version'")
  expect_equal(ver$value, as.character(utils::packageVersion("ciecl")))
  ciecl::cie10_disconnect()
})

test_that("get_cie10_db limpia conexion invalida", {
  skip_on_cran()
  on.exit(ciecl::cie10_disconnect(), add = TRUE)
  con <- ciecl:::get_cie10_db()
  DBI::dbDisconnect(con)
  # .ciecl_env aun tiene referencia invalida
  con2 <- ciecl:::get_cie10_db()
  expect_true(DBI::dbIsValid(con2))
  ciecl::cie10_disconnect()
})

test_that("get_cie10_db reconstruye si tabla cie10 falta", {
  skip_on_cran()
  ciecl::cie10_disconnect()
  con_direct <- DBI::dbConnect(RSQLite::SQLite(),
    file.path(tools::R_user_dir("ciecl", "data"), "cie10.db"))
  on.exit(try(DBI::dbDisconnect(con_direct), silent = TRUE), add = TRUE)
  DBI::dbExecute(con_direct, "DROP TABLE IF EXISTS cie10")
  DBI::dbDisconnect(con_direct)
  con2 <- ciecl:::get_cie10_db()
  expect_true(DBI::dbExistsTable(con2, "cie10"))
  ciecl::cie10_disconnect()
})

test_that("cie10_sql maneja error de ejecucion SQL", {
  skip_on_cran()
  expect_error(ciecl::cie10_sql("SELECT * FROM tabla_inexistente"))
})

test_that("cache_is_current retorna FALSE cuando query falla", {
  skip_on_cran()
  tmp_db <- tempfile(fileext = ".db")
  on.exit(unlink(tmp_db), add = TRUE)
  con <- DBI::dbConnect(RSQLite::SQLite(), tmp_db)
  on.exit(try(DBI::dbDisconnect(con), silent = TRUE), add = TRUE)
  # Crear tabla cie10_meta con schema incorrecto para que el SELECT falle
  DBI::dbExecute(con, "CREATE TABLE cie10_meta (x INTEGER)")
  expect_false(ciecl:::cache_is_current(con))
})

test_that("build_cache_atomic limpia .tmp residual", {
  skip_on_cran()
  on.exit(ciecl::cie10_disconnect(), add = TRUE)
  ciecl::cie10_disconnect()
  cache_dir <- tools::R_user_dir("ciecl", "data")
  tmp_file <- file.path(cache_dir, "cie10.db.tmp")
  if (!dir.exists(cache_dir)) dir.create(cache_dir, recursive = TRUE)
  file.create(tmp_file)
  expect_true(file.exists(tmp_file))
  suppressMessages(ciecl:::cie10_clear_cache())
  con <- ciecl:::get_cie10_db()
  expect_false(file.exists(tmp_file))
  ciecl::cie10_disconnect()
})

# test-robustness.R
# Pruebas de robustez, concurrencia y estabilidad del paquete

# ============================================================
# PRUEBAS DE ROBUSTEZ DE CONEXION SQLite
# ============================================================

test_that("multiples conexiones consecutivas funcionan", {
  skip_on_cran()

  # Abrir y cerrar multiples conexiones seguidas
  for (i in 1:10) {
    resultado <- cie_lookup("E11.0")
    expect_equal(nrow(resultado), 1)
  }
})

test_that("multiples queries consecutivas funcionan", {
  skip_on_cran()

  # Ejecutar multiples queries seguidas
  queries <- c(
    "SELECT COUNT(*) as n FROM cie10",
    "SELECT * FROM cie10 WHERE codigo = 'E11.0'",
    "SELECT * FROM cie10 WHERE codigo LIKE 'E11%' LIMIT 5",
    "SELECT capitulo, COUNT(*) as n FROM cie10 GROUP BY capitulo"
  )

  for (query in queries) {
    resultado <- cie10_sql(query)
    expect_s3_class(resultado, "tbl_df")
  }
})

test_that("funciones diferentes pueden ejecutarse intercaladas", {
  skip_on_cran()

  # Intercalar diferentes funciones
  resultado1 <- cie_lookup("E11.0")
  resultado2 <- cie_search("diabetes", threshold = 0.70, max_results = 5)
  resultado3 <- cie10_sql("SELECT COUNT(*) as n FROM cie10")
  resultado4 <- cie_lookup("Z00")
  resultado5 <- cie_expand("E11")

  expect_equal(nrow(resultado1), 1)
  expect_gt(nrow(resultado2), 0)
  expect_gt(resultado3$n, 5000)
  expect_s3_class(resultado4, "tbl_df")
  expect_true(length(resultado5) > 0)
})

# ============================================================
# PRUEBAS DE MEMORIA Y RECURSOS
# ============================================================

test_that("queries grandes no causan problemas de memoria", {
  skip_on_cran()

  # Query que retorna muchos resultados
  resultado <- cie10_sql("SELECT * FROM cie10 LIMIT 5000")
  expect_s3_class(resultado, "tbl_df")
  expect_lte(nrow(resultado), 5000)

  # Liberar memoria
  rm(resultado)
  gc()
})

test_that("cie_search con muchos resultados funciona", {
  skip_on_cran()

  # Busqueda con threshold bajo que retorne muchos resultados
  # Nota: texto debe tener al menos 3 caracteres
  resultado <- cie_search("diabetes", threshold = 0.01, max_results = 1000)
  expect_s3_class(resultado, "tbl_df")
  expect_lte(nrow(resultado), 1000)
})

test_that("cie_lookup con vector grande funciona", {
  skip_on_cran()

  # Vector de codigos grande
  codigos <- c(
    paste0("E11.", 0:9),
    paste0("E10.", 0:9),
    paste0("I10.", 0:9),
    paste0("Z00.", 0:9)
  )

  suppressMessages({
    resultado <- cie_lookup(codigos)
  })

  expect_s3_class(resultado, "tbl_df")
})

# ============================================================
# PRUEBAS DE CACHE Y ESTADO
# ============================================================

test_that("cie10_clear_cache funciona sin base de datos", {
  skip_on_cran()

  # No debe dar error si la cache no existe
  # (puede que ya exista, pero el mensaje debe ser apropiado)
  expect_no_error({
    suppressMessages(cie10_clear_cache())
  })
})

test_that("base de datos se reconstruye despues de clear_cache", {
  skip_on_cran()

  # Limpiar cache
  suppressMessages(cie10_clear_cache())

  # La siguiente query debe inicializar la base (mensajes solo en interactive)
  resultado <- cie_lookup("E11.0")

  expect_equal(nrow(resultado), 1)
})

test_that("base de datos persiste entre llamadas", {
  skip_on_cran()

  # Primera llamada
  resultado1 <- cie_lookup("E11.0")

  # Segunda llamada no debe mostrar mensaje de inicializacion
  expect_silent({
    resultado2 <- cie_lookup("E11.0")
  })

  expect_equal(resultado1$codigo, resultado2$codigo)
})

# ============================================================
# PRUEBAS DE INTEGRIDAD DE DATOS
# ============================================================

test_that("todos los codigos tienen descripcion", {
  skip_on_cran()

  resultado <- cie10_sql("SELECT COUNT(*) as n FROM cie10 WHERE descripcion IS NULL OR descripcion = ''")
  expect_equal(resultado$n, 0, info = "Todos los codigos deben tener descripcion")
})

test_that("todos los codigos son unicos", {
  skip_on_cran()

  resultado_total <- cie10_sql("SELECT COUNT(*) as n FROM cie10")
  resultado_unique <- cie10_sql("SELECT COUNT(DISTINCT codigo) as n FROM cie10")

  expect_equal(resultado_total$n, resultado_unique$n)
})

test_that("codigos principales siguen formato CIE-10 valido", {
  skip_on_cran()

  # Obtener codigos principales (excluyendo morfologia M80xx/x que tienen formato especial)
  codigos <- cie10_sql("SELECT codigo FROM cie10 WHERE codigo NOT LIKE 'M80%/%'")$codigo

  # Validar formato
  validos <- cie_validate_vector(codigos)

  # La mayoria deben ser validos (permitir algunos especiales)
  porcentaje_validos <- sum(validos) / length(validos)
  expect_gt(porcentaje_validos, 0.95)
})

test_that("capitulos estan correctamente asignados", {
  skip_on_cran()

  # Verificar que los capitulos existen y son consistentes
  resultado <- cie10_sql("SELECT DISTINCT capitulo FROM cie10 WHERE capitulo IS NOT NULL")

  expect_gt(nrow(resultado), 0)
})

# ============================================================
# PRUEBAS DE RECUPERACION DE ERRORES
# ============================================================

test_that("error en query no corrompe estado", {
  skip_on_cran()

  # Query invalida debe dar error
  expect_error(cie10_sql("DROP TABLE cie10"))

  # Pero la siguiente query valida debe funcionar
  resultado <- cie10_sql("SELECT COUNT(*) as n FROM cie10")
  expect_gt(resultado$n, 5000)
})

test_that("codigo invalido no afecta siguientes busquedas", {
  skip_on_cran()

  # Buscar codigo invalido
  suppressMessages({
    resultado_invalido <- cie_lookup("XXXXXXXXX")
  })
  expect_equal(nrow(resultado_invalido), 0)

  # Buscar codigo valido despues
  resultado_valido <- cie_lookup("E11.0")
  expect_equal(nrow(resultado_valido), 1)
})

test_that("texto invalido en search no afecta siguientes busquedas", {
  skip_on_cran()

  # Busqueda que no encuentra nada
  suppressMessages({
    resultado_vacio <- cie_search("xyzxyzxyzxyz", threshold = 0.99)
  })

  # Busqueda normal debe funcionar
  resultado_normal <- cie_search("diabetes", threshold = 0.70)
  expect_gt(nrow(resultado_normal), 0)
})

# ============================================================
# PRUEBAS DE TIPOS DE RETORNO
# ============================================================

test_that("cie_lookup siempre retorna tibble", {
  skip_on_cran()

  # Con resultado
  resultado1 <- cie_lookup("E11.0")
  expect_s3_class(resultado1, "tbl_df")

  # Sin resultado
  suppressMessages({
    resultado2 <- cie_lookup("XXXXX")
  })
  expect_s3_class(resultado2, "tbl_df")

  # Con vector
  resultado3 <- cie_lookup(c("E11.0", "Z00"))
  expect_s3_class(resultado3, "tbl_df")
})

test_that("cie_search siempre retorna tibble", {
  skip_on_cran()

  # Con resultados
  resultado1 <- cie_search("diabetes", threshold = 0.70)
  expect_s3_class(resultado1, "tbl_df")

  # Sin resultados
  suppressMessages({
    resultado2 <- cie_search("xyzxyzxyz", threshold = 0.99)
  })
  expect_s3_class(resultado2, "tbl_df")
})

test_that("cie10_sql siempre retorna tibble", {
  skip_on_cran()

  resultado <- cie10_sql("SELECT * FROM cie10 LIMIT 1")
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_map_comorbid siempre retorna tibble", {
  resultado <- cie_map_comorbid(c("E11.0", "I50.9"))
  expect_s3_class(resultado, "tbl_df")

  resultado_vacio <- cie_map_comorbid(character(0))
  expect_s3_class(resultado_vacio, "tbl_df")
})

# ============================================================
# PRUEBAS DE COLUMNAS ESPERADAS
# ============================================================

test_that("cie_lookup retorna columnas esperadas", {
  skip_on_cran()

  resultado <- cie_lookup("E11.0")

  columnas_esperadas <- c("codigo", "descripcion", "categoria")
  expect_true(all(columnas_esperadas %in% names(resultado)))
})

test_that("cie_search retorna columnas esperadas", {
  skip_on_cran()

  resultado <- cie_search("diabetes", threshold = 0.70)

  columnas_esperadas <- c("codigo", "descripcion", "score")
  expect_true(all(columnas_esperadas %in% names(resultado)))
})

test_that("cie_map_comorbid retorna columnas esperadas", {
  resultado <- cie_map_comorbid(c("E11.0"))

  columnas_esperadas <- c("codigo", "categoria")
  expect_equal(names(resultado), columnas_esperadas)
})

# ============================================================
# PRUEBAS DE LIMITES Y CASOS EXTREMOS
# ============================================================

test_that("cie_search con threshold 0 no crashea", {
  skip_on_cran()

  resultado <- cie_search("diabetes", threshold = 0, max_results = 10)
  expect_s3_class(resultado, "tbl_df")
})

test_that("cie_search con threshold 1 retorna solo exactos", {
  skip_on_cran()

  # Threshold = 1 significa match exacto
  suppressMessages({
    resultado <- cie_search("diabetes mellitus", threshold = 1.0)
  })
  expect_s3_class(resultado, "tbl_df")

  # Si hay resultados, todos deben tener score = 1
  if (nrow(resultado) > 0) {
    expect_true(all(resultado$score == 1))
  }
})

test_that("cie_lookup con expansion de categoria completa", {
  skip_on_cran()

  # Expandir categoria E (muy grande)
  hijos <- cie_lookup("E", expandir = TRUE)
  expect_s3_class(hijos, "tbl_df")
  expect_gt(nrow(hijos), 100)
})

test_that("cie_validate_vector con vector muy grande", {
  # Vector grande de codigos mixtos
  codigos <- c(
    rep("E11.0", 100),
    rep("INVALIDO", 100),
    rep("Z00", 100)
  )

  resultado <- cie_validate_vector(codigos)
  expect_length(resultado, 300)
  expect_equal(sum(resultado), 200)  # 100 E11.0 + 100 Z00
})

# ============================================================
# PRUEBAS ADICIONALES get_cie10_db()
# ============================================================

test_that("get_cie10_db retorna conexion DBI valida", {
  skip_on_cran()

  con <- ciecl:::get_cie10_db()

  expect_true(DBI::dbIsValid(con))
  expect_s4_class(con, "SQLiteConnection")
})

test_that("get_cie10_db crea tabla cie10 si no existe", {
  skip_on_cran()

  con <- ciecl:::get_cie10_db()

  # Tabla debe existir
  expect_true(DBI::dbExistsTable(con, "cie10"))
})

test_that("get_cie10_db tabla tiene indices", {
  skip_on_cran()

  con <- ciecl:::get_cie10_db()

  # Verificar que existen indices (SQLite)
  indices <- DBI::dbGetQuery(con, "SELECT name FROM sqlite_master WHERE type='index'")

  expect_gt(nrow(indices), 0)
})

test_that("get_cie10_db usa directorio cache correcto", {
  skip_on_cran()

  cache_dir <- tools::R_user_dir("ciecl", "data")
  db_path <- file.path(cache_dir, "cie10.db")

  ciecl:::get_cie10_db()

  expect_true(file.exists(db_path))
})

# ============================================================
# PRUEBAS ADICIONALES cie10_clear_cache()
# ============================================================

test_that("cie10_clear_cache elimina archivo db", {
  skip_on_cran()

  cache_dir <- tools::R_user_dir("ciecl", "data")
  db_path <- file.path(cache_dir, "cie10.db")

  # Asegurar que existe
  ciecl:::get_cie10_db()

  expect_true(file.exists(db_path))

  # Limpiar cache
  suppressMessages(cie10_clear_cache())

  expect_false(file.exists(db_path))
})

test_that("cie10_clear_cache es idempotente", {
  skip_on_cran()

  # Llamar dos veces no debe dar error
  expect_no_error({
    suppressMessages(cie10_clear_cache())
    suppressMessages(cie10_clear_cache())
  })
})

test_that("cie10_clear_cache emite mensaje apropiado", {
  skip_on_cran()

  # Asegurar que existe cache
  ciecl:::get_cie10_db()

  # Debe emitir mensaje de eliminacion
  expect_message(cie10_clear_cache(), "eliminado")

  # Segunda vez mensaje diferente
  expect_message(cie10_clear_cache(), "no existe")
})

test_that("cie10_clear_cache retorna invisible NULL", {
  skip_on_cran()

  resultado <- suppressMessages(cie10_clear_cache())
  expect_null(resultado)
})

# ============================================================
# PRUEBAS cie10_sql() ADICIONALES
# ============================================================

test_that("cie10_sql bloquea DROP TABLE", {
  skip_on_cran()

  expect_error(
    cie10_sql("DROP TABLE cie10"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea DELETE", {
  skip_on_cran()

  expect_error(
    cie10_sql("DELETE FROM cie10 WHERE codigo = 'E11.0'"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea UPDATE", {
  skip_on_cran()

  expect_error(
    cie10_sql("UPDATE cie10 SET descripcion = 'test' WHERE codigo = 'E11.0'"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea INSERT", {
  skip_on_cran()

  expect_error(
    cie10_sql("INSERT INTO cie10 VALUES ('X99', 'test', NULL, NULL, NULL, NULL, NULL, NULL, 0, 0)"),
    "Solo queries SELECT"
  )
})

test_that("cie10_sql bloquea multiples statements", {
  skip_on_cran()

  # El error puede ser por keyword no permitido (DROP) o por multiples statements
  expect_error(
    cie10_sql("SELECT * FROM cie10; DROP TABLE cie10")
  )
})

test_that("cie10_sql permite SELECT con subconsulta", {
  skip_on_cran()

  # Subconsulta valida
  resultado <- cie10_sql("SELECT * FROM cie10 WHERE codigo IN (SELECT codigo FROM cie10 LIMIT 5)")

  expect_s3_class(resultado, "tbl_df")
})

test_that("cie10_sql permite punto y coma dentro de strings", {
  skip_on_cran()

  # Punto y coma dentro de string no debe ser bloqueado
  resultado <- cie10_sql("SELECT * FROM cie10 WHERE descripcion LIKE '%test; test%' LIMIT 1")

  expect_s3_class(resultado, "tbl_df")
})

test_that("cie10_sql con close=FALSE mantiene conexion", {
  skip_on_cran()

  # Este test verifica comportamiento interno
  # La conexion no debe cerrarse si close=FALSE
  resultado <- cie10_sql("SELECT COUNT(*) as n FROM cie10", close = TRUE)
  expect_gt(resultado$n, 0)
})

test_that("cie10_sql normaliza espacios al inicio", {
  skip_on_cran()

  # Query con espacios y saltos de linea al inicio
  resultado <- cie10_sql("
    SELECT COUNT(*) as n FROM cie10
  ")

  expect_s3_class(resultado, "tbl_df")
  expect_gt(resultado$n, 0)
})

# Declarar variables NSE para evitar NOTEs en R CMD check
utils::globalVariables(c(
  # Variables dplyr NSE
  "codigo", "descripcion", "score", "categoria", "inclusion", "exclusion",
  "patron", "title", "capitulo",
  # Dataset
  "cie10_cl",
  # Dot placeholder
  "."
))

#' Tibble vacio con estructura CIE-10
#' @keywords internal
#' @noRd
cie10_empty_tibble <- function(add_descripcion_completa = FALSE) {

  resultado <- tibble::tibble(
    codigo = character(0),
    descripcion = character(0),
    categoria = character(0),
    seccion = character(0),
    capitulo_nombre = character(0),
    inclusion = character(0),
    exclusion = character(0),
    capitulo = character(0),
    es_daga = logical(0),
    es_cruz = logical(0)
  )

  if (add_descripcion_completa) {
    resultado$descripcion_completa <- character(0)
  }

  return(resultado)
}

# setup.R - Configuracion para tests
# Solo establecer NOT_CRAN=true si no viene de R CMD check --as-cran
# --as-cran establece NOT_CRAN="" o "false", respetamos eso
if (identical(Sys.getenv("NOT_CRAN"), "")) {
  # Verificar si es un check de CRAN real mirando otras variables
  if (identical(Sys.getenv("_R_CHECK_PACKAGE_NAME_"), "")) {
    # No es R CMD check, establecer NOT_CRAN=true para tests locales
    Sys.setenv(NOT_CRAN = "true")
  }
}

# Buscar .Renviron en multiples ubicaciones posibles
renviron_paths <- c(
  file.path(getwd(), ".Renviron"),
  normalizePath(file.path("..", "..", ".Renviron"), mustWork = FALSE),
  normalizePath(file.path("..", ".Renviron"), mustWork = FALSE)
)

for (p in renviron_paths) {
  if (file.exists(p)) {
    readRenviron(p)
    break
  }
}

#' @importFrom stringr str_trim str_replace_all str_detect str_extract
#' @importFrom dplyr select mutate filter all_of any_of %>%
#' @importFrom tibble as_tibble
NULL

#' Parsear datos CIE-10 MINSAL/DEIS desde XLS
#' 
#' @description
#' Funcion interna para procesar Lista-Tabular-CIE-10-1-1.xls oficial DEIS.
#' Estructura columnas: codigo, descripcion, categoria, inclusion, exclusion.
#' Detecta columnas automaticamente para mayor robustez.
#' 
#' @param xls_path Ruta al archivo XLS descargado DEIS
#' @return tibble con 39,877 codigos CIE-10 Chile limpios
#' @keywords internal
#' @noRd
parsear_cie10_minsal <- function(xls_path) {
  # Verificar que readxl este instalado
  if (!requireNamespace("readxl", quietly = TRUE)) {
    stop("El paquete 'readxl' es necesario para generar el dataset.\n",
         "Inst\u00e1lalo con: install.packages('readxl')")
  }
  
  if (!file.exists(xls_path)) {
    stop("Archivo XLS no encontrado: ", xls_path)
  }
  
  # Leer XLS con deteccion automatica columnas
  raw <- readxl::read_excel(
    xls_path,
    sheet = 1,
    col_names = TRUE,
    skip = 0,
    .name_repair = "minimal"
  )
  
  # Normalizar nombres
  names(raw) <- tolower(stringr::str_trim(names(raw)))
  names(raw) <- stringr::str_replace_all(names(raw), "\\s+", "_")
  
  # Detectar columnas dinamicamente (soporta tildes: codigo/descripcion)
  col_codigo <- names(raw)[stringr::str_detect(names(raw), "c[o\u00f3]d|clave")]
  col_desc <- names(raw)[stringr::str_detect(names(raw), "desc|t[i\u00ed]tulo")]
  
  if (length(col_codigo) == 0 || length(col_desc) == 0) {
    stop("No se detectaron columnas codigo/descripcion. Columnas: ", 
         paste(names(raw), collapse = ", "))
  }
  
  # Construir dataset
  cie10_limpio <- raw %>%
    dplyr::select(
      codigo = dplyr::all_of(col_codigo[1]),
      descripcion = dplyr::all_of(col_desc[1]),
      categoria = dplyr::any_of(
        names(raw)[stringr::str_detect(
          names(raw), "cat|tipo|categor[i\u00ed]a"
        )]
      ),
      inclusion = dplyr::any_of(
        names(raw)[stringr::str_detect(names(raw), "incl")]
      ),
      exclusion = dplyr::any_of(
        names(raw)[stringr::str_detect(names(raw), "excl")]
      )
    ) %>%
    dplyr::mutate(
      codigo = stringr::str_trim(as.character(codigo)),
      descripcion = stringr::str_trim(as.character(descripcion)),
      capitulo = stringr::str_extract(codigo, "^[A-Z]\\d{1,2}"),
      es_daga = stringr::str_detect(codigo, "\u2020"),
      es_cruz = stringr::str_detect(codigo, "\\*")
    ) %>%
    dplyr::filter(!is.na(codigo), !is.na(descripcion), nchar(codigo) >= 3) %>%
    tibble::as_tibble()
  
  return(cie10_limpio)
}

#' Generar dataset cie10_cl.rda
#'
#' @description
#' EJECUTAR UNA VEZ para crear data/cie10_cl.rda desde archivo DEIS.
#' Deteccion automatica por prioridad: XLSX completo (39K+ codigos)
#' vs XLS legado (~8K).
#'
#' @param archivo_path Ruta al archivo XLSX/XLS DEIS
#'   (opcional, deteccion automatica).
#'   Debe ser una ruta de confianza; no usar con input de usuarios finales.
#' @return Invisible tibble with generated 'ICD-10' data.
#' @examples
#' \dontrun{
#' # Desde ciecl/ ejecutar:
#' generar_cie10_cl()
#' }
#' @keywords internal
#' @noRd
generar_cie10_cl <- function(archivo_path = NULL) {
  # Verificar que usethis este instalado
  if (!requireNamespace("usethis", quietly = TRUE)) {
    stop("El paquete 'usethis' es necesario para generar el dataset.\n",
         "Inst\u00e1lalo con: install.packages('usethis')")
  }

  # Deteccion automatica si no se proporciona ruta (prioridad: XLSX > XLS)
  if (is.null(archivo_path)) {
    candidatos <- c(
      normalizePath("../CIE-10 (1).xlsx", mustWork = FALSE),
      normalizePath("CIE-10 (1).xlsx", mustWork = FALSE),
      normalizePath("../Lista-Tabular-CIE-10-1-1.xls", mustWork = FALSE),
      normalizePath("Lista-Tabular-CIE-10-1-1.xls", mustWork = FALSE)
    )
    existentes <- candidatos[file.exists(candidatos)]
    if (length(existentes) == 0) {
      stop("Archivo no encontrado. Proporcionar ruta con archivo_path = '...'")
    }
    archivo_path <- existentes[1]
  }
  xls_path <- archivo_path
  
  message("Parseando: ", xls_path)
  cie10_cl <- parsear_cie10_minsal(xls_path)
  
  # Guardar en data/
  usethis::use_data(cie10_cl, overwrite = TRUE, compress = "xz")
  
  message("Generado data/cie10_cl.rda con ", nrow(cie10_cl), " codigos")
  invisible(cie10_cl)
}

#' Dataset CIE-10 Chile oficial MINSAL/DEIS v2018
#'
#' @format tibble con 39,877 filas (categorias y subcategorias):
#' \describe{
#'   \item{codigo}{Codigo CIE-10 (ej. "E11.0")}
#'   \item{descripcion}{Diagnostico en espanol chileno}
#'   \item{categoria}{Categoria jerarquica}
#'   \item{seccion}{Seccion dentro del capitulo}
#'   \item{capitulo_nombre}{Nombre descriptivo del capitulo}
#'   \item{inclusion}{Terminos incluidos}
#'   \item{exclusion}{Terminos excluidos}
#'   \item{capitulo}{Capitulo CIE-10 (A-Z)}
#'   \item{es_daga}{Logical, codigo daga (+)}
#'   \item{es_cruz}{Logical, codigo asterisco (*)}
#' }
#' @source \url{https://deis.minsal.cl/centrofic/}
#' @examples
#' data(cie10_cl)
#' head(cie10_cl)
"cie10_cl"

# test-cie-data.R
# Pruebas para funciones de generacion de datos CIE-10

# ============================================================
# PRUEBAS parsear_cie10_minsal()
# ============================================================

test_that("parsear_cie10_minsal requiere readxl", {
  # Solo testear si readxl NO esta instalado
  skip_if(requireNamespace("readxl", quietly = TRUE),
          "readxl esta instalado")

  expect_error(
    ciecl:::parsear_cie10_minsal("test.xls"),
    "readxl"
  )
})

test_that("parsear_cie10_minsal error si archivo no existe", {
  skip_if_not_installed("readxl")

  expect_error(
    ciecl:::parsear_cie10_minsal("archivo_inexistente.xls"),
    "no encontrado"
  )
})

test_that("parsear_cie10_minsal con XLS real", {
  skip_on_cran()
  skip_if_not_installed("readxl")

  # Buscar XLSX/XLS MINSAL en rutas conocidas (formato CIE-10 correcto)
  xls_paths <- c(
    "../CIE-10 (1).xlsx",
    "CIE-10 (1).xlsx"
  )

  xls_path <- NULL
  for (path in xls_paths) {
    if (file.exists(path)) {
      xls_path <- path
      break
    }
  }

  skip_if(is.null(xls_path), "Archivo XLSX CIE-10 MINSAL no disponible")

  # Intentar parsear, skip si formato incompatible
  resultado <- tryCatch(
    ciecl:::parsear_cie10_minsal(xls_path),
    error = function(e) {
      if (grepl("columnas codigo/descripcion", e$message)) {
        skip("XLS tiene formato de columnas incompatible")
      }
      stop(e)
    }
  )

  # Verificar estructura
  expect_s3_class(resultado, "tbl_df")
  expect_true("codigo" %in% names(resultado))
  expect_true("descripcion" %in% names(resultado))
  expect_true("capitulo" %in% names(resultado))
  expect_true("es_daga" %in% names(resultado))
  expect_true("es_cruz" %in% names(resultado))

  # Verificar contenido minimo
  expect_gt(nrow(resultado), 30000)


  # Verificar tipos de datos
  expect_type(resultado$codigo, "character")
  expect_type(resultado$descripcion, "character")
  expect_type(resultado$es_daga, "logical")
  expect_type(resultado$es_cruz, "logical")
})

test_that("parsear_cie10_minsal detecta codigos daga y cruz", {
  skip_on_cran()
  skip_if_not_installed("readxl")

  # Buscar XLSX/XLS MINSAL (formato CIE-10 correcto)
  xls_paths <- c(
    "../CIE-10 (1).xlsx",
    "CIE-10 (1).xlsx"
  )


  xls_path <- NULL
  for (path in xls_paths) {
    if (file.exists(path)) {
      xls_path <- path
      break
    }
  }

  skip_if(is.null(xls_path), "Archivo XLSX CIE-10 MINSAL no disponible")

  # Intentar parsear, skip si formato incompatible
  resultado <- tryCatch(
    ciecl:::parsear_cie10_minsal(xls_path),
    error = function(e) {
      if (grepl("columnas codigo/descripcion", e$message)) {
        skip("XLS tiene formato de columnas incompatible")
      }
      stop(e)
    }
  )

  # Verificar que hay codigos con flags daga/cruz
  n_daga <- sum(resultado$es_daga, na.rm = TRUE)
  n_cruz <- sum(resultado$es_cruz, na.rm = TRUE)

  # Los flags pueden ser 0 si el archivo no tiene simbolos
  # Pero la columna debe existir y ser logica
  expect_type(resultado$es_daga, "logical")
  expect_type(resultado$es_cruz, "logical")
})

# ============================================================
# PRUEBAS ciecl:::generar_cie10_cl()
# ============================================================

test_that("generar_cie10_cl requiere usethis", {
  # Solo testear si usethis NO esta instalado
  skip_if(requireNamespace("usethis", quietly = TRUE),
          "usethis esta instalado")

  expect_error(
    ciecl:::generar_cie10_cl("test.xls"),
    "usethis"
  )
})

test_that("generar_cie10_cl error si archivo no encontrado", {
  skip_if_not_installed("usethis")
  skip_if_not_installed("readxl")

  # En directorio temporal sin archivos DEIS
  withr::with_tempdir({
    expect_error(
      ciecl:::generar_cie10_cl(),
      "no encontrado"
    )
  })
})

test_that("generar_cie10_cl acepta ruta explicita", {
  skip_on_cran()
  skip_if_not_installed("usethis")
  skip_if_not_installed("readxl")

  # Verificar que acepta argumento archivo_path
  # (fallara si archivo no existe, pero no debe dar error de argumento)
  expect_error(
    ciecl:::generar_cie10_cl(archivo_path = "ruta_explicita.xls"),
    "no encontrado"
  )
})

test_that("generar_cie10_cl detecta XLS en directorio padre", {
  skip_on_cran()
  skip_if_not_installed("usethis")
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    # Crear subdirectorio
    dir.create("child")

    # Crear mock XLSX en directorio padre con formato CIE-10 correcto
    # Nota: usamos .xlsx porque writexl crea archivos xlsx
    mock_data <- data.frame(
      codigo = c("E11.0", "I50.9", "A00.0"),
      descripcion = c("Diabetes tipo 2", "Insuficiencia cardiaca", "Colera"),
      stringsAsFactors = FALSE
    )
    writexl::write_xlsx(mock_data, "mock_cie10.xlsx")

    # Verificar que el archivo existe en padre
    expect_true(file.exists("mock_cie10.xlsx"))

    # Desde child, parsear_cie10_minsal deberia poder leer archivo en padre
    withr::with_dir("child", {
      parent_path <- normalizePath("../mock_cie10.xlsx", mustWork = FALSE)
      expect_true(file.exists(parent_path))

      # Ejecutar parsear_cie10_minsal con ruta explicita padre
      resultado <- ciecl:::parsear_cie10_minsal(parent_path)
      expect_s3_class(resultado, "tbl_df")
      expect_equal(nrow(resultado), 3)
    })
  })
})

test_that("generar_cie10_cl detecta XLS en directorio actual", {
  skip_on_cran()
  skip_if_not_installed("usethis")
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    # Crear mock XLSX en directorio actual con formato CIE-10 correcto
    mock_data <- data.frame(
      codigo = c("E11.0", "I50.9", "A00.0"),
      descripcion = c("Diabetes tipo 2", "Insuficiencia cardiaca", "Colera"),
      stringsAsFactors = FALSE
    )
    writexl::write_xlsx(mock_data, "mock_cie10.xlsx")

    # Verificar que el archivo existe
    expect_true(file.exists("mock_cie10.xlsx"))

    # parsear_cie10_minsal con ruta directa
    resultado <- ciecl:::parsear_cie10_minsal("mock_cie10.xlsx")
    expect_s3_class(resultado, "tbl_df")
    expect_equal(nrow(resultado), 3)
  })
})

test_that("generar_cie10_cl autodeteccion prueba todas las rutas", {
  skip_on_cran()
  skip_if_not_installed("usethis")
  skip_if_not_installed("readxl")

  # Este test verifica que la funcion intenta todas las rutas candidatas
  # cuando archivo_path es NULL y ninguna existe
  withr::with_tempdir({
    dir.create("subdir")
    withr::with_dir("subdir", {
      # Desde subdir, ninguna ruta candidata tiene el archivo
      expect_error(
        ciecl:::generar_cie10_cl(archivo_path = NULL),
        "no encontrado"
      )
    })
  })
})

# ============================================================
# PRUEBAS DATASET cie10_cl
# ============================================================

test_that("dataset cie10_cl codigos son unicos", {
  skip_on_cran()

  data("cie10_cl", package = "ciecl", envir = environment())

  # Contar duplicados
  n_total <- nrow(cie10_cl)
  n_unicos <- length(unique(cie10_cl$codigo))

  # Puede haber algunos duplicados por daga/cruz pero no deberia ser excesivo
  ratio_duplicados <- (n_total - n_unicos) / n_total
  expect_lt(ratio_duplicados, 0.1)  # < 10% duplicados
})

test_that("dataset cie10_cl carga con estructura y formato valido", {
  skip_on_cran()

  data("cie10_cl", package = "ciecl", envir = environment())

  # Carga correcta como tibble con >30k filas
  expect_s3_class(cie10_cl, "tbl_df")
  expect_gt(nrow(cie10_cl), 30000)

  # Columnas requeridas presentes
  expect_true(all(c("codigo", "descripcion") %in% names(cie10_cl)))

  # Todos los codigos deben empezar con letra
  expect_true(all(grepl("^[A-Z]", cie10_cl$codigo)))

  # Todos los codigos deben tener al menos 3 caracteres
  expect_true(all(nchar(cie10_cl$codigo) >= 3))
})

test_that("dataset cie10_cl contiene codigos de capitulos clinicos principales", {
  skip_on_cran()

  data("cie10_cl", package = "ciecl", envir = environment())

  # Verificar presencia de letras de capitulos clinicos clave
  letras_iniciales <- unique(substr(cie10_cl$codigo, 1, 1))
  expect_true("E" %in% letras_iniciales)  # Endocrinas (diabetes)
  expect_true("I" %in% letras_iniciales)  # Circulatorias (hipertension)
  expect_true("C" %in% letras_iniciales)  # Neoplasias
  expect_true("J" %in% letras_iniciales)  # Respiratorias
})

# ============================================================
# PRUEBAS DE INTEGRIDAD DE DATOS
# ============================================================

test_that("dataset cie10_cl sin valores NA en columnas criticas", {
  skip_on_cran()

  data("cie10_cl", package = "ciecl", envir = environment())

  # codigo y descripcion no deben tener NA
  expect_equal(sum(is.na(cie10_cl$codigo)), 0)
  expect_equal(sum(is.na(cie10_cl$descripcion)), 0)
})

test_that("dataset cie10_cl descripciones no vacias", {
  skip_on_cran()

  data("cie10_cl", package = "ciecl", envir = environment())

  # Todas las descripciones deben tener contenido
  expect_true(all(nchar(cie10_cl$descripcion) > 0))
})

test_that("dataset cie10_cl capitulos extraidos correctamente", {
  skip_on_cran()

  data("cie10_cl", package = "ciecl", envir = environment())

  # Si existe columna capitulo, verificar
  if ("capitulo" %in% names(cie10_cl)) {
    # Capitulos validos: A00-Z99
    capitulos_validos <- unique(cie10_cl$capitulo)
    expect_true(all(grepl("^[A-Z]\\d{1,2}$", capitulos_validos, perl = TRUE) |
                    is.na(capitulos_validos)))
  }
})

# ============================================================
# PRUEBAS ADICIONALES PARA COBERTURA
# ============================================================

test_that("parsear_cie10_minsal verifica codigos con formato correcto", {
  skip_on_cran()
  skip_if_not_installed("readxl")

  xls_paths <- c(
    "../CIE-10 (1).xlsx",
    "CIE-10 (1).xlsx"
  )

  xls_path <- NULL
  for (path in xls_paths) {
    if (file.exists(path)) {
      xls_path <- path
      break
    }
  }

  skip_if(is.null(xls_path), "Archivo XLSX CIE-10 MINSAL no disponible")

  resultado <- tryCatch(
    ciecl:::parsear_cie10_minsal(xls_path),
    error = function(e) {
      if (grepl("columnas codigo/descripcion", e$message)) {
        skip("XLS tiene formato de columnas incompatible")
      }
      stop(e)
    }
  )

  # Todos los codigos deben empezar con letra y tener minimo 3 caracteres
  expect_true(all(grepl("^[A-Z]", resultado$codigo)))
  expect_true(all(nchar(resultado$codigo) >= 3))
})

test_that("parsear_cie10_minsal extrae capitulo de codigo", {
  skip_on_cran()
  skip_if_not_installed("readxl")

  xls_paths <- c(
    "../CIE-10 (1).xlsx",
    "CIE-10 (1).xlsx"
  )


  xls_path <- NULL
  for (path in xls_paths) {
    if (file.exists(path)) {
      xls_path <- path
      break
    }
  }

  skip_if(is.null(xls_path), "Archivo XLSX CIE-10 MINSAL no disponible")

  resultado <- tryCatch(
    ciecl:::parsear_cie10_minsal(xls_path),
    error = function(e) {
      if (grepl("columnas codigo/descripcion", e$message)) {
        skip("XLS tiene formato de columnas incompatible")
      }
      stop(e)
    }
  )

  # Capitulo debe extraerse correctamente del codigo
  # E11.0 -> E11, A00 -> A00
  expect_true(all(!is.na(resultado$capitulo) | is.na(resultado$codigo)))
})

test_that("parsear_cie10_minsal limpia descripciones", {
  skip_on_cran()
  skip_if_not_installed("readxl")

  xls_paths <- c(
    "../CIE-10 (1).xlsx",
    "CIE-10 (1).xlsx"
  )

  xls_path <- NULL
  for (path in xls_paths) {
    if (file.exists(path)) {
      xls_path <- path
      break
    }
  }

  skip_if(is.null(xls_path), "Archivo XLSX CIE-10 MINSAL no disponible")

  resultado <- tryCatch(
    ciecl:::parsear_cie10_minsal(xls_path),
    error = function(e) {
      if (grepl("columnas codigo/descripcion", e$message)) {
        skip("XLS tiene formato de columnas incompatible")
      }
      stop(e)
    }
  )

  # Descripciones deben estar limpias (sin espacios extra al inicio/fin)
  expect_false(any(grepl("^\\s|\\s$", resultado$descripcion)))
})

test_that("generar_cie10_cl valida parametros y parsea XLS", {
  skip_on_cran()
  skip_if_not_installed("usethis")
  skip_if_not_installed("readxl")

  # Test 1: Error cuando no encuentra archivo
  expect_error(
    ciecl:::generar_cie10_cl(archivo_path = NULL),
    "no encontrado"
  )

  # Test 2: Error con ruta invalida
  expect_error(
    ciecl:::generar_cie10_cl(archivo_path = "archivo_inexistente.xlsx"),
    class = "error"
  )

  # Test 3: Verificar que parsear_cie10_minsal funciona con XLS real
  # (generar_cie10_cl usa internamente parsear_cie10_minsal)
  xls_paths <- c(
    "../CIE-10 (1).xlsx",
    "CIE-10 (1).xlsx"
  )

  xls_path <- NULL
  for (path in xls_paths) {
    if (file.exists(path)) {
      xls_path <- path
      break
    }
  }

  skip_if(is.null(xls_path), "Archivo XLSX CIE-10 MINSAL no disponible")

  # Testear parsear_cie10_minsal directamente (lo que generar_cie10_cl usa)
  resultado <- parsear_cie10_minsal(xls_path)

  expect_s3_class(resultado, "tbl_df")
  expect_gt(nrow(resultado), 30000)
  expect_true("codigo" %in% names(resultado))
  expect_true("descripcion" %in% names(resultado))
})


# ============================================================
# PRUEBAS parsear_cie10_minsal() CON MOCK DATA
# ============================================================

test_that("parsear_cie10_minsal con mock XLSX detecta columnas codigo/descripcion", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    # Crear mock XLSX con estructura minima
    mock_data <- data.frame(
      codigo = c("E11.0", "E11.1", "I50.9", "C50.9"),
      descripcion = c(
        "Diabetes mellitus tipo 2 con coma",
        "Diabetes mellitus tipo 2 con cetoacidosis",
        "Insuficiencia cardiaca, no especificada",
        "Tumor maligno de la mama, sin especificar"
      ),
      stringsAsFactors = FALSE
    )

    mock_path <- "mock_cie10.xlsx"
    writexl::write_xlsx(mock_data, mock_path)

    resultado <- ciecl:::parsear_cie10_minsal(mock_path)

    expect_s3_class(resultado, "tbl_df")
    expect_true("codigo" %in% names(resultado))
    expect_true("descripcion" %in% names(resultado))
    expect_equal(nrow(resultado), 4)
  })
})

test_that("parsear_cie10_minsal con mock extrae capitulo correctamente", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    mock_data <- data.frame(
      codigo = c("E11.0", "I50.9", "A00.0", "Z99.9"),
      descripcion = c("Desc E11", "Desc I50", "Desc A00", "Desc Z99"),
      stringsAsFactors = FALSE
    )

    mock_path <- "mock_cie10.xlsx"
    writexl::write_xlsx(mock_data, mock_path)

    resultado <- ciecl:::parsear_cie10_minsal(mock_path)

    expect_true("capitulo" %in% names(resultado))
    expect_equal(resultado$capitulo[1], "E11")
    expect_equal(resultado$capitulo[2], "I50")
    expect_equal(resultado$capitulo[3], "A00")
    expect_equal(resultado$capitulo[4], "Z99")
  })
})

test_that("parsear_cie10_minsal detecta codigo con tilde en nombre columna", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    # Columna con tilde: codigo -> código
    mock_data <- data.frame(
      `código` = c("E11.0", "I50.9"),
      `descripción` = c("Diabetes", "Insuficiencia cardiaca"),
      stringsAsFactors = FALSE,
      check.names = FALSE
    )

    mock_path <- "mock_cie10.xlsx"
    writexl::write_xlsx(mock_data, mock_path)

    resultado <- ciecl:::parsear_cie10_minsal(mock_path)

    expect_s3_class(resultado, "tbl_df")
    expect_true("codigo" %in% names(resultado))
    expect_equal(nrow(resultado), 2)
  })
})

test_that("parsear_cie10_minsal detecta columna 'clave' como codigo", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    mock_data <- data.frame(
      clave = c("E11.0", "I50.9"),
      titulo = c("Diabetes", "Insuficiencia cardiaca"),
      stringsAsFactors = FALSE
    )

    mock_path <- "mock_cie10.xlsx"
    writexl::write_xlsx(mock_data, mock_path)

    resultado <- ciecl:::parsear_cie10_minsal(mock_path)

    expect_s3_class(resultado, "tbl_df")
    expect_equal(nrow(resultado), 2)
  })
})

test_that("parsear_cie10_minsal filtra codigos con menos de 3 caracteres", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    mock_data <- data.frame(
      codigo = c("E11.0", "AB", "I", "C50.9"),  # AB e I deben ser filtrados
      descripcion = c("Desc1", "Desc2", "Desc3", "Desc4"),
      stringsAsFactors = FALSE
    )

    mock_path <- "mock_cie10.xlsx"
    writexl::write_xlsx(mock_data, mock_path)

    resultado <- ciecl:::parsear_cie10_minsal(mock_path)

    expect_equal(nrow(resultado), 2)  # Solo E11.0 y C50.9
    expect_true(all(nchar(resultado$codigo) >= 3))
  })
})

test_that("parsear_cie10_minsal filtra filas con codigo NA", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    mock_data <- data.frame(
      codigo = c("E11.0", NA, "I50.9", NA),
      descripcion = c("Desc1", "Desc2", "Desc3", "Desc4"),
      stringsAsFactors = FALSE
    )

    mock_path <- "mock_cie10.xlsx"
    writexl::write_xlsx(mock_data, mock_path)

    resultado <- ciecl:::parsear_cie10_minsal(mock_path)

    expect_equal(nrow(resultado), 2)  # Solo E11.0 e I50.9
  })
})

test_that("parsear_cie10_minsal limpia espacios en codigo y descripcion", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    mock_data <- data.frame(
      codigo = c("  E11.0  ", "I50.9 "),
      descripcion = c("  Diabetes  ", " Insuficiencia "),
      stringsAsFactors = FALSE
    )

    mock_path <- "mock_cie10.xlsx"
    writexl::write_xlsx(mock_data, mock_path)

    resultado <- ciecl:::parsear_cie10_minsal(mock_path)

    expect_equal(resultado$codigo[1], "E11.0")
    expect_equal(resultado$codigo[2], "I50.9")
    expect_equal(resultado$descripcion[1], "Diabetes")
    expect_equal(resultado$descripcion[2], "Insuficiencia")
  })
})

test_that("parsear_cie10_minsal detecta simbolo daga", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    mock_data <- data.frame(
      codigo = c("E11.0\u2020", "I50.9"),  # \u2020 es simbolo daga
      descripcion = c("Diabetes daga", "Insuficiencia"),
      stringsAsFactors = FALSE
    )

    mock_path <- "mock_cie10.xlsx"
    writexl::write_xlsx(mock_data, mock_path)

    resultado <- ciecl:::parsear_cie10_minsal(mock_path)

    expect_true("es_daga" %in% names(resultado))
    expect_true(resultado$es_daga[1])
    expect_false(resultado$es_daga[2])
  })
})

test_that("parsear_cie10_minsal detecta simbolo asterisco", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    mock_data <- data.frame(
      codigo = c("E11.0*", "I50.9"),  # * es simbolo cruz
      descripcion = c("Diabetes cruz", "Insuficiencia"),
      stringsAsFactors = FALSE
    )

    mock_path <- "mock_cie10.xlsx"
    writexl::write_xlsx(mock_data, mock_path)

    resultado <- ciecl:::parsear_cie10_minsal(mock_path)

    expect_true("es_cruz" %in% names(resultado))
    expect_true(resultado$es_cruz[1])
    expect_false(resultado$es_cruz[2])
  })
})

test_that("parsear_cie10_minsal incluye columna categoria si existe", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    mock_data <- data.frame(
      codigo = c("E11.0", "I50.9"),
      descripcion = c("Diabetes", "Insuficiencia"),
      categoria = c("Endocrinas", "Circulatorias"),
      stringsAsFactors = FALSE
    )

    mock_path <- "mock_cie10.xlsx"
    writexl::write_xlsx(mock_data, mock_path)

    resultado <- ciecl:::parsear_cie10_minsal(mock_path)

    expect_true("categoria" %in% names(resultado))
  })
})

# ============================================================
# PRUEBAS ADICIONALES PARA COVERAGE (branches faltantes)
# ============================================================

test_that("parsear_cie10_minsal funciona sin columnas opcionales", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    # Solo codigo + descripcion, sin categoria/inclusion/exclusion
    df <- data.frame(
      codigo = c("A00", "A01.0", "B02.9"),
      descripcion = c("Colera", "Fiebre tifoidea", "Zoster sin complicaciones"),
      stringsAsFactors = FALSE
    )
    writexl::write_xlsx(df, "minimal.xlsx")

    resultado <- ciecl:::parsear_cie10_minsal("minimal.xlsx")

    expect_s3_class(resultado, "tbl_df")
    expect_equal(nrow(resultado), 3)
    expect_true("codigo" %in% names(resultado))
    expect_true("descripcion" %in% names(resultado))
    # Columnas opcionales no deben estar
    expect_false("categoria" %in% names(resultado))
    expect_false("inclusion" %in% names(resultado))
    expect_false("exclusion" %in% names(resultado))
  })
})

test_that("parsear_cie10_minsal maneja codigos sin match de capitulo", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    df <- data.frame(
      codigo = c("A00", "ZZZ"),
      descripcion = c("Colera", "Codigo raro"),
      stringsAsFactors = FALSE
    )
    writexl::write_xlsx(df, "raro.xlsx")

    resultado <- ciecl:::parsear_cie10_minsal("raro.xlsx")

    # "ZZZ" no matchea ^[A-Z]\d{1,2} -> capitulo NA
    fila_zzz <- resultado[resultado$codigo == "ZZZ", ]
    if (nrow(fila_zzz) > 0) {
      expect_true(is.na(fila_zzz$capitulo))
    }
    # A00 si debe tener capitulo
    fila_a00 <- resultado[resultado$codigo == "A00", ]
    expect_equal(fila_a00$capitulo, "A00")
  })
})

test_that("parsear_cie10_minsal filtra filas con NA en codigo y descripcion corta", {
  skip_on_cran()
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    df <- data.frame(
      codigo = c("A00", NA, "AB", "B01.0"),
      descripcion = c("Colera", NA, "Corta", "Tifoidea"),
      stringsAsFactors = FALSE
    )
    writexl::write_xlsx(df, "filtro.xlsx")

    resultado <- ciecl:::parsear_cie10_minsal("filtro.xlsx")

    # NA en codigo se filtra, "AB" (nchar < 3) se filtra
    expect_equal(nrow(resultado), 2)
    expect_true(all(resultado$codigo %in% c("A00", "B01.0")))
  })
})

test_that("generar_cie10_cl emite mensaje Parseando antes de error", {
  skip_on_cran()
  skip_if_not_installed("usethis")
  skip_if_not_installed("readxl")
  skip_if_not_installed("writexl")

  withr::with_tempdir({
    df <- data.frame(
      codigo = c("A00", "B01.0", "C02.1"),
      descripcion = c("Colera", "Tifoidea", "Tumor"),
      stringsAsFactors = FALSE
    )
    writexl::write_xlsx(df, "test_cie10.xlsx")

    # generar_cie10_cl llama usethis::use_data() que requiere estar en un paquete,
    # pero el message "Parseando" se emite antes de eso
    msgs <- character()
    tryCatch(
      withCallingHandlers(
        ciecl:::generar_cie10_cl(archivo_path = "test_cie10.xlsx"),
        message = function(m) {
          msgs <<- c(msgs, conditionMessage(m))
          invokeRestart("muffleMessage")
        }
      ),
      error = function(e) NULL
    )
    expect_true(any(grepl("Parseando", msgs)))
  })
})

# test-data-integrity.R
# Pruebas de integridad del dataset cie10_cl

# ============================================================
# PRUEBAS DE ESTRUCTURA DEL DATASET
# ============================================================

test_that("cie10_cl tiene estructura correcta", {
  # Cargar dataset
  data(cie10_cl, package = "ciecl", envir = environment())

  expect_s3_class(cie10_cl, "tbl_df")

  # Columnas esperadas
 columnas_esperadas <- c(
    "codigo", "descripcion", "categoria", "seccion",
    "capitulo_nombre", "inclusion", "exclusion", "capitulo",
    "es_daga", "es_cruz"
  )

  expect_true(all(columnas_esperadas %in% names(cie10_cl)),
              info = paste("Columnas faltantes:",
                           paste(setdiff(columnas_esperadas, names(cie10_cl)),
                                 collapse = ", ")))
})

test_that("cie10_cl tiene tipos de columna correctos", {
  data(cie10_cl, package = "ciecl", envir = environment())

  # Columnas character
  expect_type(cie10_cl$codigo, "character")
  expect_type(cie10_cl$descripcion, "character")

  # Columnas logical
  expect_type(cie10_cl$es_daga, "logical")
  expect_type(cie10_cl$es_cruz, "logical")
})

test_that("cie10_cl tiene numero esperado de codigos", {
  data(cie10_cl, package = "ciecl", envir = environment())

  # El dataset MINSAL tiene aproximadamente 39877 codigos
  # Verificar rango razonable
  expect_gte(nrow(cie10_cl), 39000)
  expect_lte(nrow(cie10_cl), 40000)
})

test_that("cie10_cl no tiene duplicados en codigo", {
  data(cie10_cl, package = "ciecl", envir = environment())

  n_total <- nrow(cie10_cl)
  n_unique <- length(unique(cie10_cl$codigo))

  expect_equal(n_total, n_unique,
               info = paste("Duplicados encontrados:", n_total - n_unique))
})

test_that("cie10_cl no tiene codigos NA", {
  data(cie10_cl, package = "ciecl", envir = environment())

  n_na <- sum(is.na(cie10_cl$codigo))
  expect_equal(n_na, 0,
               info = paste("Codigos NA encontrados:", n_na))
})

test_that("cie10_cl no tiene descripciones NA o vacias", {
  data(cie10_cl, package = "ciecl", envir = environment())

  n_na <- sum(is.na(cie10_cl$descripcion))
  n_empty <- sum(nchar(trimws(cie10_cl$descripcion)) == 0, na.rm = TRUE)

  expect_equal(n_na, 0, info = paste("Descripciones NA:", n_na))
  expect_equal(n_empty, 0, info = paste("Descripciones vacias:", n_empty))
})

# ============================================================
# PRUEBAS DE FORMATO DE CODIGOS
# ============================================================

test_that("codigos siguen formato CIE-10 valido", {
  data(cie10_cl, package = "ciecl", envir = environment())

  # Patron CIE-10: letra + 2 digitos + opcional (.digitos)
  patron <- "^[A-Z][0-9]{2}(\\.[0-9X]{1,2})?$"

  # Excluir codigos especiales de morfologia
  codigos_regulares <- cie10_cl$codigo[!grepl("^M80", cie10_cl$codigo)]

  validos <- grepl(patron, codigos_regulares)
  porcentaje <- sum(validos) / length(validos) * 100

  # Al menos 95% deben cumplir el patron
  expect_gt(porcentaje, 95)
})

test_that("todos los capitulos estan representados", {
  data(cie10_cl, package = "ciecl", envir = environment())

  # Capitulos CIE-10: A-Z (algunos no usados)
  capitulos <- unique(substr(cie10_cl$codigo, 1, 1))

  # Capitulos esperados en CIE-10 Chile
  esperados <- c("A", "B", "C", "D", "E", "F", "G", "H", "I", "J",
                 "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T",
                 "U", "V", "W", "X", "Y", "Z")

  # Al menos los principales deben estar
  principales <- c("A", "B", "C", "D", "E", "F", "G", "I", "J", "K")
  expect_true(all(principales %in% capitulos),
              info = paste("Capitulos faltantes:",
                           paste(setdiff(principales, capitulos), collapse = ", ")))
})

# ============================================================
# PRUEBAS DE CODIGOS ESPECIFICOS CONOCIDOS
# ============================================================

test_that("codigos diabetes existen", {
  data(cie10_cl, package = "ciecl", envir = environment())

  codigos_dm <- cie10_cl$codigo[grepl("^E1[0-4]", cie10_cl$codigo)]
  expect_gt(length(codigos_dm), 50)

  # Codigo especifico
  expect_true("E11.0" %in% cie10_cl$codigo)
})

test_that("codigos hipertension existen", {
  data(cie10_cl, package = "ciecl", envir = environment())

  codigos_hta <- cie10_cl$codigo[grepl("^I1[0-5]", cie10_cl$codigo)]
  expect_gt(length(codigos_hta), 10)  # Hay al menos 10 codigos de hipertension

  # Codigo especifico
  expect_true("I10" %in% cie10_cl$codigo)
})

test_that("codigos neoplasias malignas existen", {
  data(cie10_cl, package = "ciecl", envir = environment())

  codigos_neo <- cie10_cl$codigo[grepl("^C[0-9]", cie10_cl$codigo)]
  expect_gt(length(codigos_neo), 200)
})

test_that("codigos infarto miocardio existen", {
  data(cie10_cl, package = "ciecl", envir = environment())

  codigos_iam <- cie10_cl$codigo[grepl("^I2[1-2]", cie10_cl$codigo)]
  expect_gt(length(codigos_iam), 10)

  expect_true("I21.0" %in% cie10_cl$codigo)
})

test_that("codigos COVID-19 existen", {
  data(cie10_cl, package = "ciecl", envir = environment())

  # COVID-19 usa codigo U07 (capitulo especial)
  codigos_covid <- cie10_cl$codigo[grepl("^U07", cie10_cl$codigo)]

  # Puede que no esten en version 2018
  # Solo verificar si existen que sean validos
  if (length(codigos_covid) > 0) {
    expect_true(all(grepl("^U07", codigos_covid)))
  }
})

# ============================================================
# PRUEBAS DE COLUMNAS DAGA/CRUZ
# ============================================================

test_that("columnas es_daga y es_cruz son consistentes", {
  data(cie10_cl, package = "ciecl", envir = environment())

  # No deben ser todos TRUE ni todos FALSE
  expect_true(any(cie10_cl$es_daga, na.rm = TRUE) || all(!cie10_cl$es_daga, na.rm = TRUE),
              info = "es_daga debe tener variabilidad o todos FALSE")

  expect_true(any(cie10_cl$es_cruz, na.rm = TRUE) || all(!cie10_cl$es_cruz, na.rm = TRUE),
              info = "es_cruz debe tener variabilidad o todos FALSE")
})

---
title: "Soporte de Idiomas e Internacionalizacion"
author: "Rodolfo Tasso Suazo"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Soporte de Idiomas e Internacionalizacion}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

## Idiomas Soportados

El paquete `ciecl` soporta busquedas en multiples idiomas:

| Funcion | Idioma Dataset | Idioma Busqueda | Notas |
|---------|----------------|-----------------|-------|
| `cie_lookup()` | Espanol (Chile) | - | Busqueda por codigo, idioma no aplica |
| `cie_search()` | Espanol (Chile) | Espanol | Descripciones en espanol chileno |
| `cie11_search()` | Espanol/Ingles | Espanol/Ingles | Configurable via parametro `lang` |
| `cie10_sql()` | Espanol (Chile) | SQL | Descripciones en espanol |

## Dataset CIE-10 Chile (Espanol)

El dataset `cie10_cl` contiene **39,877 codigos** con descripciones en espanol
de Chile, basado en el catalogo oficial MINSAL/DEIS v2018.

```{r}
library(ciecl)

# Ver estructura del dataset
head(cie10_cl)
#> # A tibble: 6 x 8
#>   codigo descripcion                    categoria inclusion exclusion capitulo
#>   <chr>  <chr>                          <chr>     <chr>     <chr>     <chr>
#> 1 A00    Colera                         A00       ...       ...       01
#> 2 A00.0  Colera debido a Vibrio...      A00       ...       ...       01
```

### Caracteristicas del Espanol Chileno

- **Tildes preservadas**: "Neumonia", "Rinon", "Corazon"
- **Terminologia local**: Usa terminos medicos comunes en Chile
- **Sin anglicanismos**: Traducciones oficiales MINSAL

## Busqueda Tolerante a Tildes

La funcion `cie_search()` normaliza automaticamente tildes, permitiendo
buscar sin preocuparse por acentos:

```{r}
# Todas estas busquedas encuentran "neumonia"
cie_search("neumonia")
cie_search("neumonía")
cie_search("NEUMONIA")

# Funciona con n
cie_search("rinon")     # Encuentra "rinon"
cie_search("espanol")   # Encuentra terminos con "n"
```

## Siglas Medicas Chilenas

El paquete incluye **88 siglas medicas** comunes en el contexto clinico chileno:

```{r}
# Ver todas las siglas disponibles
cie_siglas()

# Filtrar por categoria
cie_siglas("cardiovascular")
cie_siglas("respiratoria")
cie_siglas("oncologica")

# Buscar usando siglas
cie_search("IAM")   # Infarto Agudo del Miocardio
cie_search("EPOC")  # Enfermedad Pulmonar Obstructiva Cronica
cie_search("DM2")   # Diabetes Mellitus tipo 2
cie_search("HTA")   # Hipertension Arterial
cie_search("TBC")   # Tuberculosis
```

### Categorias de Siglas

| Categoria | Ejemplos | Cantidad |
|-----------|----------|----------|
| Cardiovascular | IAM, HTA, ACV, FA, ICC | 15 |
| Respiratoria | TBC, EPOC, NAC, SDRA | 9 |
| Metabolica | DM, DM1, DM2, ERC, IRC | 7 |
| Gastrointestinal | HDA, HDB, RGE, DHC | 9 |
| Infecciosa | VIH, ITU, ITS, sepsis | 8 |
| Oncologica | CA, LMA, LMC, LLA, LLC | 10 |
| Neurologica | TEC, EPI, EM, ELA | 8 |
| Psiquiatrica | TDAH, TOC, TAG, TEPT | 6 |
| Otras | Reumatologicas, pediatricas, gineco-obstetricas | 16 |

## API CIE-11 Multilingue

La funcion `cie11_search()` permite buscar en la clasificacion CIE-11 de la OMS
en espanol o ingles:

```{r}
# Busqueda en espanol (default)
cie11_search("diabetes mellitus", lang = "es")

# Busqueda en ingles
cie11_search("diabetes mellitus", lang = "en")

# Comparar resultados
es <- cie11_search("infarto", lang = "es")
en <- cie11_search("infarction", lang = "en")
```

### Configuracion de Idioma API

El parametro `lang` acepta:

- `"es"` - Espanol (default)
- `"en"` - Ingles

```{r}
# Configurar idioma por defecto en sesion
options(ciecl.lang = "es")

# Ahora todas las busquedas CIE-11 usaran espanol
cie11_search("neumonia")
```

## Encoding y Caracteres Especiales

### UTF-8 por Defecto

El paquete usa encoding UTF-8 para garantizar compatibilidad con caracteres
especiales del espanol:

```{r}
# Verificar encoding del dataset
Encoding(cie10_cl$descripcion[1])
#> [1] "UTF-8"
```

### Caracteres Especiales Soportados

| Caracter | Ejemplo | Soporte |
|----------|---------|---------|
| Tildes | a, e, i, o, u | Completo |
| Ene | n | Completo |
| Dieresis | u | Completo |
| Simbolos medicos | daga, cruz | Preservados en dataset |

## Contribuir Traducciones

Si deseas contribuir traducciones o mejorar el soporte de idiomas:

1. Abre un issue en <https://github.com/RodoTasso/ciecl/issues>
2. Describe el idioma o mejora propuesta
3. Incluye ejemplos de uso

### Traducciones Pendientes

- [ ] Portugues (Brasil) - Dataset CID-10
- [ ] Ingles - Dataset ICD-10-CM (USA)
- [ ] Frances - Dataset CIM-10

## Referencias

- **CIE-10 Chile**: <https://deis.minsal.cl/centrofic/>
- **CIE-11 OMS**: <https://icd.who.int/>
- **API CIE-11**: <https://icd.who.int/icdapi>

---
title: "Introduccion a ciecl: CIE-10 Chile en R"
author: "Rodolfo Tasso Suazo"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduccion a ciecl: CIE-10 Chile en R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
library(ciecl)
```

> **Version 0.9.3**: Disponible en CRAN con optimizaciones SQLite y soporte XLSX.

## Acerca del dataset

El paquete incluye **39,877 codigos CIE-10** del catalogo oficial MINSAL/DEIS v2018,
incluyendo categorias (3 digitos) y subcategorias (4+ digitos).

## Instalacion
 
```{r eval=FALSE}
# Desde GitHub (version beta)
pak::pak("RodoTasso/ciecl")

# Alternativa con devtools
devtools::install_github("RodoTasso/ciecl")
```

## Busqueda SQL rapida

```{r}
# Todos los codigos diabetes tipo 2
cie10_sql("SELECT codigo, descripcion FROM cie10 WHERE codigo LIKE 'E11%' LIMIT 5")
```

## Busqueda por codigo

```{r}
# Busqueda de un solo codigo
cie_lookup("E11.0")

# Busqueda vectorizada - multiples codigos a la vez
codigos <- c("E11.0", "I10", "Z00", "J44.0")
cie_lookup(codigos)

# Expansion jerarquica
cie_lookup("E11", expandir = TRUE)
```

## Busqueda fuzzy con typos

```{r}
# Encuentra aunque este mal escrito
cie_search("diabetis con coma", threshold = 0.75)
```

## Comorbilidades Charlson

```{r eval=FALSE}
# Requiere: install.packages("comorbidity")
df_pacientes <- data.frame(
  id_pac = c(1, 1, 2, 2, 3),
  diagnostico = c("E11.0", "I50.9", "C50.9", "N18.5", "J44.0")
)

cie_comorbid(df_pacientes, id = "id_pac", code = "diagnostico", map = "charlson")
```

## Tablas interactivas

```{r eval=FALSE}
# Requiere: install.packages("gt")
cie_table("E11")  # Visualizacion GT completa
```

## Fuente de datos

Datos oficiales CIE-10 Chile MINSAL/DEIS v2018:

- Centro FIC Chile: <https://deis.minsal.cl/centrofic/>
- Repositorio DEIS: <https://deis.minsal.cl>

## Mas informacion

- Reportar problemas: <https://github.com/RodoTasso/ciecl/issues>
- Repositorio: <https://github.com/RodoTasso/ciecl>

---
title: "Introduction to ciecl: Chilean ICD-10 in R"
author: "Rodolfo Tasso Suazo"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to ciecl: Chilean ICD-10 in R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
library(ciecl)
```

> **Version 0.9.3**: Available on CRAN with SQLite optimizations and XLSX support.

## About the dataset

The package includes **39,877 ICD-10 codes** from the official MINSAL/DEIS v2018 catalog,
including categories (3 digits) and subcategories (4+ digits).

## Installation

```{r eval=FALSE}
# From GitHub (beta version)
pak::pak("RodoTasso/ciecl")

# Alternative with devtools
devtools::install_github("RodoTasso/ciecl")
```

## Fast SQL search

```{r}
# All type 2 diabetes codes
cie10_sql("SELECT codigo, descripcion FROM cie10 WHERE codigo LIKE 'E11%' LIMIT 5")
```

## Search by code

```{r}
# Single code search
cie_lookup("E11.0")

# Vectorized search - multiple codes at once
codes <- c("E11.0", "I10", "Z00", "J44.0")
cie_lookup(codes)

# Hierarchical expansion
cie_lookup("E11", expandir = TRUE)
```

## Fuzzy search with typos

```{r}
# Finds even if misspelled
cie_search("diabetis with coma", threshold = 0.75)
```

## Charlson Comorbidities

```{r eval=FALSE}
# Requires: install.packages("comorbidity")
patient_df <- data.frame(
  patient_id = c(1, 1, 2, 2, 3),
  diagnosis = c("E11.0", "I50.9", "C50.9", "N18.5", "J44.0")
)

cie_comorbid(patient_df, id = "patient_id", code = "diagnosis", map = "charlson")
```

## Interactive tables

```{r eval=FALSE}
# Requires: install.packages("gt")
cie_table("E11")  # Full GT visualization
```

## Data source

Official ICD-10 Chile data MINSAL/DEIS v2018:

- FIC Chile Center: <https://deis.minsal.cl/centrofic/>
- DEIS Repository: <https://deis.minsal.cl>

## More information

- Report issues: <https://github.com/RodoTasso/ciecl/issues>
- Repository: <https://github.com/RodoTasso/ciecl>

---
title: "Guia de Instalacion y Configuracion"
author: "Rodolfo Tasso Suazo"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Guia de Instalacion y Configuracion}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

## Instalacion Basica

### Desde CRAN (cuando este disponible)

```{r}
install.packages("ciecl")
```
### Desde GitHub (version desarrollo)

```{r}
# Opcion 1: pak (recomendado)
install.packages("pak")
pak::pak("RodoTasso/ciecl")

# Opcion 2: devtools
install.packages("devtools")
devtools::install_github("RodoTasso/ciecl")

# Opcion 3: remotes
install.packages("remotes")
remotes::install_github("RodoTasso/ciecl")
```

## Instalacion con Dependencias Opcionales

El paquete tiene dependencias minimas para funcionalidad core. Para habilitar
todas las funcionalidades:

```{r}
# Instalacion completa con todos los paquetes opcionales
pak::pak("RodoTasso/ciecl", dependencies = TRUE)
```

### Dependencias por Funcionalidad

| Funcionalidad | Paquete Requerido | Instalacion |
|---------------|-------------------|-------------|
| Comorbilidades Charlson/Elixhauser | `comorbidity` | `install.packages("comorbidity")` |
| Tablas interactivas GT | `gt` | `install.packages("gt")` |
| API CIE-11 OMS | `httr2` | `install.packages("httr2")` |
| Leer archivos Excel MINSAL | `readxl` | `install.packages("readxl")` |

## Requisitos del Sistema

### Windows

No requiere dependencias adicionales. La instalacion funciona directamente.

### macOS

Instalar Xcode Command Line Tools:

```bash
xcode-select --install
```

### Linux (Ubuntu/Debian)

```bash
sudo apt-get update
sudo apt-get install -y \
  r-base-dev \
  libcurl4-openssl-dev \
  libssl-dev \
  libxml2-dev
```

### Linux (Fedora/RHEL/CentOS)

```bash
sudo dnf install -y \
  R-devel \
  libcurl-devel \
  openssl-devel \
  libxml2-devel
```

## Configuracion de API CIE-11 (Opcional)
Para usar `cie11_search()` y acceder a la clasificacion internacional CIE-11
de la OMS, necesitas credenciales gratuitas.

### Paso 1: Obtener Credenciales

1. Visita <https://icd.who.int/icdapi>
2. Registrate con tu email (proceso gratuito)
3. Obtendras un `Client ID` y `Client Secret`

### Paso 2: Configurar Variables de Entorno

**Opcion A: Archivo .Renviron (recomendado para uso permanente)**

Crea o edita el archivo `~/.Renviron`:

```{r}
# Abrir archivo .Renviron para editar
usethis::edit_r_environ()
```

Agrega la linea:

```
ICD_API_KEY=tu_client_id:tu_client_secret
```

Reinicia R para que tome efecto.

**Opcion B: En cada sesion (temporal)**

```{r}
Sys.setenv(ICD_API_KEY = "tu_client_id:tu_client_secret")
```

### Paso 3: Verificar Configuracion

```{r}
# Verificar que la API key esta configurada
Sys.getenv("ICD_API_KEY")

# Probar busqueda CIE-11
library(ciecl)
cie11_search("diabetes")
```

## Cache SQLite

El paquete usa SQLite para almacenar y buscar codigos CIE-10 de forma eficiente.
La base de datos se crea automaticamente en:

```{r}
# Ver ubicacion del cache
tools::R_user_dir("ciecl", "data")
```

### Limpiar Cache

Si necesitas forzar una reconstruccion de la base de datos:

```{r}
library(ciecl)
cie10_clear_cache()
```

## Verificar Instalacion

```{r}
library(ciecl)

# Verificar que el paquete carga correctamente
packageVersion("ciecl")

# Verificar acceso al dataset
nrow(cie10_cl)  # Debe retornar 39873

# Probar busqueda basica
cie_lookup("E11.0")

# Probar busqueda fuzzy
cie_search("diabetes")
```

## Problemas Comunes

### Error: "package 'ciecl' is not available"

Asegurate de instalar desde GitHub, no desde CRAN (aun no esta disponible):

```{r}
pak::pak("RodoTasso/ciecl")
```

### Error de compilacion en Linux

Instala las dependencias del sistema mencionadas arriba.

### API CIE-11 no funciona

1. Verifica que `httr2` esta instalado: `install.packages("httr2")`
2. Verifica que la API key esta configurada: `Sys.getenv("ICD_API_KEY")`
3. Verifica conexion a internet

### Cache corrupto

Limpia el cache y reinicia R:

```{r}
ciecl::cie10_clear_cache()
```

## Soporte

- Reportar problemas: <https://github.com/RodoTasso/ciecl/issues>
- Documentacion: <https://github.com/RodoTasso/ciecl>

---
title: "Caso de uso: Egresos hospitalarios con ciecl"
author: "Rodolfo Tasso Suazo"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Caso de uso: Egresos hospitalarios con ciecl}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(ciecl)
library(dplyr)
```

Los registros de egresos hospitalarios en Chile (fuente DEIS) almacenan
diagnosticos en formato CIE-10 sin punto (`J189`, `O800`) y en algunos casos
con sufijo `X` (`I10X`, `N40X`) para indicar categorias sin subcategoria
adicional. Esta convencion de codificacion dificulta unir los datos con
catalogos externos que usan el formato normalizado con punto (`J18.9`, `O80.0`).

Esta vignette reproduce el flujo utilizado en investigacion con datos DEIS:
obtener descripciones CIE-10 para toda la base, unirlas con los registros
originales, y generar perfiles de diagnostico.

## Datos de ejemplo

Simulamos una base con la estructura tipica de los egresos hospitalarios DEIS.
Las columnas `DIAG1` y `ANO` imitan el formato original:

```{r datos}
set.seed(42)

egresos <- data.frame(
  ID_EGRESO = 1:200,
  ANO       = sample(2018:2022, 200, replace = TRUE),
  DIAG1     = sample(
    c(
      # Codigos frecuentes en egresos DEIS (sin punto, formato original)
      "J189", "O800", "Z380", "K359", "N390",
      "I10X", "J449", "E119", "O829", "J069",
      "K922", "N185", "I509", "C509", "A099",
      # Algunos con sufijo X (convencion DEIS)
      "N40X", "K800", "I259", "J180", "E149"
    ),
    size    = 200,
    replace = TRUE,
    prob    = c(
      0.12, 0.10, 0.09, 0.07, 0.06,
      0.08, 0.06, 0.07, 0.05, 0.04,
      0.04, 0.03, 0.05, 0.03, 0.03,
      0.03, 0.03, 0.04, 0.03, 0.05
    )
  ),
  stringsAsFactors = FALSE
)

cat("Registros totales:", nrow(egresos), "\n")
cat("Codigos unicos:", length(unique(egresos$DIAG1)), "\n")
cat("Periodo:", min(egresos$ANO), "-", max(egresos$ANO), "\n\n")
head(egresos)
```

## 1. Obtener descripciones CIE-10

Filtramos registros validos y usamos `cie_lookup()` con `normalizar = TRUE`
para procesar todos los codigos unicos en una sola llamada vectorizada.
El argumento `normalizar = TRUE` acepta los codigos tal como vienen del
sistema: sin punto, con sufijo `X`, en cualquier capitalizacion.

```{r lookup}
# Filtrar registros validos (sin NA en diagnostico)
datos_filtrados <- egresos %>%
  filter(!is.na(DIAG1))

# Codigos unicos a procesar
codigos_unicos <- unique(datos_filtrados$DIAG1)
cat("Codigos unicos a procesar:", length(codigos_unicos), "\n")

# Busqueda vectorizada con cie_lookup
inicio <- Sys.time()

resultado_ciecl <- cie_lookup(
  codigo             = codigos_unicos,
  normalizar         = TRUE,
  descripcion_completa = TRUE
)

tiempo <- round(as.numeric(difftime(Sys.time(), inicio, units = "secs")), 2)

cat("Codigos encontrados:", nrow(resultado_ciecl), "de", length(codigos_unicos), "\n")
cat("Tiempo de busqueda:", tiempo, "segundos\n")
cat("Tasa de exito:", round(nrow(resultado_ciecl) / length(codigos_unicos) * 100, 1), "%\n\n")

# Muestra de resultados
head(resultado_ciecl[, c("codigo", "descripcion")], 5)
```

## 2. Unir descripciones con la base de egresos

Los codigos en la base DEIS no tienen punto (`J189`), pero `cie_lookup()`
devuelve los codigos en formato normalizado (`J18.9`). Para el join es
necesario quitar el punto y ademas eliminar el sufijo `X` que algunos
codigos tienen en la base original:

```{r unir}
# Tabla de lookup: codigo sin punto -> descripcion completa
# cie_lookup() devuelve con punto (J18.9), la BBDD tiene sin punto (J189)
tabla_lookup <- resultado_ciecl %>%
  mutate(
    codigo_sin_punto = gsub("\\.", "", codigo),
    DIAG_COMPLETO    = paste0(codigo, " - ", descripcion)
  ) %>%
  select(codigo_sin_punto, DIAG_COMPLETO)

# Funcion para normalizar el codigo de la BBDD antes del join:
# 1. Quitar puntos si los hubiera
# 2. Quitar sufijo X final (I10X -> I10, N40X -> N40)
normalizar_codigo_bbdd <- function(codigo) {
  codigo <- gsub("\\.", "", codigo)
  codigo <- gsub("X$", "", codigo)
  return(codigo)
}

# Unir con los datos originales
datos_con_desc <- datos_filtrados %>%
  mutate(codigo_busqueda = normalizar_codigo_bbdd(DIAG1)) %>%
  left_join(tabla_lookup, by = c("codigo_busqueda" = "codigo_sin_punto")) %>%
  mutate(
    DIAG_COMPLETO = if_else(is.na(DIAG_COMPLETO), DIAG1, DIAG_COMPLETO)
  ) %>%
  select(-codigo_busqueda)

# Verificar resultado
n_con_desc <- sum(grepl(" - ", datos_con_desc$DIAG_COMPLETO), na.rm = TRUE)
pct_exito  <- round(n_con_desc / nrow(datos_con_desc) * 100, 1)

cat("Registros con descripcion CIE-10:", n_con_desc, "de", nrow(datos_con_desc), "\n")
cat("Tasa de exito:", pct_exito, "%\n\n")

# Ejemplos de diagnosticos procesados
datos_con_desc %>%
  select(DIAG1, DIAG_COMPLETO) %>%
  distinct() %>%
  head(10)
```

## 3. Perfil de diagnosticos

Con las descripciones incorporadas calculamos la distribucion de diagnosticos
y la proporcion que representa cada uno sobre el total de egresos:

```{r perfil}
perfil_diag <- datos_con_desc %>%
  filter(grepl(" - ", DIAG_COMPLETO)) %>%
  count(DIAG_COMPLETO, sort = TRUE) %>%
  mutate(
    pct       = round(n / sum(n) * 100, 1),
    pct_acum  = cumsum(pct)
  )

cat("Top 10 diagnosticos:\n\n")
head(perfil_diag, 10)
```

```{r cobertura}
top10_pct <- sum(head(perfil_diag, 10)$pct)
cat("Los 10 diagnosticos mas frecuentes representan el", top10_pct, "% del total de egresos\n")
```

## 4. Busqueda por texto

Cuando no se conoce el codigo exacto, `cie_search()` encuentra diagnosticos
por descripcion textual con tolerancia a errores tipograficos:

```{r busqueda_fuzzy}
# Terminos clinicos comunes en egresos (con y sin typos)
terminos <- c("neumonia", "diabetis", "hipertension")

for (termino in terminos) {
  cat("Buscando:", termino, "\n")
  res <- cie_search(termino, threshold = 0.70, max_results = 3)
  if (nrow(res) > 0) {
    print(res[, c("codigo", "descripcion")])
  }
  cat("\n")
}
```

## 5. Validacion de codigos

Antes de procesar una base conviene identificar codigos que no corresponden
a ningun diagnostico valido en el catalogo CIE-10:

```{r validacion}
validos <- cie_validate_vector(codigos_unicos)

cat("Codigos validos:  ", sum(validos), "\n")
cat("Codigos invalidos:", sum(!validos), "\n")

if (any(!validos)) {
  cat("\nCodigos no reconocidos:\n")
  print(codigos_unicos[!validos])
}
```

## Fuente de datos

- Catalogo CIE-10 Chile: Centro FIC DEIS <https://deis.minsal.cl/centrofic/>
- Repositorio: <https://github.com/RodoTasso/ciecl>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-search.R
\name{cie_siglas}
\alias{cie_siglas}
\title{Listar siglas medicas soportadas}
\usage{
cie_siglas(categoria = NULL)
}
\arguments{
\item{categoria}{Character opcional, filtrar por categoria. Valores validos:
"cardiovascular", "respiratoria", "metabolica", "gastrointestinal",
"infecciosa", "oncologica", "reumatologica", "neurologica",
"psiquiatrica", "traumatologica", "pediatrica", "gineco_obstetrica".
Si es NULL (default), retorna todas las siglas.}
}
\value{
tibble con columnas: sigla, termino_busqueda, categoria
}
\description{
Muestra todas las siglas medicas que pueden usarse en cie_search().
}
\examples{
# Ver todas las siglas
cie_siglas()

# Filtrar por categoria
cie_siglas("cardiovascular")
cie_siglas("oncologica")

# Buscar una sigla especifica
cie_siglas() |> dplyr::filter(sigla == "iam")
}
\seealso{
\code{\link{cie_search}}, \code{\link{cie_lookup}}

Other busqueda: 
\code{\link{cie_guia_busqueda}()},
\code{\link{cie_lookup}()},
\code{\link{cie_search}()}
}
\concept{busqueda}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-sql.R
\name{cie10_clear_cache}
\alias{cie10_clear_cache}
\title{Limpiar cache SQLite (forzar rebuild)}
\usage{
cie10_clear_cache()
}
\value{
No return value, called for side effects (deletes SQLite cache).
}
\description{
Limpiar cache SQLite (forzar rebuild)
}
\examples{
# Ver ubicacion del cache
tools::R_user_dir("ciecl", "data")

\donttest{
cie10_clear_cache()  # Elimina cie10.db local
}
}
\seealso{
\code{\link{cie10_sql}}, \code{\link{cie10_disconnect}}

Other sql: 
\code{\link{cie10_disconnect}()},
\code{\link{cie10_sql}()}
}
\concept{sql}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-search.R
\name{cie_lookup}
\alias{cie_lookup}
\title{Busqueda exacta por codigo CIE-10}
\usage{
cie_lookup(
  codigo,
  expandir = FALSE,
  normalizar = TRUE,
  descripcion_completa = FALSE,
  extract = FALSE,
  check_siglas = FALSE
)
}
\arguments{
\item{codigo}{Character vector de codigos
(ej. "E11", "E11.0", c("E11.0", "Z00"))
o rango (ej. "E10-E14"). Acepta vectores.
Soporta formatos: con punto (E11.0),
sin punto (E110), o solo categoria (E11).}

\item{expandir}{Logical, expandir jerarquia completa (default FALSE)}

\item{normalizar}{Logical, normalizar formato de codigos
automaticamente (default TRUE)}

\item{descripcion_completa}{Logical, agregar columna descripcion_completa
con formato "CODIGO - DESCRIPCION" (default FALSE)}

\item{extract}{Logical, extraer codigo CIE-10 de texto con
prefijos/sufijos (default FALSE).
IMPORTANTE: Solo usar con codigo ESCALAR (longitud 1).
Ejemplo: "CIE:E11.0" -> "E11.0", "E11.0-confirmado" -> "E11.0".
Para vectores multiples usar extract=FALSE (default).}

\item{check_siglas}{Logical, buscar siglas medicas comunes (default FALSE).
Ejemplo: "IAM" -> I21.0 (Infarto agudo miocardio)}
}
\value{
tibble con codigo(s) matcheado(s)
}
\description{
Busqueda exacta por codigo CIE-10
}
\examples{
# Busqueda directa por codigo
cie_lookup("E11.0")

\donttest{
cie_lookup("E110")        # Sin punto
cie_lookup("E11")         # Solo categoria
cie_lookup("E11", expandir = TRUE)  # Todos E11.x
# Vectorizado - multiples codigos y formatos
cie_lookup(c("E11.0", "Z00", "I10"))
# Con descripcion completa
cie_lookup("E110", descripcion_completa = TRUE)
# Extraer codigo de texto con ruido (solo codigo escalar)
cie_lookup("CIE:E11.0", extract = TRUE)
cie_lookup("E11.0-confirmado", extract = TRUE)
# Buscar por siglas medicas
cie_lookup("IAM", check_siglas = TRUE)
cie_lookup("DM2", check_siglas = TRUE)
}
}
\seealso{
\code{\link{cie_search}},
\code{\link{cie_normalizar}}, \code{\link{cie_expand}}

Other busqueda: 
\code{\link{cie_guia_busqueda}()},
\code{\link{cie_search}()},
\code{\link{cie_siglas}()}
}
\concept{busqueda}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-search.R
\name{cie_search}
\alias{cie_search}
\title{Busqueda difusa (fuzzy) de terminos medicos CIE-10}
\usage{
cie_search(
  texto,
  threshold = 0.7,
  max_results = 50,
  campo = c("descripcion", "inclusion"),
  solo_fuzzy = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{texto}{String termino medico en espanol o sigla
(ej. "diabetes", "IAM", "TBC")}

\item{threshold}{Numeric entre 0 y 1, umbral similitud
Jaro-Winkler (default 0.70)}

\item{max_results}{Integer, maximo resultados a retornar (default 50)}

\item{campo}{Character, campo busqueda ("descripcion" o "inclusion")}

\item{solo_fuzzy}{Logical, usar solo busqueda fuzzy
sin busqueda exacta (default FALSE)}

\item{verbose}{Logical, mostrar mensajes informativos
(default TRUE). Usar FALSE en scripts.}
}
\value{
tibble ordenado por score descendente (1.0 = coincidencia exacta).
Si el texto corresponde a una sigla medica, se expande
automaticamente antes de buscar.
}
\description{
Busca en descripciones CIE-10 usando multiples estrategias:
\enumerate{
\item Expansion de siglas medicas (IAM, TBC, DM, etc.)
\item Busqueda exacta por subcadena (mas rapida)
\item Busqueda fuzzy con Jaro-Winkler (tolera typos)
}
}
\details{
La busqueda es tolerante a tildes: "neumonia" encuentra "neumonia".
Soporta siglas medicas comunes: "IAM" busca "infarto agudo miocardio".
}
\examples{
# Busqueda basica
cie_search("diabetes")

\donttest{
cie_search("neumonia")

# Busqueda por siglas medicas
cie_search("IAM")
cie_search("DM2")

# Tolerante a tildes y typos
cie_search("diabetis")

# Buscar en inclusiones
cie_search("bacteriana", campo = "inclusion")
}
}
\seealso{
\code{\link{cie_lookup}},
\code{\link{cie_siglas}}, \code{\link{cie10_sql}}

Other busqueda: 
\code{\link{cie_guia_busqueda}()},
\code{\link{cie_lookup}()},
\code{\link{cie_siglas}()}
}
\concept{busqueda}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-sql.R
\name{cie10_disconnect}
\alias{cie10_disconnect}
\title{Cerrar conexion pooled SQLite}
\usage{
cie10_disconnect()
}
\value{
No return value, called for side effects.
}
\description{
Cierra la conexion reutilizable al archivo SQLite.
Util para liberar el lock del archivo .db.
}
\examples{
# Verificar si hay conexion activa
is.null(ciecl:::.ciecl_env$con)

\donttest{
cie10_disconnect()
}
}
\seealso{
\code{\link{cie10_sql}}, \code{\link{cie10_clear_cache}}

Other sql: 
\code{\link{cie10_clear_cache}()},
\code{\link{cie10_sql}()}
}
\concept{sql}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-comorbid.R
\name{cie_map_comorbid}
\alias{cie_map_comorbid}
\title{Mapeo manual grupos comorbilidad Chile-especifico}
\usage{
cie_map_comorbid(codigos)
}
\arguments{
\item{codigos}{Character vector codigos CIE-10}
}
\value{
tibble con codigo + categoria_comorbilidad
}
\description{
Agrupa codigos CIE-10 chilenos en categorias comorbilidad MINSAL.
Basado en Decreto 1301/2016 MINSAL + icd::icd10_map_charlson.
}
\examples{
cie_map_comorbid(c("E11.0", "I50.9", "C50.9"))
}
\seealso{
\code{\link{cie_comorbid}}, \code{\link{cie_normalizar}}

Other comorbilidades: 
\code{\link{cie_comorbid}()}
}
\concept{comorbilidades}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-utils.R
\name{cie_validate_vector}
\alias{cie_validate_vector}
\title{Validar vector de codigos CIE-10 formato}
\usage{
cie_validate_vector(codigos, strict = FALSE)
}
\arguments{
\item{codigos}{Character vector codigos (ej. c("E11.0", "Z00.0"))}

\item{strict}{Logical, validar existencia en DB (default FALSE)}
}
\value{
Logical vector de la misma longitud que \code{codigos}. TRUE si el
codigo tiene formato CIE-10 valido (y existe en DB si \code{strict = TRUE}).
}
\description{
Validar vector de codigos CIE-10 formato
}
\examples{
cie_validate_vector(c("E11.0", "INVALIDO", "Z00"))
}
\seealso{
\code{\link{cie_normalizar}}, \code{\link{cie_expand}}

Other validacion: 
\code{\link{cie_expand}()},
\code{\link{cie_normalizar}()}
}
\concept{validacion}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-utils.R
\name{cie_expand}
\alias{cie_expand}
\title{Expandir codigo jerarquico (ej. E11 -> E11.0-E11.9)}
\usage{
cie_expand(codigo)
}
\arguments{
\item{codigo}{String codigo padre (ej. "E11")}
}
\value{
Character vector con todos los codigos hijos del codigo padre.
Vector vacio si el codigo no existe en la base de datos.
}
\description{
Expandir codigo jerarquico (ej. E11 -> E11.0-E11.9)
}
\examples{
cie_expand("E11")
}
\seealso{
\code{\link{cie_normalizar}}, \code{\link{cie_lookup}}

Other validacion: 
\code{\link{cie_normalizar}()},
\code{\link{cie_validate_vector}()}
}
\concept{validacion}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-search.R
\name{cie_guia_busqueda}
\alias{cie_guia_busqueda}
\title{Guia de funciones de busqueda CIE-10}
\usage{
cie_guia_busqueda()
}
\value{
tibble con guia comparativa de funciones de busqueda
}
\description{
Muestra tabla comparativa de cuando usar cada funcion de busqueda.
}
\examples{
cie_guia_busqueda()
}
\seealso{
\code{\link{cie_search}},
\code{\link{cie_lookup}}, \code{\link{cie_siglas}}

Other busqueda: 
\code{\link{cie_lookup}()},
\code{\link{cie_search}()},
\code{\link{cie_siglas}()}
}
\concept{busqueda}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-sql.R
\name{cie10_sql}
\alias{cie10_sql}
\title{Ejecutar consultas SQL sobre CIE-10 Chile}
\usage{
cie10_sql(query, close = TRUE)
}
\arguments{
\item{query}{String SQL valido SQLite (SELECT/WHERE/JOIN)}

\item{close}{Deprecated. Ignorado — la conexion es pooled y se
gestiona automaticamente. Sera eliminado en una version futura.}
}
\value{
tibble resultado query
}
\description{
Ejecutar consultas SQL sobre CIE-10 Chile
}
\examples{
# Buscar diabetes
cie10_sql("SELECT codigo, descripcion FROM cie10 WHERE codigo LIKE 'E11\%'")

\donttest{
# Contar por capitulo
cie10_sql("SELECT capitulo, COUNT(*) n FROM cie10 GROUP BY capitulo")
}
}
\seealso{
\code{\link{cie10_clear_cache}}, \code{\link{cie10_disconnect}},
\code{\link{cie_search}}

Other sql: 
\code{\link{cie10_clear_cache}()},
\code{\link{cie10_disconnect}()}
}
\concept{sql}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-api.R
\name{cie11_search}
\alias{cie11_search}
\title{Buscar codigos CIE-11 via API OMS}
\usage{
cie11_search(
  texto,
  api_key = NULL,
  lang = "es",
  max_results = 10,
  release = "2024-01"
)
}
\arguments{
\item{texto}{String termino busqueda espanol/ingles}

\item{api_key}{String opcional, Client ID + Secret OMS separados ":"
Obtener en: https://icd.who.int/icdapi}

\item{lang}{Character, idioma respuesta ("es" o "en")}

\item{max_results}{Integer, maximo resultados (default 10)}

\item{release}{Character, version de release CIE-11 a consultar
(default "2024-01"). Ver releases disponibles en la API OMS.}
}
\value{
tibble con codigos CIE-11 + titulos o vacio si error
}
\description{
Buscar codigos CIE-11 via API OMS
}
\examples{
# Ver parametros disponibles
args(cie11_search)

\donttest{
# Requiere credenciales OMS gratuitas (https://icd.who.int/icdapi)
Sys.setenv(ICD_API_KEY = "client_id:client_secret")
cie11_search("depresion mayor")
}
}
\seealso{
\code{\link{cie_search}}, \code{\link{cie_lookup}}
}
\concept{api}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ciecl-package.R
\docType{package}
\name{ciecl-package}
\alias{ciecl}
\alias{ciecl-package}
\title{ciecl: International Classification of Diseases 'ICD-10'/'ICD-11' for Chile}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Tools for working with the International Classification of Diseases ('ICD-10' Chile official 'MINSAL'/'DEIS' v2018). Includes optimized 'SQL' search with 'SQLite', fuzzy matching of medical terms ('Jaro-Winkler'), Charlson and Elixhauser comorbidity calculation, 'WHO' 'ICD-11' 'API' integration, and hierarchical code validation. Data from Centro FIC Chile 'DEIS' \url{https://deis.minsal.cl/centrofic/}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://rodotasso.github.io/ciecl/}
  \item \url{https://github.com/RodoTasso/ciecl}
  \item Report bugs at \url{https://github.com/RodoTasso/ciecl/issues}
}

}
\author{
\strong{Maintainer}: Rodolfo Tasso Suazo \email{rtasso@uchile.cl} (\href{https://orcid.org/0000-0002-1264-4933}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-data.R
\docType{data}
\name{cie10_cl}
\alias{cie10_cl}
\title{Dataset CIE-10 Chile oficial MINSAL/DEIS v2018}
\format{
tibble con 39,877 filas (categorias y subcategorias):
\describe{
\item{codigo}{Codigo CIE-10 (ej. "E11.0")}
\item{descripcion}{Diagnostico en espanol chileno}
\item{categoria}{Categoria jerarquica}
\item{seccion}{Seccion dentro del capitulo}
\item{capitulo_nombre}{Nombre descriptivo del capitulo}
\item{inclusion}{Terminos incluidos}
\item{exclusion}{Terminos excluidos}
\item{capitulo}{Capitulo CIE-10 (A-Z)}
\item{es_daga}{Logical, codigo daga (+)}
\item{es_cruz}{Logical, codigo asterisco (*)}
}
}
\source{
\url{https://deis.minsal.cl/centrofic/}
}
\usage{
cie10_cl
}
\description{
Dataset CIE-10 Chile oficial MINSAL/DEIS v2018
}
\examples{
data(cie10_cl)
head(cie10_cl)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-table.R
\name{cie_table}
\alias{cie_table}
\title{Generar tabla HTML interactiva GT de codigo CIE-10}
\usage{
cie_table(codigo)
}
\arguments{
\item{codigo}{String codigo (ej. "E11" muestra jerarquia)}
}
\value{
Objeto de clase \code{gt_tbl} (tabla HTML interactiva)
}
\description{
Generar tabla HTML interactiva GT de codigo CIE-10
}
\examples{
# Requiere paquete gt
requireNamespace("gt", quietly = TRUE)

\donttest{
cie_table("E11")  # Diabetes tipo 2 completo
}
}
\seealso{
\code{\link{cie_search}}, \code{\link{cie_lookup}}
}
\concept{visualizacion}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-utils.R
\name{cie_normalizar}
\alias{cie_normalizar}
\title{Normalizar codigos CIE-10 a formato con punto}
\usage{
cie_normalizar(codigos, buscar_db = TRUE)
}
\arguments{
\item{codigos}{Character vector de codigos en cualquier formato}

\item{buscar_db}{Logical, buscar codigo en base de datos
si no se encuentra exacto (default TRUE)}
}
\value{
Character vector con codigos normalizados al formato con punto
}
\description{
Convierte codigos CIE-10 de diferentes formatos al
formato estandar (con punto).
Maneja multiples variaciones de entrada comunes en datos clinicos.
}
\details{
La normalizacion incluye:
\itemize{
\item Conversion a mayusculas
\item Eliminacion de espacios (inicio, fin e internos)
\item Eliminacion de simbolos daga y asterisco (codificacion dual)
\item Conversion de guiones a puntos (I10-0 -> I10.0)
\item Eliminacion de puntos iniciales (.I10 -> I10)
\item Correccion de puntos multiples (E..11 -> E.11)
\item Eliminacion de sufijo X en codigos cortos (I10X -> I10)
\item Preservacion de X en codigos largos (placeholder 7o caracter)
\item Agregado de punto en posicion correcta (E110 -> E11.0)
}

El sistema de daga/asterisco indica codificacion dual donde la daga
marca la enfermedad subyacente y el asterisco la manifestacion.
Ambos simbolos se eliminan para normalizacion.
}
\examples{
cie_normalizar("E110")     # Retorna "E11.0"
cie_normalizar("E11")      # Retorna "E11" (categoria)
cie_normalizar("I10X")     # Retorna "I10" (elimina X)
cie_normalizar("E 11 0")   # Retorna "E11.0" (espacios internos)
cie_normalizar("I10-0")    # Retorna "I10.0" (guion a punto)
cie_normalizar("A17.0\u2020") # Retorna "A17.0" (elimina daga)
cie_normalizar("G01*")     # Retorna "G01" (elimina asterisco)
cie_normalizar(c("E110", "I10X", "Z00"))  # Vectorizado
}
\seealso{
\code{\link{cie_validate_vector}},
\code{\link{cie_expand}}, \code{\link{cie_lookup}}

Other validacion: 
\code{\link{cie_expand}()},
\code{\link{cie_validate_vector}()}
}
\concept{validacion}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cie-comorbid.R
\name{cie_comorbid}
\alias{cie_comorbid}
\title{Calcular comorbilidades Charlson/Elixhauser para Chile}
\usage{
cie_comorbid(data, id, code, map = c("charlson", "elixhauser"), assign0 = TRUE)
}
\arguments{
\item{data}{data.frame con columnas id paciente + codigos CIE-10}

\item{id}{String nombre columna identificador paciente}

\item{code}{String nombre columna con codigos CIE-10 (uno por fila)}

\item{map}{Character, esquema comorbilidad ("charlson" o "elixhauser")}

\item{assign0}{Logical, asignar 0 si sin comorbilidad (default TRUE)}
}
\value{
data.frame ancho con scores comorbilidad por paciente
}
\description{
Calcular comorbilidades Charlson/Elixhauser para Chile
}
\examples{
# Ver documentacion de parametros
args(cie_comorbid)

\donttest{
df <- data.frame(
  id_pac = c(1, 1, 2, 2),
  diag = c("E11.0", "I21.0", "C50.9", "E10.9")
)
cie_comorbid(df, id = "id_pac", code = "diag", map = "charlson")
}
}
\seealso{
\code{\link{cie_map_comorbid}}, \code{\link{cie_normalizar}}

Other comorbilidades: 
\code{\link{cie_map_comorbid}()}
}
\concept{comorbilidades}
